@@ -590,6 +590,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 	db->flags.no_serial = TRUE;
 	db->flags.no_blob = TRUE;
 	db->flags.no_nest = TRUE;
+	db->flags.no_collation = TRUE;
 
 	db->db_name_char = ".";
 	
@@ -620,6 +621,21 @@ static void close_database(DB_DATABASE *db)
 }
 
 
+/*****************************************************************************
+
+	get_collations()
+
+	Return the available collations as a Gambas string array.
+
+*****************************************************************************/
+
+static GB_ARRAY get_collations(DB_DATABASE *db)
+{
+	return NULL;
+}
+
+
+
 /*****************************************************************************
 
 	format_value()
@@ -1624,7 +1640,7 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 		return TRUE;
 	}
 
-	result_set* r = (result_set*) res->getExecRes();
+	result_set *r = (result_set*) res->getExecRes();
 	if ((n = r->records.size()) == 0)
 	{
 		GB.Error("Unable to find field &1.&2", table, field);
@@ -1669,6 +1685,8 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 		}
 	}
 
+	info->collation = NULL;
+
 	res->close();
 	return FALSE;
 }
