@@ -44,12 +44,13 @@ extern "C"
 {
 	GB_INTERFACE GB EXPORT;
 	DB_INTERFACE DB EXPORT;
-}																//end extern "C"
+}
 
 static char _buffer[32];
 
 static int _print_query = FALSE;
 static bool _need_field_type = FALSE;
+static char *_table_schema = NULL;
 
 /*****************************************************************************
 
@@ -299,7 +300,6 @@ static int do_query(DB_DATABASE *db, const char *error, Dataset **pres, const ch
 	int err;
 	int retry = 0;
 	int max_retry;
-	bool select;
 	bool success;
 
 	if (nsubst)
@@ -330,18 +330,13 @@ static int do_query(DB_DATABASE *db, const char *error, Dataset **pres, const ch
 	else
 		max_retry = 0;
 	
-	select = (strncasecmp("select ", query, 7) == 0);
-	
 	for(;;)
 	{
 		err = 0;
 
 		res->setNeedFieldType(_need_field_type);
 
-		if (select)
-			success = res->query(query);
-		else
-			success = res->exec(query);
+		success = res->query(query);
 		
 		if (success)
 		{
@@ -473,7 +468,7 @@ static char *FindDatabase(const char *name, const char *hostName)
 
 #if 0
 	/* Now check for database in current working directory */
-	if (getcwd(cwd, MAX_PATH) == NULL)
+	if (getcwd(cwd, PATH_MAX) == NULL)
 	{
 		GB.Error("Unable to get databases: &1", "Can't find current directory");
 		return NULL;
@@ -501,7 +496,7 @@ static char *GetDatabaseHome()
 	char *env = NULL;
 	char *dbhome = NULL;
 
-	GB.Alloc(POINTER(&dbhome), MAX_PATH);
+	GB.Alloc(POINTER(&dbhome), PATH_MAX);
 
 	/* Check for Environment variable */
 
@@ -511,7 +506,7 @@ static char *GetDatabaseHome()
 	if (env == NULL)
 	{
 		/*
-		   if (getcwd(dbhome, MAX_PATH) == NULL){
+		   if (getcwd(dbhome, PATH_MAX) == NULL){
 		   GB.Error("Unable to get databases: &1", "Can't find current directory");
 		   GB.Free((void **)&dbhome);
 		   return NULL;
@@ -537,12 +532,12 @@ static int WalkDirectory(const char *dir, char ***databases)
 	DIR *dp;
 	struct dirent *entry;
 	struct stat statbuf;
-	char cwd[MAX_PATH];
+	char cwd[PATH_MAX];
 
 	if ((dp = opendir(dir)) == NULL)
 		return -1;
 
-	if (getcwd(cwd, MAX_PATH) == NULL)
+	if (getcwd(cwd, PATH_MAX) == NULL)
 	{
 		fprintf(stderr, "gb.db.sqlite3: warning: getcwd: %s\n", strerror(errno));
 		return -1;
@@ -587,6 +582,21 @@ static int db_version()
 	return dbversion;
 }
 
+/* Get the schema of a table */
+static char *get_table_schema(DB_DATABASE *db, const char *table)
+{
+	char *schema = NULL;
+	Dataset *res;
+
+	if (!do_query(db, "Unable to get table schema: &1", &res, "select sql from sqlite_master where type = 'table' and tbl_name = '&1'", 1, table))
+	{
+		schema = GB.NewZeroString((char *)res->fv(0).get_asString().data());
+		res->close();
+	}
+
+	return schema;
+}
+
 /*****************************************************************************
 
   get_quote()
@@ -650,7 +660,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE * db)
 	}
 	else
 	{
-		GB.Error("Unable to locate database: &1", name);
+		GB.Error("Unable to locate database `&1` in `&2`", name, desc->host);
 		GB.FreeString(&name);
 		delete conn;
 		return TRUE;
@@ -672,26 +682,45 @@ static int open_database(DB_DESC *desc, DB_DATABASE * db)
 	if (conn->connect() != DB_CONNECTION_OK)
 	{
 		GB.Error("Cannot open database: &1", conn->getErrorMsg());
-		conn->disconnect();
-		delete conn;
-		return TRUE;
+		goto CANNOT_OPEN;
 	}
 
+	db->handle = conn;
+	/* set dbversion */
+	db->version = db_version();
+
+	if (do_query(db, "Unable to initialize connection: &1", NULL, "PRAGMA empty_result_callbacks = ON", 0))
+		goto CANNOT_OPEN;
+
+	if (db->version < 30803)
+	{
+		/* NG 29/12/2005 - 3.2.1 introduced a problem with columns names
+		* which is resolved by setting short columns off first */
+
+		if (do_query(db, "Unable to initialize connection: &1", NULL, "PRAGMA short_column_names = OFF", 0))
+			goto CANNOT_OPEN;
+	}
+
+	if (do_query(db, "Unable to initialize connection: &1", NULL, "PRAGMA full_column_names = ON", 0))
+		goto CANNOT_OPEN;
+
 	/* Character set cannot be set for sqlite. A sqlite re-compile
 	 * is required.                                             */
 	db->charset = GB.NewZeroString("UTF-8");
 
-	/* set dbversion */
-	db->version = db_version();
-
 	/* flags */
 	db->flags.no_table_type = TRUE;
 	db->flags.no_nest = TRUE;
 
 	db->db_name_char = ".";
 
-	db->handle = conn;
 	return FALSE;
+
+CANNOT_OPEN:
+
+	conn->disconnect();
+	delete conn;
+	return TRUE;
 }
 
 
@@ -717,6 +746,28 @@ static void close_database(DB_DATABASE * db)
 }
 
 
+/*****************************************************************************
+
+	get_collations()
+
+	Return the available collations as a Gambas string array.
+
+*****************************************************************************/
+
+static GB_ARRAY get_collations(DB_DATABASE *db)
+{
+	static const char *const collations[] = { "BINARY", "NOCASE", "RTRIM" };
+	GB_ARRAY array;
+	int i;
+
+	GB.Array.New(&array, GB_T_STRING, 3);
+	for (i = 0; i < 3; i++)
+		*((char **)GB.Array.Get(array, i)) = GB.NewZeroString(collations[i]);
+
+	return array;
+}
+
+
 /*****************************************************************************
 
   format_value()
@@ -896,7 +947,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 	fType type;
 
 	if (!next)
-		res->seek(pos);							/* move to record */
+		res->seek(pos);
 	else
 		res->next();
 
@@ -1139,7 +1190,7 @@ static int table_init(DB_DATABASE * db, const char *table, DB_INFO * info)
 	if (do_query(db, "Unable to get table fields: &1", &res, qfield, 1, table))
 		return TRUE;
 
-	result_set *r = (result_set *) res->getExecRes();
+	result_set *r = (result_set *) res->getResult();
 
 	info->nfield = n = r->records.size();
 	if (n == 0)
@@ -1211,35 +1262,38 @@ static int table_index(DB_DATABASE * db, const char *table, DB_INFO * info)
 	if (do_query(db, "Unable to get primary index: &1", &res, qindex1, 1, table))
 		return TRUE;
 
-	result_set *r = (result_set *) res->getExecRes();
+	result_set *r = (result_set *) res->getResult();
 	n = r->records.size();
 
-	for (int i = 0; i < n; i++)
+	for (i = 0; i < n; i++)
 	{
 		if (strstr(r->records[i][1].get_asString().data(), "autoindex") != NULL)
 		{
-			sql = GB.NewZeroString(r->records[i][1].get_asString().data());
-			res->close();
-
-			if (do_query
-					(db, "Unable to get information on primary index: &1", &res,
-					 qindex2, 1, sql))
+			if (strncmp("1", r->records[i][2].get_asString().data(), 1) == 0)
 			{
+				sql = GB.NewZeroString(r->records[i][1].get_asString().data());
 				res->close();
+
+				if (do_query
+						(db, "Unable to get information on primary index: &1", &res,
+						qindex2, 1, sql))
+				{
+					res->close();
+					GB.FreeString(&sql);
+					return TRUE;
+				}
 				GB.FreeString(&sql);
-				return TRUE;
-			}
-			GB.FreeString(&sql);
 
-			r = (result_set *) res->getExecRes();
-			info->nindex = r->records.size();
-			GB.Alloc(POINTER(&info->index), sizeof(int) * info->nindex);
+				r = (result_set *) res->getResult();
+				info->nindex = r->records.size();
+				GB.Alloc(POINTER(&info->index), sizeof(int) * info->nindex);
 
-			for (i = 0; i < info->nindex; i++)
-			{
-				info->index[i] = r->records[i][1].get_asInteger();
+				for (i = 0; i < info->nindex; i++)
+				{
+					info->index[i] = r->records[i][1].get_asInteger();
+				}
+				break;
 			}
-			break;
 		}
 	}
 
@@ -1255,7 +1309,7 @@ static int table_index(DB_DATABASE * db, const char *table, DB_INFO * info)
 				 "PRAGMA table_info('&1')", 1, table))
 			return TRUE;
 
-		r = (result_set *) res->getExecRes();
+		r = (result_set *) res->getResult();
 
 		info->nindex = 1;
 		GB.Alloc(POINTER(&info->index), sizeof(int));
@@ -1405,7 +1459,8 @@ static int table_list(DB_DATABASE * db, char ***tables)
 
 *****************************************************************************/
 
-static int table_primary_key(DB_DATABASE * db, const char *table, char ***primary)
+#if 0
+static int old_table_primary_key(DB_DATABASE * db, const char *table, char ***primary)
 {
 	const char *qindex1 = "PRAGMA index_list('&1')";
 	const char *qindex2 = "PRAGMA index_info('&1')";
@@ -1421,37 +1476,40 @@ static int table_primary_key(DB_DATABASE * db, const char *table, char ***primar
 
 	GB.NewArray(primary, sizeof(char *), 0);
 
-	r = (result_set *) res->getExecRes();
+	r = (result_set *) res->getResult();
 	n = r->records.size();
+
 	for (i = 0; i < n; i++)
 	{
 		if (strstr(r->records[i][1].get_asString().data(), "autoindex"))
 		{
-			sql = GB.NewZeroString(r->records[i][1].get_asString().data());
-			res->close();
-
-			if (do_query
-					(db, "Unable to get primary key: &1", &res, qindex2, 1, sql))
+			if (strncmp("1", r->records[i][2].get_asString().data(), 1) == 0)
 			{
+				sql = GB.NewZeroString(r->records[i][1].get_asString().data());
 				res->close();
+
+				if (do_query(db, "Unable to get primary key: &1", &res, qindex2, 1, sql))
+				{
+					res->close();
+					GB.FreeString(&sql);
+					return TRUE;
+				}
 				GB.FreeString(&sql);
-				return TRUE;
-			}
-			GB.FreeString(&sql);
 
-			r = (result_set *) res->getExecRes();
-			if ((n = r->records.size()) < 1)
-			{
-				// No information returned for key
-				res->close();
-				return TRUE;
-			}
+				r = (result_set *) res->getResult();
+				if ((n = r->records.size()) < 1)
+				{
+					// No information returned for key
+					res->close();
+					return TRUE;
+				}
 
-			for (i = 0; i < n; i++)
-			{
-				*(char **)GB.Add(primary) = GB.NewZeroString(r->records[i][2].get_asString().data());
+				for (i = 0; i < n; i++)
+				{
+					*(char **)GB.Add(primary) = GB.NewZeroString(r->records[i][2].get_asString().data());
+				}
+				break;
 			}
-			break;
 		}
 	}
 
@@ -1467,11 +1525,11 @@ static int table_primary_key(DB_DATABASE * db, const char *table, char ***primar
 				 "PRAGMA table_info('&1')", 1, table))
 			return TRUE;
 
-		r = (result_set *) res->getExecRes();
+		r = (result_set *) res->getResult();
 
 		for (i = 0; i < (int) r->records.size(); i++)
 		{
-			if (strcasecmp(r->records[i][2].get_asString().data(), "INTEGER") == 0)
+			if (strcasecmp(r->records[i][5].get_asString().data(), "0") != 0)
 			{
 				*(char **)GB.Add(primary) = GB.NewZeroString(r->records[i][1].get_asString().data());
 				break;
@@ -1481,6 +1539,39 @@ static int table_primary_key(DB_DATABASE * db, const char *table, char ***primar
 
 	return FALSE;
 }
+#endif
+
+static int table_primary_key(DB_DATABASE * db, const char *table, char ***primary)
+{
+	Dataset *res;
+	int i, j, n;
+	result_set *r;
+
+	if (do_query(db, "Unable to get primary key: &1", &res, "PRAGMA table_info('&1')", 1, table))
+		return TRUE;
+
+	r = (result_set *) res->getResult();
+
+	n = 0;
+	for (i = 0; i < (int) r->records.size(); i++)
+	{
+		j = r->records[i][5].get_asInteger();
+		if (j > n) n = j;
+	}
+
+	GB.NewArray(primary, sizeof(char *), n);
+
+	for (i = 0; i < (int) r->records.size(); i++)
+	{
+		j = r->records[i][5].get_asInteger();
+		if (j > 0)
+			(*primary)[j - 1] = GB.NewZeroString(r->records[i][1].get_asString().data());
+	}
+
+	res->close();
+
+	return FALSE;
+}
 
 /*****************************************************************************
 
@@ -1635,6 +1726,12 @@ static int table_create(DB_DATABASE * db, const char *table, DB_FIELD * fields,
 			DB.Query.Add(" ");
 			DB.Query.Add(type);
 
+			if (fp->collation && *fp->collation)
+			{
+				DB.Query.Add(" COLLATE ");
+				DB.Query.Add(fp->collation);
+			}
+
 			if (fp->def.type != GB_T_NULL)
 			{
 				DB.Query.Add(" NOT NULL DEFAULT ");
@@ -1696,7 +1793,7 @@ static int field_exist(DB_DATABASE * db, const char *table, const char *field)
 		return FALSE;
 	}
 
-	result_set *r = (result_set *) res->getExecRes();
+	result_set *r = (result_set *) res->getResult();
 
 	n = r->records.size();
 
@@ -1739,7 +1836,7 @@ static int field_list(DB_DATABASE * db, const char *table, char ***fields)
 		return -1;
 	}
 
-	result_set *r = (result_set *) res->getExecRes();
+	result_set *r = (result_set *) res->getResult();
 
 	n = r->records.size();
 
@@ -1778,7 +1875,7 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 {
 	const char *query = "PRAGMA table_info('&1')";
 
-	Dataset *res, *res_autoinc;
+	Dataset *res;
 	GB_VARIANT def;
 	char *val;
 	char *_fieldName = NULL;
@@ -1789,17 +1886,15 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	//sqlite3 *db_handle = ((SqliteDatabase *)db->handle)->getHandle();
 	fType type;
 	result_set *r;
-	char *schema;
-	char *p, *p2;
-	char *search;
 	bool autoinc;
+	char *schema;
 
 	if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
 	{
 		return TRUE;
 	}
 
-	r = (result_set *) res->getExecRes();
+	r = (result_set *) res->getResult();
 
 	if ((n = r->records.size()) == 0)
 	{
@@ -1842,39 +1937,81 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	}*/
 	
 	autoinc = false;
-	
-	if (strstr(_fieldType, "INTEGER"))
+	info->collation = NULL;
+
+	if (_table_schema)
+		schema = _table_schema;
+	else
+		schema = get_table_schema(db, table);
+
+	if (schema)
 	{
-		if (!do_query(db, "Unable to get table schema: &1", &res_autoinc, "select sql from sqlite_master where type = 'table' and tbl_name = '&1'", 1, table))
+		char *p, *p2;
+		char *field_desc;
+		int len;
+
+		p = strchr(schema, '(');
+		if (p)
 		{
-			schema = (char *)res_autoinc->fv(0).get_asString().data();
-			
-			search = (char *)alloca(2 + strlen(_fieldName));
-			sprintf(search, "'%s'", _fieldName);
-			
-			p = strchr(schema, '(');
-			if (p)
+			while (*p != ')')
 			{
-				p = strstr(p, search);
+				p++;
+				p2 = strchr(p, ',');
+				if (!p2)
+					p2 = p + strlen(p) - 1;
+
+				while (p < p2 && *p == ' ')
+					p++;
+
+				if (*p == '\'')
+					p++;
+
+				len = strlen(_fieldName);
+				if ((p2 - p) < len || strncasecmp(p, _fieldName, len))
+				{
+					p = p2;
+					continue;
+				}
+
+				p += len;
+				if (*p == '\'')
+					p++;
+
+				len = p2 - p;
+				if (len <= 0)
+					break;
+				
+				field_desc = GB.NewString(p, len);
+
+				if (strstr(_fieldType, "INTEGER"))
+				{
+					if (strstr(field_desc, "AUTOINCREMENT"))
+						autoinc = true;
+				}
+
+				p = strstr(field_desc, "COLLATE");
 				if (p)
 				{
-					p2 = strchr(p, ',');
+					p += 7;
+					while (*p == ' ')
+						p++;
+
+					p2 = strchr(p, ' ');
 					if (!p2)
-						p2 = strchr(p, ')');
-					if (p2)
-					{
-						p = strstr(p, "AUTOINCREMENT");
-						if (p && p < p2)
-							autoinc = true;
-					}
+						p2 = field_desc + len;
+					info->collation = GB.NewString(p, p2 - p);
 				}
+
+				GB.FreeString(&field_desc);
+				break;
 			}
-			
-			res_autoinc->close();
+
 		}
-		
 	}
 	
+	if (!_table_schema)
+		GB.FreeString(&schema);
+
 	type = GetFieldType(_fieldType, (unsigned int *) &info->length);
 
 	if (autoinc)
@@ -2015,7 +2152,7 @@ static int index_info(DB_DATABASE * db, const char *table, const char *index, DB
 			 table))
 		return TRUE;
 
-	result_set *r = (result_set *) res->getExecRes();
+	result_set *r = (result_set *) res->getResult();
 
 	if ((n = r->records.size()) == 0)
 	{
@@ -2041,12 +2178,8 @@ static int index_info(DB_DATABASE * db, const char *table, const char *index, DB
 	}
 
 	info->name = NULL;
-	info->unique =
-		strncmp("1", r->records[i][2].get_asString().data(),
-						1) == 0 ? TRUE : FALSE;
-	info->primary =
-		strstr(r->records[i][1].get_asString().data(),
-					 "autoindex") != NULL ? TRUE : FALSE;
+	info->unique = strncmp("1", r->records[i][2].get_asString().data(), 1) == 0 ? TRUE : FALSE;
+	info->primary = strstr(r->records[i][1].get_asString().data(), "autoindex") != NULL ? TRUE : FALSE;
 
 	DB.Query.Init();
 
@@ -2057,7 +2190,7 @@ static int index_info(DB_DATABASE * db, const char *table, const char *index, DB
 		return TRUE;
 	}
 
-	r = (result_set *) res->getExecRes();
+	r = (result_set *) res->getResult();
 	n = r->records.size();
 	i = 0;
 	/* (BM) row can be null if we are seeking the last index */
@@ -2212,7 +2345,7 @@ static int database_list(DB_DATABASE * db, char ***databases)
 		GB.Free(POINTER(&dbhome));
 	}
 
-	/*if (getcwd(cwd, MAX_PATH) != NULL){
+	/*if (getcwd(cwd, PATH_MAX) != NULL){
 	   if (strcmp(cwd, dbhome) != 0){
 	   WalkDirectory( cwd, databases );
 	   }
