@@ -221,8 +221,10 @@ static int my_sqlite3_exec(
 		char **azVals = 0;
 
 		pStmt = 0;
+		//fprintf(stderr, "my_sqlite3_exec: sqlite3_prepare_v2 ?\n");
 		rc = sqlite3_prepare_v2(db, zSql, -1, &pStmt, &zLeftover);
 		if( rc!=SQLITE_OK ){
+			//fprintf(stderr, "my_sqlite3_exec: sqlite3_prepare_v2 failed: %s\n", sqlite3_errmsg(db));
 			if( pStmt ) sqlite3_finalize(pStmt);
 			continue;
 		}
@@ -421,10 +423,10 @@ static int callback(void *res_ptr, int ncol, char **reslt, char **cols, sqlite3_
 	return 0;
 }
 
-static int old_callback(void *res_ptr, int ncol, char **reslt, char **cols)
+/*static int old_callback(void *res_ptr, int ncol, char **reslt, char **cols)
 {
 	return callback(res_ptr, ncol, reslt, cols, NULL);
-}
+}*/
 
 
 //************* SqliteDatabase implementation ***************
@@ -470,7 +472,7 @@ int SqliteDatabase::setErr(int err_code)
 			error = "Successful result";
 			break;
 		case SQLITE_ERROR:
-			error = "SQL error or missing database";
+			error = sqlite3_errmsg(getHandle());
 			break;
 		case SQLITE_INTERNAL:
 			error = "Internal logic error - Report this error on the mailing-list at sqlite.org";
@@ -544,39 +546,12 @@ int SqliteDatabase::connect()
 {
 	disconnect();
 
-	if (sqlite3_open(db.c_str(), &conn) == SQLITE_OK)
-	{
-		//cout << "Connected!\n";
-		//char *err = NULL;
+	if (sqlite3_open(db.c_str(), &conn) != SQLITE_OK)
+		return DB_CONNECTION_NONE;
 
-		if (setErr
-				(sqlite3_exec
-				(getHandle(), "PRAGMA empty_result_callbacks=ON", NULL, NULL,
-					NULL)) != SQLITE_OK)
-		{
-			GB.Error(getErrorMsg());
-		}
-		active = true;
-		/* NG 29/12/2005 - 3.2.1 introduced a problem with columns names
-		* which is resolved by setting short columns off first */
-		if (setErr
-				(sqlite3_exec
-				(getHandle(), "PRAGMA short_column_names=OFF", NULL, NULL,
-					NULL)) != SQLITE_OK)
-		{														//NG
-			GB.Error(getErrorMsg());	//NG
-		}
-		if (setErr
-				(sqlite3_exec
-				(getHandle(), "PRAGMA full_column_names=ON", NULL, NULL,
-					NULL)) != SQLITE_OK)
-		{														//NG
-			GB.Error(getErrorMsg());	//NG
-		}
-		return DB_CONNECTION_OK;
-		sqlite3_close(conn);
-	}
-	return DB_CONNECTION_NONE;
+	active = true;
+
+	return DB_CONNECTION_OK;
 };
 
 void SqliteDatabase::disconnect(void)
@@ -821,64 +796,11 @@ void SqliteDataset::fill_fields()
 
 // BM: should retry if error = SQLITE_SCHEMA
 
-int SqliteDataset::exec(const string & sql)
-{
-	int res;
-	int retry;
-
-
-	if (!handle())
-		GB.Error("No database connection");
-
-	exec_res.record_header.clear();
-	exec_res.records.clear();
-	exec_res.conn = handle();			//NG
-	//if ((strncmp("select",sql.c_str(),6) == 0) || (strncmp("SELECT",sql.c_str(),6) == 0))
-
-	for (retry = 1; retry <= 2; retry++)
-	{
-		res = my_sqlite3_exec(handle(), sql.c_str(), &callback, &exec_res, NULL);
-		if (res != SQLITE_SCHEMA)
-			break;
-	}
-
-	db->setErr(res);
-	//if (res != SQLITE_OK)
-	//  GB.Error(db->getErrorMsg());
-
-	return (res == SQLITE_OK);
-}
-
-int SqliteDataset::exec()
-{
-	return exec(sql);
-}
-
-const void *SqliteDataset::getExecRes()
-{
-	return &exec_res;
-}
-
-
 bool SqliteDataset::query(const char *query)
 {
 	int res;
 	int retry;
 
-	//try{
-	if (db == NULL)
-		GB.Error("Database is not defined");
-	if (dynamic_cast < SqliteDatabase * >(db)->getHandle() == NULL)
-		GB.Error("No database connection");
-	if ((strncasecmp("select", query, 6) != 0)	/*&&
-																								(strncasecmp("PRAGMA table",query,12) !=0) &&
-																								(strncasecmp("PRAGMA index",query,12) !=0) */
-		)
-		GB.Error("Syntax error in request: SELECT expected.");
-
-	//close();
-
-	//cout <<  "Curr size "<<num_rows()<<"\n\n";
 	result.conn = handle();				//NG
 
 	for (retry = 1; retry <= 2; retry++)
@@ -903,11 +825,6 @@ bool SqliteDataset::query(const char *query)
 	return (res == SQLITE_OK);
 }
 
-bool SqliteDataset::query(const string & q)
-{
-	return query(q.c_str());
-}
-
 void SqliteDataset::open(const string & sql)
 {
 	set_select_sql(sql);
@@ -973,7 +890,7 @@ bool SqliteDataset::bof()
 void SqliteDataset::first()
 {
 	Dataset::first();
-	this->fill_fields();
+	fill_fields();
 	//cout << "In first "<< fields_object->size()<<"\n";
 }
 
@@ -1201,3 +1118,8 @@ fType GetFieldType(const char *Type, unsigned int *length)
 
 	return rType;
 }
+
+result_set *SqliteDataset::getResult()
+{
+	return &result;
+}
