@@ -406,6 +406,14 @@ BEGIN_PROPERTY(CCOMBOBOX_list)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(ComboBox_Border)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(COMBOBOX->hasBorder());
+	else
+		COMBOBOX->setBorder(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
 
 /***************************************************************************
 
@@ -490,6 +498,7 @@ GB_DESC CComboBoxDesc[] =
   GB_PROPERTY("ReadOnly", "b", CTEXTBOX_read_only),
   GB_PROPERTY("Password", "b", CTEXTBOX_password),
   GB_PROPERTY("MaxLength", "i", CTEXTBOX_max_length),
+  GB_PROPERTY("Border", "b", ComboBox_Border),
   
   GB_PROPERTY_SELF("Selection", ".TextBox.Selection"),
   GB_METHOD("Select", 0, CTEXTBOX_sel_select, "[(Start)i(Length)i]"),
