@@ -109,8 +109,8 @@ public:
 	gColor foreground();
 	virtual void setBackground(gColor color = COLOR_DEFAULT);
 	virtual void setForeground(gColor color = COLOR_DEFAULT);
-	gColor realBackground();
-	gColor realForeground();
+	gColor realBackground(bool no_default = false);
+	gColor realForeground(bool no_default = false);
 	virtual void setRealBackground(gColor color);
 	virtual void setRealForeground(gColor color);
 
@@ -120,6 +120,16 @@ public:
 	bool ownFont() { return _font != NULL; }
 	virtual void updateFont();
 	virtual void updateSize();
+#ifdef GTK3
+	virtual void updateColor();
+	void setColorNames(const char *bg_names[], const char *fg_names[]);
+	void setColorBase();
+	void setColorButton();
+#else
+	void setColorNames(const char **, const char **) {}
+	void setColorBase() { use_base = TRUE; }
+	void setColorButton() { use_base = FALSE; }
+#endif
 
 	bool canFocus() const;
 	void setCanFocus(bool vl);
@@ -132,10 +142,11 @@ public:
 	void scroll(int x, int y);
 	void setScrollX(int vl);
 	void setScrollY(int vl);
-	virtual int scrollWidth();
-	virtual int scrollHeight();
-	int scrollBar();
+	//virtual int scrollWidth();
+	//virtual int scrollHeight();
+	int scrollBar() const { return _scrollbar; }
 	void setScrollBar(int vl);
+	virtual void updateScrollBar();
 
 // "Methods"
 	void dragText(char *txt, char *format = NULL) { gDrag::dragText(this, txt, format); }
@@ -149,7 +160,12 @@ public:
 	virtual void resize(int w, int h);
 	virtual void moveResize(int x, int y, int w, int h);
 	virtual void setFocus();
-	bool hasFocus();
+	bool hasFocus() const;
+#if GTK_CHECK_VERSION(3, 2, 0)
+	bool hasVisibleFocus() const;
+#else
+	bool hasVisibleFocus() const { return hasFocus(); }
+#endif
 	void resize() { resize(width(), height()); }
 	void show() { setVisible(true); }
 	void refresh();
@@ -194,8 +210,17 @@ public:
 	GtkWidget *frame;
 	GtkScrolledWindow *_scroll;
 	short g_typ;
-	short mous;
+	short _mouse;
 	gControl *_proxy, *_proxy_for;
+	gColor _bg, _fg;
+#ifdef GTK3
+	const char *_bg_name;
+	const char **_bg_name_list;
+	GdkRGBA _bg_default;
+	const char *_fg_name;
+	const char **_fg_name_list;
+	GdkRGBA _fg_default;
+#endif
 	
 	unsigned dsg : 1;
 	unsigned expa : 1;
@@ -209,7 +234,6 @@ public:
 	unsigned _old_tracking : 1;            // real value when Tracking is false
 	unsigned _bg_set : 1;                  // Have a private background
 	unsigned _fg_set : 1;                  // Have a private foreground
-	unsigned _font_set : 1;                // Have a private font
 	unsigned have_cursor : 1;              // If gApplication::setBusy() must update the cursor
 	unsigned use_base : 1;                 // Use base and text color for foreground and background
 	unsigned visible : 1;                  // A control can be hidden if its width or height is zero
@@ -217,14 +241,13 @@ public:
 	
 	unsigned _locked : 4;                  // For locking events
 	unsigned frame_border : 4;
-
 	unsigned frame_padding : 8;
-	
-	unsigned _scrolled_window : 1;
+
+	unsigned _scrollbar : 2;
 	unsigned _dirty_pos : 1;               // If the position of the widget has changed
 	unsigned _dirty_size : 1;              // If the size of the widget has changed
 	unsigned _no_delete : 1;               // Do not delete on destroy signal
-	unsigned no_input_method : 1;          // No input method management
+	unsigned _has_input_method : 1;        // Has its own input method management
 	unsigned _no_default_mouse_event : 1;  // No default mouse events
 	unsigned _grab : 1;                    // control is currently grabbing mouse and keyboard
 	unsigned _has_border : 1;              // if the control has a border
@@ -246,7 +269,8 @@ public:
 	void updateGeometry();
 	bool mustUpdateCursor() { return mouse() != -1 || have_cursor; }
 	
-	bool noInputMethod() { return no_input_method; }
+	bool hasInputMethod() { return _has_input_method; }
+	virtual GtkIMContext *getInputMethod();
 	
 	GdkCursor *getGdkCursor();
 	virtual void updateBorder();
@@ -258,12 +282,18 @@ public:
 	void setFramePadding(int padding);
 	virtual int getFrameWidth();
 	virtual gColor getFrameColor();
+#ifdef GTK3
+	void drawBorder(cairo_t *cr);
+	void drawBackground(cairo_t *cr);
+#else
 	void drawBorder(GdkEventExpose *e);
-	void drawBackground(GtkWidget *wid, GdkEventExpose *e);
+	void drawBackground(GdkEventExpose *e);
+#endif
 	
-	virtual int minimumHeight();
+	virtual int minimumHeight() const;
+	virtual int minimumWidth() const;
 	void resolveFont();
-	
+
 	void emitEnterEvent(bool no_leave = false);
 	void emitLeaveEvent();
 	
