@@ -22,7 +22,6 @@
 ***************************************************************************/
 
 #include "widgets.h"
-#include "widgets_private.h"
 #include "gb.form.font.h"
 
 #ifndef GAMBAS_DIRECTFB
@@ -55,6 +54,7 @@ bool gDesktop::rightToLeft()
 void gDesktop::init()
 {
 	_desktop_font = new gFont();
+	_desktop_font->setAll(true);
 	_desktop_scale = 0;
 }
 
@@ -115,84 +115,160 @@ gMainWindow* gDesktop::activeWindow()
 	return gMainWindow::_active ? gMainWindow::_active->topLevel() : NULL;
 }
 
+#ifdef GTK3
+static gColor get_color(GType type, gColor default_color, GtkStateFlags state, bool fg, bool text)
+{
+	static const char *bg_names[] = { "bg_color", "theme_bg_color", NULL };
+	static const char *fg_names[] = { "fg_color", "theme_fg_color", NULL };
+	static const char *base_bg_names[] = { "base_color", "theme_base_color", NULL };
+	static const char *base_fg_names[] = { "text_color", "theme_text_color", NULL };
+	static const char *sel_bg_names[] = { "selected_bg_color", "theme_selected_bg_color", NULL };
+	static const char *sel_fg_names[] = { "selected_fg_color", "theme_selected_fg_color", NULL };
+	static const char *tt_bg_names[] = { "tooltip_bg_color", "theme_tooltip_bg_color", NULL };
+	static const char *tt_fg_names[] = { "link_fg_color", "theme_tooltip_fg_color", NULL };
+	static const char *link_fg_names[] = { "link_color", "theme_link_color", NULL };
+	static const char *visited_fg_names[] = { "visited_link_color", "theme_visited_link_color", NULL };
+
+	GtkStyleContext *st = gt_get_style(type);
+	GdkRGBA rgba;
+
+	if (type == GTK_TYPE_WINDOW)
+	{
+		if (!gt_style_lookup_color(st, fg ? fg_names : bg_names, NULL, &rgba))
+			goto __OK;
+	}
+	else if (type == GTK_TYPE_ENTRY)
+	{
+		if (state == GTK_STATE_FLAG_SELECTED)
+		{
+			if (!gt_style_lookup_color(st, fg ? sel_fg_names : sel_bg_names, NULL, &rgba))
+				goto __OK;
+		}
+		else
+		{
+			if (!gt_style_lookup_color(st, fg ? base_fg_names : base_bg_names, NULL, &rgba))
+				goto __OK;
+		}
+	}
+	else if (type == GTK_TYPE_TOOLTIP)
+	{
+		if (!gt_style_lookup_color(st, fg ? tt_fg_names : tt_bg_names, NULL, &rgba))
+			goto __OK;
+	}
+	else if (type == GTK_TYPE_LINK_BUTTON)
+	{
+		if (!gt_style_lookup_color(st, fg ? link_fg_names : visited_fg_names, NULL, &rgba))
+			goto __OK;
+	}
+
+	if (!st)
+		return default_color;
+
+	if (fg)
+		gtk_style_context_get_color(st, state, &rgba);
+	else
+		gtk_style_context_get_background_color(st, state, &rgba);
+
+__OK:
+
+	return gt_to_color(&rgba);
+}
+#else
+static gColor get_color(GType type, gColor default_color, GtkStateType state, bool fg, bool text)
+{
+	GtkStyle *st = gt_get_style(type);
+	GdkColor *color;
+
+	if (!st)
+		return default_color;
+
+	if (text)
+	{
+		if (fg)
+			color = &st->text[state];
+		else
+			color = &st->base[state];
+	}
+	else
+	{
+		if (fg)
+			color = &st->fg[state];
+		else
+			color = &st->bg[state];
+	}
+	return get_gdk_color(color);
+}
+#endif
+
 gColor gDesktop::buttonfgColor()
 {
-	GtkStyle *st = gt_get_style("GtkButton", GTK_TYPE_BUTTON);
-
-	if (!st) return 0;
-	return get_gdk_color(&st->fg[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_BUTTON, 0, STATE_NORMAL, true, false);
 }
 
 gColor gDesktop::buttonbgColor()
 {
-	GtkStyle *st = gt_get_style("GtkButton", GTK_TYPE_BUTTON);	
-
-	if (!st) return 0xC0C0C0;
-	return get_gdk_color(&st->bg[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_BUTTON, 0xC0C0C0, STATE_NORMAL, false, false);
 }
 
 gColor gDesktop::fgColor()
 {
-	GtkStyle *st = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
-
-	if (!st) return 0;
-	return get_gdk_color(&st->fg[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_WINDOW, 0, STATE_NORMAL, true, false);
 }
 
 gColor gDesktop::bgColor()
 {
-	GtkStyle *st = gt_get_style("GtkLayout", GTK_TYPE_LAYOUT);
-
-	if (!st) return 0xC0C0C0;
-	return get_gdk_color(&st->bg[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_WINDOW, 0xC0C0C0, STATE_NORMAL, false, false);
 }
 
 gColor gDesktop::textfgColor()
 {
-	GtkStyle *st = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
-
-	if (!st) return 0;
-	return get_gdk_color(&st->text[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_ENTRY, 0, STATE_NORMAL, true, true);
 }
 
 gColor gDesktop::textbgColor()
 {
-	GtkStyle *st = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
-
-	if (!st) return 0xFFFFFF;
-	return get_gdk_color(&st->base[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_ENTRY, 0xFFFFFF, STATE_NORMAL, false, true);
 }
 
 gColor gDesktop::selfgColor()
 {
-	GtkStyle *st = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
-
-	if (!st) return 0XFFFFFF;
-	return get_gdk_color(&st->text[GTK_STATE_SELECTED]);
+	return get_color(GTK_TYPE_ENTRY, 0xFFFFFF, STATE_SELECTED, true, true);
 }
 
 gColor gDesktop::selbgColor()
 {
-	GtkStyle *st = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
-
-	if (!st) return 0x0000FF;
-	return get_gdk_color(&st->base[GTK_STATE_SELECTED]);
+	return get_color(GTK_TYPE_ENTRY, 0x0000FF, STATE_SELECTED, false, true);
 }
 
 gColor gDesktop::tooltipForeground()
 {
-	GtkStyle *st = gt_get_widget_style("gtk-tooltip");
-
-	if (!st) return fgColor();
-	return get_gdk_color(&st->fg[GTK_STATE_NORMAL]);
+	return get_color(GTK_TYPE_TOOLTIP, 0, STATE_NORMAL, true, false);
 }
 
 gColor gDesktop::tooltipBackground()
 {
-	GtkStyle *st = gt_get_widget_style("gtk-tooltip");
+	return get_color(GTK_TYPE_TOOLTIP, 0xC0C0C0, STATE_NORMAL, false, false);
+}
 
-	if (!st) return bgColor();
-	return get_gdk_color(&st->bg[GTK_STATE_NORMAL]);
+gColor gDesktop::linkForeground()
+{
+	return get_color(GTK_TYPE_LINK_BUTTON, 0x0000FF, STATE_NORMAL, true, false);
+}
+
+gColor gDesktop::visitedForeground()
+{
+#ifdef GTK3
+	return get_color(GTK_TYPE_LINK_BUTTON, 0x0080FF, STATE_NORMAL, false, false);
+#else
+	int r, g ,b, h, s, v;
+	gt_color_to_rgb(linkForeground(), &r, &g, &b);
+	gt_rgb_to_hsv(r, g, b, &h, &s, &v);
+	h -= 30;
+	if (h < 0)
+		h += 360;
+	gt_hsv_to_rgb(h, s, v, &r, &g, &b);
+	return gt_rgb_to_color(r, g, b);
+#endif
 }
 
 gColor gDesktop::lightbgColor()
@@ -219,11 +295,10 @@ int gDesktop::width()
 
 int gDesktop::resolution()
 {
-	/*int d_pix=gdk_screen_get_height(gdk_screen_get_default());
-	int d_mm=gdk_screen_get_height_mm(gdk_screen_get_default());
-	
-	return (int)(d_pix*25.4)/d_mm;*/
-	return gdk_screen_get_resolution(gdk_screen_get_default());
+	gdouble res = gdk_screen_get_resolution(gdk_screen_get_default());
+	if (res == -1)
+		res = 96;
+	return res;
 }
 
 int gDesktop::scale()
