@@ -25,12 +25,44 @@
 #include "gmouse.h"
 #include "gdrawingarea.h"
 
+#ifdef GTK3
+#define UNREF_BUFFER() (cairo_surface_destroy(buffer), buffer = NULL)
+#else
+#define UNREF_BUFFER() (g_object_unref(G_OBJECT(buffer)), buffer = NULL)
+#endif
+
+
 /****************************************************************************************
 
 gDrawingArea Widget
 
 *****************************************************************************************/
 
+#ifdef GTK3
+static gboolean cb_draw(GtkWidget *wid, cairo_t *cr, gDrawingArea *data)
+{
+	if (data->cached())
+	{
+		cairo_set_source_surface(cr, data->buffer, 0, 0);
+		cairo_paint(cr);
+		data->drawBorder(cr);
+	}
+	else
+	{
+		//data->drawBackground();
+
+		if (data->onExpose)
+		{
+			data->_in_draw_event = true;
+			data->onExpose(data, cr);
+			data->_in_draw_event = false;
+		}
+		data->drawBorder(cr);
+	}
+
+	return false;
+}
+#else
 static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gDrawingArea *data)
 {
 	if (data->cached())
@@ -44,7 +76,17 @@ static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gDrawingArea *data)
 		if (data->onExpose)
 		{
 			data->_in_draw_event = true;
-			data->onExpose(data,e->area.x-wid->allocation.x,e->area.y-wid->allocation.y,e->area.width,e->area.height);
+
+			/*GdkRectangle *rect;
+			int i, n;
+			gdk_region_get_rectangles(e->region, &rect, &n);
+
+			for (i = 0; i < n; i++)
+				fprintf(stderr, "[%d] %d %d %d %d\n", i, rect[i].x, rect[i].y, rect[i].width, rect[i].height);
+
+			g_free(rect);*/
+
+			data->onExpose(data, e->region, wid->allocation.x, wid->allocation.y);
 			data->_in_draw_event = false;
 		}
 		data->drawBorder(e);
@@ -52,6 +94,7 @@ static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gDrawingArea *data)
 
 	return false;
 }
+#endif
 
 static void cb_size(GtkWidget *wid, GtkAllocation *a, gDrawingArea *data)
 {
@@ -66,26 +109,6 @@ static void cb_size(GtkWidget *wid, GtkAllocation *a, gDrawingArea *data)
 	return false;
 }*/
 
-void gDrawingArea::init()
-{
-	/*gtk_widget_add_events(border, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
-		| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK 
-		| GDK_SCROLL_MASK | GDK_POINTER_MOTION_MASK);
-
-	gtk_widget_add_events(widget, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
-		| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK 
-		| GDK_SCROLL_MASK | GDK_POINTER_MOTION_MASK);*/
-
-	//GTK_WIDGET_UNSET_FLAGS(border, GTK_APP_PAINTABLE);
-	//GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS);
-		
-	//_event_mask = gtk_widget_get_events(widget);
-	
-	//g_signal_connect(G_OBJECT(border), "button-press-event",G_CALLBACK(cb_button_press),(gpointer)this);
-}	
-
 void gDrawingArea::create(void)
 {
 	int i;
@@ -132,7 +155,7 @@ void gDrawingArea::create(void)
 	realize(false);
 	
 	g_signal_connect(G_OBJECT(border), "size-allocate", G_CALLBACK(cb_size), (gpointer)this);
-	g_signal_connect(G_OBJECT(border), "expose-event", G_CALLBACK(cb_expose), (gpointer)this);
+	ON_DRAW_BEFORE(border, this, cb_expose, cb_draw);
 	
 	updateUseTablet();
 	
@@ -173,6 +196,7 @@ gDrawingArea::gDrawingArea(gContainer *parent) : gContainer(parent)
 	_use_tablet = false;
 	
 	onExpose = NULL;
+	onFontChange = NULL;
 		
 	g_typ = Type_gDrawingArea;
 	
@@ -182,7 +206,7 @@ gDrawingArea::gDrawingArea(gContainer *parent) : gContainer(parent)
 gDrawingArea::~gDrawingArea()
 {
 	if (buffer)
-		g_object_unref(G_OBJECT(buffer));
+		UNREF_BUFFER();
 }
 
 void gDrawingArea::resize(int w, int h)
@@ -227,8 +251,7 @@ void gDrawingArea::setCached(bool vl)
 	
 	if (!_cached)
 	{
-		g_object_unref(G_OBJECT(buffer));
-		buffer = NULL;
+		UNREF_BUFFER();
 		set_gdk_bg_color(border, background());	
 	}
 	
@@ -240,14 +263,18 @@ void gDrawingArea::resizeCache()
 {
 	int bw, bh;
 	int w, h;
+#ifdef GTK3
+	cairo_surface_t *buf;
+#else
 	GdkPixmap *buf;
+#endif
 	GdkWindow *win;
 	cairo_t *cr;
 	
 	if (!_cached)
 		return;
 	
-	win = GTK_WIDGET(box)->window;
+	win = gtk_widget_get_window(GTK_WIDGET(box));
 	if (!win)
 		return;
 	
@@ -255,19 +282,30 @@ void gDrawingArea::resizeCache()
 	h = height();
 	
 	if (buffer)
+	{
+#ifdef GTK3
+		bw = cairo_image_surface_get_width(buffer);
+		bh = cairo_image_surface_get_height(buffer);
+#else
 		gdk_drawable_get_size(buffer, &bw, &bh);
+#endif
+	}
 	else
 		bw = bh = 0;
 	
 	if (bw != w || bh != h)
-	{		
+	{
+#ifdef GTK3
+		buf = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
+		cr = cairo_create(buf);
+#else
 		buf = gdk_pixmap_new(win, w, h, -1);
-		
 		cr = gdk_cairo_create(buf);
+#endif
 		
 		if (w > bw || h > bh || !buffer)
 		{
-			gt_cairo_set_source_color(cr, realBackground());
+			gt_cairo_set_source_color(cr, realBackground(true));
 			cairo_rectangle(cr, 0, 0, w, h);
 			cairo_fill(cr);
 		}
@@ -277,10 +315,15 @@ void gDrawingArea::resizeCache()
 			if (bw > w) bw = w;
 			if (bh > h) bh = h;
 			//gdk_draw_drawable(buf, gc2, buffer, 0, 0, 0, 0, bw, bh);
+#ifdef GTK3
+			cairo_set_source_surface(cr, buffer, 0, 0);
+#else
 			gdk_cairo_set_source_pixmap(cr, buffer, 0, 0);
+#endif
 			cairo_rectangle(cr, 0, 0, bw, bh);
 			cairo_fill(cr);
-			g_object_unref(buffer);
+
+			UNREF_BUFFER();
 		}
 		
 		buffer = buf;
@@ -296,7 +339,10 @@ void gDrawingArea::setCache()
 	if (!_cached)
 		return;
 
-	gdk_window_set_back_pixmap(box->window, buffer, FALSE);
+#ifdef GTK3
+#else
+	gdk_window_set_back_pixmap(gtk_widget_get_window(box), buffer, FALSE);
+#endif
 	refreshCache();
 }
 
@@ -325,9 +371,9 @@ void gDrawingArea::clear()
 {
 	if (_cached && buffer) 
 	{
-		g_object_unref(buffer);
-		buffer = NULL;
+		UNREF_BUFFER();
 		resizeCache();
+		setCache();
 	}
 }
 
@@ -363,7 +409,9 @@ void gDrawingArea::updateUseTablet()
 {
 	if (_use_tablet)
 		gMouse::initDevices();
+#ifndef GTK3
 	gtk_widget_set_extension_events(border, _use_tablet ? GDK_EXTENSION_EVENTS_CURSOR : GDK_EXTENSION_EVENTS_NONE);
+#endif
 	//fprintf(stderr, "gtk_widget_set_extension_events: %s %p: %d\n", name(), border, _use_tablet);
 }
 
@@ -373,4 +421,10 @@ void gDrawingArea::setUseTablet(bool vl)
 		return;
 	_use_tablet = vl;
 	create();
-}
\ No newline at end of file
+}
+
+void gDrawingArea::updateFont()
+{
+	gContainer::updateFont();
+	emit(SIGNAL(onFontChange));
+}
