@@ -47,12 +47,17 @@ public:
 	virtual void resize(int w, int h);
 	virtual void setEnabled(bool vl);
 	virtual void setRealBackground(gColor color);
+	virtual void updateFont();
 
 //"Events"
-	void (*onExpose)(gDrawingArea *sender,int x,int y,int w,int h);
+#ifdef GTK3
+	void (*onExpose)(gDrawingArea *sender, cairo_t *cr);
+#else
+	void (*onExpose)(gDrawingArea *sender, GdkRegion *region, int dx, int dy);
+#endif
+	void (*onFontChange)(gDrawingArea *sender);
 
 //"Private"
-	void init();
 	void create();
 	void updateCache();
 	void resizeCache();
@@ -61,7 +66,11 @@ public:
 	void setCache();
 	void updateUseTablet();
 	
+#ifdef GTK3
+	cairo_surface_t *buffer;
+#else
 	GdkPixmap *buffer;
+#endif
 	GtkWidget *box;
 	uint _event_mask;
 	uint _old_bg_id;
