@@ -50,11 +50,16 @@ public:
 //"Signals"
 	void (*onChange)  (gSpinBox *sender);
 
+#ifndef GTK3
+	virtual void updateCursor(GdkCursor *cursor);
+#else
+	virtual int minimumWidth() const;
+#endif
+
 //"Private"
 private:
 	int _min;
 	int _max;
-	virtual void updateCursor(GdkCursor *cursor);
 };
 
 #endif
