@@ -27,6 +27,8 @@
 
 #include "c_media.h"
 //#include <gst/interfaces/xoverlay.h>
+#include <gst/base/gstbasesink.h>
+#include <gst/video/video.h>
 #include <gst/video/videooverlay.h>
 
 static void *_from_element = NULL;
@@ -298,6 +300,13 @@ static void return_value(const GValue *value)
 				else
 					GB.ReturnNewZeroString(enum_value->value_nick);
 			}
+			else if (type == GST_TYPE_CAPS)
+			{
+				char *caps;
+				caps = gst_caps_to_string((GstCaps *)g_value_get_boxed(value));
+				GB.ReturnNewZeroString(caps);
+				g_free(caps);
+			}
 			else
 			{
 				fprintf(stderr, "gb.media: warning: unsupported datatype: %s\n", G_VALUE_TYPE_NAME(value));
@@ -378,22 +387,50 @@ static bool set_value(GValue *value, GB_VALUE *v, GParamSpec *desc)
 			if (G_TYPE_IS_ENUM(type))
 			{
 				GEnumValue *enum_value;
+				int real_value;
 				
-				if (GB.Conv(v, GB_T_STRING))
-					return TRUE;
-				
-				enum_value = g_enum_get_value_by_nick(G_ENUM_CLASS(g_type_class_ref(type)), GB.ToZeroString((GB_STRING *)v));
-				if (!enum_value)
+				if (!GB.Conv(v, GB_T_INTEGER))
 				{
-					GB.Error("Unknown enumeration value");
-					return TRUE;
+					real_value = ((GB_INTEGER *)v)->value;
+				}
+				else
+				{
+					if (GB.Conv(v, GB_T_STRING))
+						return TRUE;
+
+					enum_value = g_enum_get_value_by_nick(G_ENUM_CLASS(g_type_class_ref(type)), GB.ToZeroString((GB_STRING *)v));
+					if (!enum_value)
+					{
+						GB.Error("Unknown enumeration value");
+						return TRUE;
+					}
+
+					real_value = enum_value->value;
 				}
 				
-				g_value_set_enum(value, enum_value->value);
-				break;
+				g_value_set_enum(value, real_value);
 			}
+			else if (type == GST_TYPE_CAPS)
+			{
+				GstCaps *caps;
 
-			fprintf(stderr, "gb.media: warning: unsupported datatype: %s\n", g_type_name(type));
+				if (GB.Conv(v, GB_T_STRING))
+					return TRUE;
+
+				caps = gst_caps_from_string(GB.ToZeroString((GB_STRING *)v));
+				if (!caps)
+				{
+					GB.Error("Incorrect filter");
+					return TRUE;
+				}
+
+				g_value_take_boxed(value, caps);
+			}
+			else
+			{
+				GB.Error("Unsupported datatype: &1", g_type_name(type));
+				return TRUE;
+			}
 	}
 	
 	return FALSE;
@@ -415,6 +452,128 @@ void MEDIA_set_property(void *_object, const char *property, GB_VALUE *v)
 	g_value_unset(&value);
 }
 
+GB_IMG *MEDIA_get_image_from_sample(GstSample *sample, bool convert)
+{
+	GstSample *temp;
+	GError *err = NULL;
+	GstStructure *s;
+	GstCaps *to_caps, *sample_caps;
+	gint outwidth = 0;
+	gint outheight = 0;
+	GstMemory *memory;
+	GstMapInfo info;
+	const char *format;
+	int gb_format;
+	GB_IMG *img;
+
+	switch (IMAGE.GetDefaultFormat())
+	{
+		case GB_IMAGE_BGRA:
+			format = "BGR";
+			gb_format = GB_IMAGE_BGR;
+			break;
+
+		case GB_IMAGE_RGBA:
+			format = "RGB";
+			gb_format = GB_IMAGE_RGB;
+			break;
+
+		default:
+			GB.Error("Unsupported default image format");
+			return NULL;
+	}
+
+	if (convert)
+	{
+		/* our desired output format (RGB or BGR) */
+		to_caps = gst_caps_new_simple ("video/x-raw",
+			"format", G_TYPE_STRING, format,
+			/* Note: we don't ask for a specific width/height here, so that
+				* videoscale can adjust dimensions from a non-1/1 pixel aspect
+				* ratio to a 1/1 pixel-aspect-ratio. We also don't ask for a
+				* specific framerate, because the input framerate won't
+				* necessarily match the output framerate if there's a deinterlacer
+				* in the pipeline. */
+			"pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
+			NULL);
+
+		temp = gst_video_convert_sample(sample, to_caps, 25 * GST_SECOND, &err);
+
+		if (temp == NULL && err)
+		{
+			GB.Error(err->message);
+			gst_caps_unref(to_caps);
+			gst_sample_unref(sample);
+			g_error_free(err);
+			return NULL;
+		}
+
+		gst_sample_unref(sample);
+		gst_caps_unref(to_caps);
+
+		sample = temp;
+	}
+
+	if (!sample)
+	{
+		GB.Error("Unable to retrieve or convert video frame");
+		return NULL;
+	}
+
+	sample_caps = gst_sample_get_caps(sample);
+	if (!sample_caps)
+	{
+		GB.Error("No caps on video frame");
+		gst_sample_unref(sample);
+		return NULL;
+	}
+
+	//fprintf(stderr, "frame caps: %" GST_PTR_FORMAT, sample_caps);
+
+	s = gst_caps_get_structure (sample_caps, 0);
+	gst_structure_get_int (s, "width", &outwidth);
+	gst_structure_get_int (s, "height", &outheight);
+	if (outwidth <= 0 || outheight <= 0)
+	{
+		GB.Error("Bad image dimensions");
+		gst_sample_unref(sample);
+		return NULL;
+	}
+
+	memory = gst_buffer_get_memory (gst_sample_get_buffer (sample), 0);
+	gst_memory_map(memory, &info, GST_MAP_READ);
+
+	/* create pixbuf from that - use our own destroy function */
+	/*pixbuf = gdk_pixbuf_new_from_data (info.data,
+			GDK_COLORSPACE_RGB, FALSE, 8, outwidth, outheight,
+			GST_ROUND_UP_4 (outwidth * 3), destroy_pixbuf, sample);*/
+
+	img = IMAGE.Create(outwidth, outheight, gb_format, info.data);
+
+	gst_memory_unmap(memory, &info);
+
+	gst_sample_unref(sample);
+	return img;
+}
+
+static GB_IMG *get_last_image(void *_object)
+{
+	GstElement *elt = GST_ELEMENT(ELEMENT);
+	GstSample *sample;
+
+	if (!GST_IS_BASE_SINK(elt))
+	{
+		GB.Error("Not supported on this control");
+		return NULL;
+	}
+
+	sample = gst_base_sink_get_last_sample(GST_BASE_SINK(elt));
+	if (sample == NULL)
+		return NULL;
+
+	return MEDIA_get_image_from_sample(sample, TRUE);
+}
+
 
 #if 0
 //---- MediaSignalArguments -----------------------------------------------
@@ -520,6 +679,76 @@ BEGIN_PROPERTY(MediaTagList_Tags)
 
 END_PROPERTY
 
+//---- MediaLink ----------------------------------------------------------
+
+static CMEDIALINK *create_link(GstPad *pad)
+{
+	CMEDIALINK *ob;
+
+	ob = GB.New(GB.FindClass("MediaLink"), NULL, NULL);
+	ob->pad = pad;
+	return ob;
+}
+
+BEGIN_METHOD_VOID(MediaLink_free)
+
+	gst_object_unref(LINK);
+
+END_METHOD
+
+BEGIN_PROPERTY(MediaLink_Name)
+
+	char *name = gst_pad_get_name(LINK);
+	GB.ReturnNewZeroString(name);
+	g_free(name);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(MediaLink_Peer)
+
+	GstPad *peer = gst_pad_get_peer(LINK);
+
+	if (!peer)
+	{
+		GB.ReturnNull();
+		return;
+	}
+
+	GB.ReturnObject(MEDIA_get_control_from_element(gst_pad_get_parent_element(peer), TRUE));
+	gst_object_unref(peer);
+
+END_PROPERTY
+
+static void return_peer_name(void *_object, GstPadDirection dir)
+{
+	if (gst_pad_get_direction(LINK) == dir)
+	{
+		GstPad *peer = gst_pad_get_peer(LINK);
+		if (peer)
+		{
+			char *name = gst_pad_get_name(peer);
+			GB.ReturnNewZeroString(name);
+			g_free(name);
+			gst_object_unref(peer);
+			return;
+		}
+	}
+
+	GB.ReturnVoidString();
+}
+
+BEGIN_PROPERTY(MediaLink_Input)
+
+	return_peer_name(THIS_LINK, GST_PAD_SRC);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(MediaLink_Output)
+
+	return_peer_name(THIS_LINK, GST_PAD_SINK);
+
+END_PROPERTY
+
 //---- MediaControl -------------------------------------------------------
 
 DECLARE_EVENT(EVENT_State);
@@ -547,23 +776,13 @@ static MEDIA_TYPE _types[] =
 static void cb_pad_added(GstElement *element, GstPad *pad, CMEDIACONTROL *_object)
 {
 	char *name;
-	//GstPad *sinkpad;
-	
-	//fprintf(stderr, "cb_pad_added: %s\n", gst_element_factory_get_klass(gst_element_get_factory(ELEMENT)));
 
 	if (!THIS->dest)
 		return;
 	
-	/* We can now link this pad with the vorbis-decoder sink pad */
-	//sinkpad = gst_element_get_static_pad (decoder, "sink");
-	//gst_pad_link (pad, sinkpad);
-	
 	name = gst_pad_get_name(pad);
 	gst_element_link_pads(ELEMENT, name, ((CMEDIACONTROL *)THIS->dest)->elt, NULL);
 	g_free(name);
-	
-	//GB.Unref(POINTER(&THIS->dest));
-	//gst_object_unref (sinkpad);
 }
 
 BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
@@ -572,6 +791,7 @@ BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
 	CMEDIACONTAINER *parent;
 	MEDIA_TYPE *mtp;
 	GB_CLASS klass;
+	char *filter = NULL;
 	
 	//fprintf(stderr, "MediaControl_new: %p\n", THIS);
 
@@ -617,11 +837,19 @@ BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
 			}
 		}
 		else
+		{
 			type = GB.ToZeroString(ARG(type));
+			if (strchr(type, '/'))
+			{
+				filter = type;
+				type = "capsfilter";
+			}
+		}
 		
 		THIS->state = GST_STATE_NULL;
 		//THIS->type = GB.NewZeroString(type);
 		
+
 		ELEMENT = gst_element_factory_make(type, NULL);
 		if (!ELEMENT)
 		{
@@ -640,6 +868,9 @@ BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
 		}
 		else if (!GST_IS_PIPELINE(ELEMENT))
 			GB.CheckObject(parent);
+
+		if (filter)
+			MEDIA_set_property(THIS, "caps", (GB_VALUE *)ARG(type));
 	}
 		
 END_METHOD
@@ -683,6 +914,15 @@ BEGIN_PROPERTY(MediaControl_Type)
 	
 END_PROPERTY
 
+BEGIN_PROPERTY(MediaControl_Parent)
+
+	GstElement *parent = GST_ELEMENT(gst_element_get_parent(ELEMENT));
+	GB.ReturnObject(MEDIA_get_control_from_element(parent, TRUE));
+	if (parent)
+		gst_object_unref(parent);
+
+END_PROPERTY
+
 BEGIN_PROPERTY(MediaControl_Name)
 
 	if (READ_PROPERTY)
@@ -755,7 +995,23 @@ BEGIN_METHOD(MediaControl_LinkTo, GB_OBJECT dest; GB_STRING output; GB_STRING in
 	input = MISSING(input) ? NULL : GB.ToZeroString(ARG(input));
 	if (input && !*input) input = NULL;
 	
-	gst_element_link_pads(ELEMENT, output, dest->elt, input);
+	if (output)
+	{
+		GstPad *pad = gst_element_get_static_pad (ELEMENT, output);
+		if (pad)
+		{
+			if (GST_PAD_IS_SRC(pad))
+			{
+				GstPad *peer = gst_pad_get_peer(pad);
+				gst_pad_unlink(pad, peer);
+				gst_object_unref(peer);
+			}
+			gst_object_unref(pad);
+		}
+	}
+
+	if (!gst_element_link_pads(ELEMENT, output, dest->elt, input))
+		GB.Error("Unable to link controls");
 
 END_METHOD
 
@@ -786,7 +1042,7 @@ static GstIteratorResult iterator_next_pad(GstIterator *iter, GstPad **pad)
 		else
 			*pad = (GstPad *)g_value_get_object(&value);
 	}
-
+	
 	return ret;
 }
 
@@ -964,62 +1220,63 @@ BEGIN_PROPERTY(MediaControl_Protocols)
 END_PROPERTY
 #endif
 
-//---- MediaFilter --------------------------------------------------------
+BEGIN_METHOD(MediaControl_GetLink, GB_STRING name)
 
-static void set_filter(void *_object, const char *filter)
-{
-	GValue value = G_VALUE_INIT;
-	GstCaps *caps;
-	GParamSpec *desc;
+	bool done = FALSE;
+	GstIterator *iter;
+	GstPad *pad;
+	char *name;
+	char *search;
+	CMEDIALINK *link = NULL;
 
-	desc = get_property(ELEMENT, "caps");
-	if (!desc)
-		return;
+	search = GB.ToZeroString(ARG(name));
+	iter = gst_element_iterate_pads(ELEMENT);
 
-	g_value_init(&value, desc->value_type);
-	
-	caps = gst_caps_from_string(filter);
-	if (!caps)
+	while (!done)
 	{
-		GB.Error("Incorrect filter");
-		return;
+		switch (iterator_next_pad(iter, &pad))
+		{
+			case GST_ITERATOR_OK:
+				name = gst_pad_get_name(pad);
+				if (strcmp(name, search) == 0)
+				{
+					link = create_link(pad);
+					done = TRUE;
+					break;
+				}
+				g_free(name);
+				gst_object_unref(pad);
+				break;
+			case GST_ITERATOR_RESYNC:
+				gst_iterator_resync(iter);
+				break;
+			case GST_ITERATOR_ERROR:
+			case GST_ITERATOR_DONE:
+				done = TRUE;
+				break;
+		}
 	}
-	g_value_take_boxed(&value, caps);
-	g_object_set_property(G_OBJECT(ELEMENT), "caps", &value);
-	g_value_unset(&value);
-}
 
-BEGIN_METHOD(MediaFilter_new, GB_STRING filter)
-
-	if (!MISSING(filter))
-		set_filter(THIS, GB.ToZeroString(ARG(filter)));
+	gst_iterator_free(iter);
+	GB.ReturnObject(link);
 
 END_METHOD
 
+BEGIN_METHOD_VOID(MediaControl_GetLastImage)
+
+	GB.ReturnObject(get_last_image(THIS));
+
+END_PROPERTY
+
+
+//---- MediaFilter --------------------------------------------------------
+
 BEGIN_PROPERTY(MediaFilter_Filter)
 
 	if (READ_PROPERTY)
-	{
-		GParamSpec *desc;
-		GValue value = G_VALUE_INIT;
-		char *caps;
-	
-		desc = get_property(ELEMENT, "caps");
-		if (!desc)
-			return;
-		
-		g_value_init(&value, desc->value_type);
-		
-		g_object_get_property(G_OBJECT(ELEMENT), "caps", &value);
-		caps = gst_caps_to_string((GstCaps *)g_value_get_boxed(&value));
-		GB.ReturnNewZeroString(caps);
-		g_free(caps);
-		g_value_unset(&value);
-	}
+		MEDIA_return_property(THIS, "caps");
 	else
-	{
-		set_filter(THIS, GB.ToZeroString(PROP(GB_STRING)));
-	}
+		MEDIA_set_property(THIS, "caps", PROP(GB_VALUE));
 
 END_PROPERTY
 
@@ -1160,7 +1417,6 @@ static int cb_message(CMEDIAPIPELINE *_object)
 	{
 		type = GST_MESSAGE_TYPE(msg);
 		control = MEDIA_get_control_from_element(GST_MESSAGE_SRC(msg), FALSE);
-		//fprintf(stderr, "cb_message: control = %p\n", control);
 		
 		if (type == GST_MESSAGE_APPLICATION && control)
 		{
@@ -1182,7 +1438,8 @@ static int cb_message(CMEDIAPIPELINE *_object)
 		{
 			switch (type)
 			{
-				case GST_MESSAGE_EOS: 
+				case GST_MESSAGE_EOS:
+					THIS->eos = TRUE;
 					GB.Raise(THIS, EVENT_End, 0); 
 					break;
 				
@@ -1263,41 +1520,90 @@ static int cb_message(CMEDIAPIPELINE *_object)
 	return FALSE;
 }
 
-BEGIN_METHOD_VOID(MediaPipeline_new)
+static void stop_pipeline(CMEDIACONTROL *_object)
+{
+	int try;
+
+	// "It is not allowed to post GST_MESSAGE_EOS when not in the PLAYING state." says the documentation
+	if ((THIS->state == GST_STATE_PLAYING) && !THIS->eos)
+	{
+		try = 0;
+		gst_element_send_event(ELEMENT, gst_event_new_eos());
+		while (!THIS->eos)
+		{
+			try++;
+			if (try > 25)
+			{
+				fprintf(stderr, "gb.media: warning: could not catch end of stream\n");
+				break;
+			}
+			cb_message(THIS_PIPELINE);
+			usleep(10000);
+		}
+	}
+
+	MEDIA_set_state(THIS, GST_STATE_READY, TRUE);
+	cb_message(THIS_PIPELINE);
+}
+
+BEGIN_METHOD(MediaPipeline_new, GB_INTEGER polling)
 	
 	if (!_from_element)
-		THIS->watch = GB.Every(250, (GB_TIMER_CALLBACK)cb_message, (intptr_t)THIS);
+	{
+		int polling = VARGOPT(polling, 250);
+
+		if (polling <= 0)
+			polling = 250;
+		else if (polling < 10)
+			polling = 10;
+		else if (polling >= 1000)
+			polling = 1000;
+
+		THIS_PIPELINE->polling = polling;
+		THIS_PIPELINE->watch = GB.Every(polling, (GB_TIMER_CALLBACK)cb_message, (intptr_t)THIS);
+	}
 
 END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_free)
 
-	MEDIA_set_state(THIS, GST_STATE_READY, TRUE);
-	GB.Unref(POINTER(&THIS->watch));
+	stop_pipeline(THIS);
+	if (THIS_PIPELINE->watch)
+	{
+		//GB.Wait(THIS_PIPELINE->polling);
+		GB.Unref(POINTER(&THIS_PIPELINE->watch));
+	}
 
 END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_Play)
 
+	THIS->eos = FALSE;
 	MEDIA_set_state(THIS, GST_STATE_PLAYING, TRUE);
+	cb_message(THIS_PIPELINE);
 
 END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_Stop)
 
-	MEDIA_set_state(THIS, GST_STATE_READY, TRUE);
+	stop_pipeline(THIS);
 
 END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_Close)
 
 	MEDIA_set_state(THIS, GST_STATE_NULL, TRUE);
+	cb_message(THIS_PIPELINE);
 
 END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_Pause)
 
+	if (THIS->state != GST_STATE_PLAYING)
+		return;
+	
 	MEDIA_set_state(THIS, GST_STATE_PAUSED, TRUE);
+	cb_message(THIS_PIPELINE);
 
 END_METHOD
 
@@ -1314,12 +1620,12 @@ BEGIN_PROPERTY(MediaPipeline_Position)
 	}
 	else
 	{
-		guint64 pos = VPROP(GB_FLOAT) * 1E9;
+		gint64 pos = VPROP(GB_FLOAT) * 1E9;
 		
 		if (pos < 0) 
 			pos = 0;
-		
-		gst_element_seek_simple(ELEMENT, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT, pos);
+
+		gst_element_seek_simple(ELEMENT, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT, (guint64)pos);
 	}
 
 END_PROPERTY
@@ -1352,7 +1658,7 @@ BEGIN_METHOD(Media_Link, GB_OBJECT controls)
 	{
 		c1 = VALUE(&controls[i]);
 		c2 = VALUE(&controls[i + 1]);
-		if (GB.CheckObject(c1))
+		if ((i == 0 && GB.CheckObject(c1)) || GB.CheckObject(c2))
 			return;
 		gst_element_link(c1->elt, c2->elt);
 	}
@@ -1391,6 +1697,22 @@ GB_DESC MediaTagListDesc[] =
 };
 
 
+GB_DESC MediaLinkDesc[] =
+{
+	GB_DECLARE("MediaLink", sizeof(CMEDIALINK)),
+	GB_NOT_CREATABLE(),
+
+	GB_METHOD("_free", NULL, MediaLink_free, NULL),
+
+	GB_PROPERTY_READ("Name", "s", MediaLink_Name),
+	GB_PROPERTY_READ("Peer", "MediaControl", MediaLink_Peer),
+	GB_PROPERTY_READ("Input", "s", MediaLink_Input),
+	GB_PROPERTY_READ("Output", "s", MediaLink_Output),
+
+	GB_END_DECLARE
+};
+
+
 GB_DESC MediaControlDesc[] = 
 {
 	GB_DECLARE("MediaControl", sizeof(CMEDIACONTROL)),
@@ -1400,20 +1722,24 @@ GB_DESC MediaControlDesc[] =
 	
 	GB_PROPERTY("Name", "s", MediaControl_Name),
 	GB_PROPERTY_READ("Type", "s", MediaControl_Type),
+	GB_PROPERTY_READ("Parent", "MediaContainer", MediaControl_Parent),
 	GB_PROPERTY("State", "i", MediaControl_State),
 	GB_PROPERTY("Tag", "v", MediaControl_Tag),
 			
 	GB_METHOD("_put", NULL, MediaControl_put, "(Value)v(Property)s"),
 	GB_METHOD("_get", "v", MediaControl_get, "(Property)s"),
 	
-	GB_METHOD("LinkTo", NULL, MediaControl_LinkTo, "(Destination)MediaControl;[(Output)s(Input)s]"),
-	GB_METHOD("LinkLaterTo", NULL, MediaControl_LinkLaterTo, "(Destination)MediaControl;"),
+	GB_METHOD("LinkTo", NULL, MediaControl_LinkTo, "(Target)MediaControl;[(Output)s(Input)s]"),
+	GB_METHOD("LinkLaterTo", NULL, MediaControl_LinkLaterTo, "(Target)MediaControl;"),
 	
 	GB_PROPERTY_READ("Inputs", "String[]", MediaControl_Inputs),
 	GB_PROPERTY_READ("Outputs", "String[]", MediaControl_Outputs),
 	
+	GB_METHOD("GetLink", "MediaLink", MediaControl_GetLink, "(Name)s"),
+
 	GB_METHOD("SetWindow", NULL, MediaControl_SetWindow, "(Control)Control;[(X)i(Y)i(Width)i(Height)i]"),
-	
+	GB_METHOD("GetLastImage", "Image", MediaControl_GetLastImage, NULL),
+
 	GB_EVENT("State", NULL, NULL, &EVENT_State),
 	//GB_EVENT("Signal", NULL, "(Arg)MediaSignalArguments", &EVENT_Signal),
 	
@@ -1427,7 +1753,7 @@ GB_DESC MediaFilterDesc[] =
 	GB_DECLARE("MediaFilter", sizeof(CMEDIACONTROL)),
 	GB_INHERITS("MediaControl"),
 	
-	GB_METHOD("_new", NULL, MediaFilter_new, "[(Filter)s]"),
+	//GB_METHOD("_new", NULL, MediaFilter_new, NULL),
 	
 	GB_PROPERTY("Filter", "s", MediaFilter_Filter),
 
@@ -1465,7 +1791,7 @@ GB_DESC MediaPipelineDesc[] =
 	GB_DECLARE("MediaPipeline", sizeof(CMEDIAPIPELINE)),
 	GB_INHERITS("MediaContainer"),
 	
-	GB_METHOD("_new", NULL, MediaPipeline_new, NULL),
+	GB_METHOD("_new", NULL, MediaPipeline_new, "[(Polling)i]"),
 	GB_METHOD("_free", NULL, MediaPipeline_free, NULL),
 	
 	/*GB_CONSTANT("Null", "i", GST_STATE_NULL),
