@@ -3,7 +3,7 @@
   main.c
 
   (c) 2004 Rob Kudla <pcre-component@kudla.org>
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,7 +33,14 @@
 
 #include "main.h"
 
-GB_INTERFACE GB EXPORT;
+GB_INTERFACE *GB_PTR EXPORT;
+
+void *GB_PCRE_1[] EXPORT = {
+
+  (void *)PCRE_INTERFACE_VERSION,
+  (void *)REGEXP_match,
+  NULL
+  };
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
