@@ -54,10 +54,10 @@ enum { MIME_UNKNOWN, MIME_TEXT, MIME_IMAGE };
 
 static int get_type(const QMimeData *src)
 {
-	if (src->formats().indexOf(QRegExp("text/.*")) >= 0)
-		return MIME_TEXT;
-	else if (src->hasImage())
+	if (src->hasImage())
 		return MIME_IMAGE;
+	else if (src->formats().indexOf(QRegExp("text/.*")) >= 0)
+		return MIME_TEXT;
 	else
 		return MIME_UNKNOWN;
 }
@@ -126,6 +126,7 @@ static void paste(const QMimeData *data, const char *fmt)
 {
 	QString format;
 	QByteArray ba;
+	int type;
 
 	if (fmt)
 		format = fmt;
@@ -138,7 +139,12 @@ static void paste(const QMimeData *data, const char *fmt)
 		return;
 	}
 
-	switch(get_type(data))
+	if (format.startsWith("text/"))
+		type = MIME_TEXT;
+	else
+		type = get_type(data);
+
+	switch(type)
 	{
 		case MIME_TEXT:
 			ba = data->data(format);
@@ -152,7 +158,7 @@ static void paste(const QMimeData *data, const char *fmt)
 			{
 				QImage *image = new QImage();
 				*image = qvariant_cast<QImage>(data->imageData());
-				image->convertToFormat(QImage::Format_ARGB32);
+				image->convertToFormat(QImage::Format_ARGB32_Premultiplied);
 				GB.ReturnObject(CIMAGE_create(image));
 			}
 			break;
