@@ -39,7 +39,7 @@ static uint get_light_foreground()
 
 QColor CCOLOR_light_foreground()
 {
-	return QColor((QRgb)get_light_foreground());
+	return TO_QCOLOR(get_light_foreground());
 }
 
 QColor CCOLOR_make(GB_COLOR color)
@@ -130,9 +130,21 @@ BEGIN_PROPERTY(Color_TooltipForeground)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Color_LinkForeground)
+
+  return_color(QPalette::Link);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(Color_VisitedForeground)
+
+  return_color(QPalette::LinkVisited);
+
+END_PROPERTY
+
 GB_DESC CColorDesc[] =
 {
-  GB_DECLARE("Color", 0), GB_VIRTUAL_CLASS(),
+  GB_DECLARE_STATIC("Color"),
 
   GB_STATIC_PROPERTY("Background", "i", Color_Background),
   GB_STATIC_PROPERTY("SelectedBackground", "i", Color_SelectedBackground),
@@ -147,6 +159,8 @@ GB_DESC CColorDesc[] =
   GB_STATIC_PROPERTY("TextForeground", "i", Color_TextForeground),
   GB_STATIC_PROPERTY("ButtonForeground", "i", Color_ButtonForeground),
   GB_STATIC_PROPERTY("TooltipForeground", "i", Color_TooltipForeground),
+  GB_STATIC_PROPERTY("LinkForeground", "i", Color_LinkForeground),
+  GB_STATIC_PROPERTY("VisitedForeground", "i", Color_VisitedForeground),
 
   GB_END_DECLARE
 };
