@@ -249,7 +249,7 @@ static void resize_container(void *_object, QWidget *cont, int w, int h)
 #define IS_EXPAND(_object) (((CWIDGET *)_object)->flag.expand)
 #define IS_IGNORE(_object) (((CWIDGET *)_object)->flag.ignore)
 #define IS_DESIGN(_object) (CWIDGET_test_flag(_object, WF_DESIGN) && CWIDGET_test_flag(_object, WF_DESIGN_LEADER))
-#define IS_WIDGET_VISIBLE(_widget) (_widget)->isVisible()
+//#define IS_WIDGET_VISIBLE(_widget) (_widget)->isVisible()
 
 //#define CAN_ARRANGE(_object) ((_object) && !CWIDGET_test_flag(_object, WF_DELETED) && (!GB.Is(_object, CLASS_Window) || (((CWINDOW *)_object)->opened)))
 #define CAN_ARRANGE(_object) ((_object) && ((CWIDGET *)(_object))->flag.shown && !CWIDGET_test_flag(_object, WF_DELETED))
@@ -360,6 +360,7 @@ void CCONTAINER_arrange(void *_object)
 }
 
 static int max_w, max_h;
+static int _gms_x, _gms_y, _gms_w, _gms_h;
 
 static void gms_move_widget(QWidget *wid, int x, int y)
 {
@@ -393,24 +394,47 @@ static void gms_move_resize_widget(QWidget *wid, int x, int y, int w, int h)
 #define FUNCTION_NAME get_max_size
 #undef RESIZE_CONTAINER
 #define RESIZE_CONTAINER(_object, _cont, _w, _h) (0)
+#undef GET_WIDGET_CONTENTS
+#define GET_WIDGET_CONTENTS(_widget, _x, _y, _w, _h) \
+	_x = _gms_x; \
+	_y = _gms_y; \
+	_w = _gms_w; \
+	_h = _gms_h;
+
 //#undef IS_WIDGET_VISIBLE
 //#define IS_WIDGET_VISIBLE(_cont) (1)
 #define GET_MAX_SIZE
 
 #include "gb.form.arrangement.h"
 
-void CCONTAINER_get_max_size(void *_object, int *w, int *h)
+void CCONTAINER_get_max_size(void *_object, int xc, int yc, int wc, int hc, int *w, int *h)
 {
-	bool locked = THIS_ARRANGEMENT->locked;
-	THIS_ARRANGEMENT->locked = false;
+	int add;
+	CCONTAINER_ARRANGEMENT *arr = THIS_ARRANGEMENT;
+	bool locked;
+
+	locked = arr->locked;
+	arr->locked = false;
 	
 	max_w = 0;
 	max_h = 0;
+	_gms_x = xc;
+	_gms_y = yc;
+	_gms_w = wc;
+	_gms_h = hc;
 	get_max_size(THIS);
-	*w = max_w + THIS_ARRANGEMENT->padding + (THIS_ARRANGEMENT->margin ? MAIN_scale : 0);
-	*h = max_h + THIS_ARRANGEMENT->padding + (THIS_ARRANGEMENT->margin ? MAIN_scale : 0);
+
+	if (arr->margin)
+		add = arr->padding ? arr->padding : MAIN_scale;
+	else if (!arr->spacing)
+		add = arr->padding;
+	else
+		add = 0;
+
+	*w = max_w + add;
+	*h = max_h + add;
 	
-	THIS_ARRANGEMENT->locked = locked;
+	arr->locked = locked;
 }
 
 
@@ -475,7 +499,7 @@ void CCONTAINER_decide(CWIDGET *control, bool *width, bool *height)
 ***************************************************************************/
 
 MyFrame::MyFrame(QWidget *parent)
-: QWidget(parent),_frame(0),_pixmap(0)
+: QWidget(parent),_frame(0),_pixmap(0),_bg(false)
 {
 }
 
@@ -506,10 +530,10 @@ static void _draw_border(QPainter *p, int frame, QWidget *w, QStyleOptionFrame &
 	bool a;
 	QBrush save_brush;
 	//QRect rect = opt.rect;
-
+	
 	if (frame == BORDER_NONE)
 		return;
-
+	
 	if (w)
 		style = w->style();
 	else
@@ -529,19 +553,19 @@ static void _draw_border(QPainter *p, int frame, QWidget *w, QStyleOptionFrame &
 			if (a)
 				p->setRenderHint(QPainter::Antialiasing, true);
 			break;
-
+			
 		case BORDER_SUNKEN:
 			optv3.rect = opt.rect;
 			optv3.state = opt.state | QStyle::State_Sunken;
 			optv3.frameShape = QFrame::StyledPanel;
-
+			
 			save_brush = p->brush();
 			p->setBrush(QBrush());
 			style->drawPrimitive(QStyle::PE_Frame, &optv3, p, w);
 			p->setBrush(save_brush);
 			//style->drawControl(QStyle::CE_ShapedFrame, &optv3, p, w);
 			break;
-
+			
 		case BORDER_RAISED:
 			optv3.rect = opt.rect;
 			optv3.state = opt.state | QStyle::State_Raised;
@@ -552,7 +576,7 @@ static void _draw_border(QPainter *p, int frame, QWidget *w, QStyleOptionFrame &
 			opt.state |= QStyle::State_Raised;
 			style->drawPrimitive(QStyle::PE_Frame, &opt, p, w);*/
 			break;
-
+			
 		case BORDER_ETCHED:
 			optv3.rect = opt.rect;
 			//optv3.state = opt.state | QStyle::State_Raised;
@@ -560,7 +584,7 @@ static void _draw_border(QPainter *p, int frame, QWidget *w, QStyleOptionFrame &
 			style->drawPrimitive(QStyle::PE_FrameGroupBox, &optv3, p, w);
 			//qDrawShadeRect(p, opt.rect, opt.palette, true, 1, 0);
 			break;
-
+			
 		default:
 			return;
 	}
@@ -579,6 +603,7 @@ void CCONTAINER_draw_border_without_widget(QPainter *p, char frame, QStyleOption
 	_draw_border(p, frame, NULL, opt);
 }
 
+
 int CCONTAINER_get_border_width(char border)
 {
 	switch (border)
@@ -624,15 +649,14 @@ int MyFrame::frameWidth()
 	{
 		case BORDER_PLAIN:
 			return 1;
-		
+
 		case BORDER_SUNKEN:
 		case BORDER_RAISED:
-			
 			return style()->pixelMetric(QStyle::PM_ComboBoxFrameWidth);
-			
+
 		case BORDER_ETCHED:
 			return 2;
-			
+
 		default:
 			return 0;
 	}
@@ -651,10 +675,18 @@ void MyFrame::setPixmap(QPixmap *pixmap)
 void MyFrame::paintEvent(QPaintEvent *e)
 {
 	QPainter painter(this);
+
+	if (_bg)
+	{
+		CWIDGET *window = CWidget::get(parentWidget());
+		GB_COLOR col = CWIDGET_get_background(window);
+		if (col != COLOR_DEFAULT)
+			painter.fillRect(e->rect(), CCOLOR_make(col));
+	}
+
 	if (_pixmap)
 		painter.drawTiledPixmap(0, 0, width(), height(), *_pixmap);
-	//else
-	//	painter.eraseRect(e->rect());
+
 	drawFrame(&painter);
 }
 
@@ -685,11 +717,11 @@ void MyContainer::showEvent(QShowEvent *e)
 	void *_object = CWidget::get(this);
 	QWidget::showEvent(e);
 	THIS->widget.flag.shown = TRUE;
-	// 	if (!qstrcmp(GB.GetClassName(THIS), "TabStrip"))
-	// 	{
-	// 		qDebug("MyContainer::showEvent: %s %p: SHOWN = 1 (%d %d)", THIS->widget.name, THIS, THIS->widget.widget->isVisible() , !THIS->widget.widget->isHidden());
-	// 		BREAKPOINT();
-	// 	}
+	/*if (!qstrcmp(GB.GetClassName(THIS), "HBox"))
+	{
+		qDebug("MyContainer::showEvent: %s %p: shown = 1 (%d %d)", THIS->widget.name, THIS, THIS->widget.widget->isVisible() , !THIS->widget.widget->isHidden());
+		//BREAKPOINT();
+	}*/
 	CCONTAINER_arrange(THIS);
 }
 
@@ -698,9 +730,9 @@ void MyContainer::hideEvent(QHideEvent *e)
 	void *_object = CWidget::get(this);
 	QWidget::hideEvent(e);
 	THIS->widget.flag.shown = FALSE;
-	/*if (!qstrcmp(GB.GetClassName(THIS), "ListContainer"))
+	/*if (!qstrcmp(GB.GetClassName(THIS), "HBox"))
 	{
-		qDebug("MyContainer::hideEvent: %s %p: SHOWN = 0", THIS->widget.name, THIS);
+		qDebug("MyContainer::hideEvent: %s %p: shown = 0", THIS->widget.name, THIS);
 		//BREAKPOINT();
 	}*/
 }
@@ -865,10 +897,14 @@ BEGIN_METHOD_VOID(Container_Children_Clear)
 	QObjectList list;
 	QObject *ob;
 	int i;
+	bool locked;
 
 	if (!wid)
 		return;
 
+	locked = THIS_ARRANGEMENT->locked;
+	THIS_ARRANGEMENT->locked = true;
+
 	list = wid->children();
 
 	for(i = 0; i < list.count(); i++)
@@ -878,6 +914,9 @@ BEGIN_METHOD_VOID(Container_Children_Clear)
 			CWIDGET_destroy(CWidget::getRealExisting(ob));
 	}
 
+	THIS_ARRANGEMENT->locked = locked;
+	CCONTAINER_arrange(THIS);
+
 END_METHOD
 
 
@@ -937,7 +976,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Container_Border)
 
 	MyContainer *w = qobject_cast<MyContainer *>(THIS->container);
-	
+
 	if (!w)
 		return;
 
@@ -951,6 +990,23 @@ BEGIN_PROPERTY(Container_Border)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Container_SimpleBorder)
+
+	MyContainer *w = qobject_cast<MyContainer *>(THIS->container);
+
+	if (!w)
+		return;
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(w->frameStyle());
+	else
+	{
+		w->setFrameStyle(VPROP(GB_BOOLEAN) ? BORDER_PLAIN : BORDER_NONE);
+		arrange_now(CONTAINER);
+	}
+
+END_PROPERTY
+
 BEGIN_PROPERTY(Container_Arrangement)
 
 	if (READ_PROPERTY)
