@@ -30,9 +30,11 @@
 #include <QX11Info>
 #include <QColormap>
 #include <QTimer>
+#include <QEvent>
 
 #include "CDraw.h"
 #include "cpaint_impl.h"
+#include "CColor.h"
 #include "CDrawingArea.h"
 
 #ifndef NO_X_WINDOW
@@ -40,7 +42,12 @@
 #include <X11/Xlib.h>
 #endif
 
+#ifdef FontChange
+#undef FontChange
+#endif
+
 DECLARE_EVENT(EVENT_Draw);
+DECLARE_EVENT(EVENT_Font);
 
 
 /***************************************************************************
@@ -169,7 +176,7 @@ void MyDrawingArea::redraw(QRect &r, bool frame)
 	bg = CWIDGET_get_background((CWIDGET *)THIS); 
 	if (bg != COLOR_DEFAULT)
 	{
-		p->fillRect(fw, fw, width() - fw * 2, height() - fw * 2, QColor((QRgb)bg));
+		p->fillRect(fw, fw, width() - fw * 2, height() - fw * 2, TO_QCOLOR(bg));
 	}
 	
 	PAINT_clip(r.x(), r.y(), r.width(), r.height());
@@ -206,7 +213,7 @@ void MyDrawingArea::createBackground(int w, int h)
 	QX11Info xinfo = x11Info();
 	QPixmap p;
 	Qt::HANDLE old = _background;
-	GC gc;
+	//GC gc;
 	
 	_background = (Qt::HANDLE)XCreatePixmap(QX11Info::display(), RootWindow(QX11Info::display(), xinfo.screen()), w, h, xinfo.depth());
 	_background_pixmap = QPixmap::fromX11Pixmap(_background, QPixmap::ExplicitlyShared);
@@ -217,11 +224,15 @@ void MyDrawingArea::createBackground(int w, int h)
 	//p = QPixmap::fromX11Pixmap(_background, QPixmap::ExplicitlyShared);
 	//qDebug("color = %06X -> %06X", palette().color(backgroundRole()).rgb(), QColormap::instance().pixel((unsigned long)palette().color(backgroundRole()).rgb()));
 	
+#if 0
 	gc = XCreateGC(QX11Info::display(), _background, 0, 0);
 	//XSetForeground(QX11Info::display(), gc, QColormap::instance().pixel((unsigned long)palette().color(backgroundRole()).rgb()));
 	XSetForeground(QX11Info::display(), gc, QColormap::instance().pixel(CWIDGET_get_real_background((CWIDGET *)THIS)));
 	XFillRectangle(QX11Info::display(), _background, gc, 0, 0, w, h);
 	XFreeGC(QX11Info::display(), gc);
+#endif
+
+	_background_pixmap.fill(CCOLOR_make(CWIDGET_get_real_background((CWIDGET *)THIS)));
 	
 	//qDebug("XSetWindowBackgroundPixmap: %08X %08X", (int)winId(), (int)_background);
 	XSetWindowBackgroundPixmap(QX11Info::display(), winId(), _background);
@@ -344,8 +355,16 @@ void MyDrawingArea::refreshBackground()
 
 void MyDrawingArea::clearBackground()
 {
+	if (drawn)
+	{
+		GB.Error("DrawingArea is being painted");
+		return;
+	}
+
 	if (_cached)
+	{
 		createBackground(width(), height());
+	}
 	else
 	{
 		XClearArea(QX11Info::display(), winId(), 0, 0, 0, 0, True);
@@ -469,6 +488,16 @@ void MyDrawingArea::hideEvent(QHideEvent *e)
 	MyContainer::hideEvent(e);
 }
 
+void MyDrawingArea::changeEvent(QEvent *e)
+{
+	if (e->type() == QEvent::FontChange)
+	{
+		MyContainer::changeEvent(e);
+		void *_object = CWidget::get(this);
+		GB.Raise(THIS, EVENT_Font, 0);
+	}
+}
+
 /***************************************************************************
 
 	DrawingArea
@@ -666,6 +695,7 @@ GB_DESC CDrawingAreaDesc[] =
 	GB_METHOD("Refresh", NULL, DrawingArea_Refresh, "[(X)i(Y)i(Width)i(Height)i]"),
 
 	GB_EVENT("Draw", NULL, NULL, &EVENT_Draw),
+	GB_EVENT("Font", NULL, NULL, &EVENT_Font),
 
 	//GB_INTERFACE("Draw", &DRAW_Interface),
 	GB_INTERFACE("Paint", &PAINT_Interface),
@@ -674,7 +704,3 @@ GB_DESC CDrawingAreaDesc[] =
 
 	GB_END_DECLARE
 };
-
-
-
-
