@@ -102,7 +102,7 @@ BEGIN_PROPERTY(Label_AutoResize)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(Label_Margin)
+BEGIN_PROPERTY(Label_Padding)
 
 	if (READ_PROPERTY)
 		GB.ReturnInteger(WIDGET->margin());
@@ -150,6 +150,20 @@ BEGIN_PROPERTY(Label_Wrap)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Label_Border)
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(WIDGET->border());
+	else
+	{
+		WIDGET->setBorder(VPROP(GB_INTEGER));
+		WIDGET->calcMinimumHeight();
+	}
+
+END_PROPERTY
+
+//---------------------------------------------------------------------------
+
 GB_DESC CLabelDesc[] =
 {
   GB_DECLARE("Label", sizeof(CLABEL)), GB_INHERITS("Control"),
@@ -159,9 +173,9 @@ GB_DESC CLabelDesc[] =
   GB_PROPERTY("Text", "s", Label_Text),
   GB_PROPERTY("Caption", "s", Label_Text),
   GB_PROPERTY("Alignment", "i", Label_Alignment),
-  GB_PROPERTY("Border", "i", CWIDGET_border_full),
+  GB_PROPERTY("Border", "i", Label_Border),
   GB_PROPERTY("AutoResize", "b", Label_AutoResize),
-  GB_PROPERTY("Padding", "i", Label_Margin),
+  GB_PROPERTY("Padding", "i", Label_Padding),
   GB_PROPERTY("Transparent", "b", Label_Transparent),
   GB_METHOD("Adjust", NULL, Label_Adjust, NULL),
 
@@ -180,9 +194,9 @@ GB_DESC CTextLabelDesc[] =
   GB_PROPERTY("Text", "s", Label_Text),
   GB_PROPERTY("Caption", "s", Label_Text),
   GB_PROPERTY("Alignment", "i", TextLabel_Alignment),
-  GB_PROPERTY("Border", "i", CWIDGET_border_full),
+  GB_PROPERTY("Border", "i", Label_Border),
   GB_PROPERTY("AutoResize", "b", Label_AutoResize),
-  GB_PROPERTY("Padding", "i", Label_Margin),
+  GB_PROPERTY("Padding", "i", Label_Padding),
   GB_PROPERTY("Transparent", "b", Label_Transparent),
   GB_PROPERTY("Wrap", "b", Label_Wrap),
   GB_METHOD("Adjust", NULL, Label_Adjust, NULL),
@@ -221,6 +235,7 @@ MyLabel::MyLabel(QWidget *parent) : QLabel(parent)
 {
 	autoResize = false;
 	locked = false;
+	_border = BORDER_NONE;
 	setIndent(0);
   calcMinimumHeight();
 }
@@ -384,6 +399,13 @@ void MyLabel::adjust()
 	calcMinimumHeight(true);
 }
 
+void MyLabel::paintEvent(QPaintEvent *e)
+{
+	QPainter p(this);
+	CCONTAINER_draw_border(&p, _border, this);
+	QLabel::paintEvent(e);
+}
+
 /** class MySeparator ******************************************************/
 
 
@@ -400,7 +422,7 @@ void MySeparator::paintEvent( QPaintEvent * )
 	{
 		void *_object = CWidget::getReal(this);
 		uint color = CWIDGET_get_foreground(&THIS->widget);
-		p.setPen(color == COLOR_DEFAULT ? CCOLOR_light_foreground() : QColor((QRgb)color));
+		p.setPen(color == COLOR_DEFAULT ? CCOLOR_light_foreground() : TO_QCOLOR(color));
 		if (width() >= height())
 			p.drawLine(0, height() / 2, width() - 1, height() / 2);
 		else
