@@ -28,6 +28,7 @@
 #include <QMenuBar>
 #include <QMenu>
 #include <QKeyEvent>
+#include <QActionGroup>
 
 #include "gambas.h"
 #include "gb_common.h"
@@ -224,7 +225,7 @@ static void update_accel_recursive(CMENU *_object)
 
 static void update_check(CMENU *_object)
 {
-	if (THIS->checked || THIS->toggle)
+	if (THIS->checked || THIS->toggle || THIS->radio)
 	{
 		ACTION->setCheckable(true);
 		ACTION->setChecked(THIS->checked);
@@ -252,6 +253,58 @@ static void toggle_menu(CMENU *_object)
 }
 #endif
 
+static void update_action_group(QMenu *parent)
+{
+	int i;
+	QAction *action;
+	CMENU *menu;
+	QActionGroup *group = NULL;
+
+	for (i = 0; i < parent->actions().count(); i++)
+	{
+		action = parent->actions().at(i);
+		menu = CMenu::dict[action];
+		if (!menu || menu->deleted)
+			continue;
+
+		if (menu->radio)
+		{
+			if (!group)
+			{
+				if (action->actionGroup())
+					group = action->actionGroup();
+				else
+					group = new QActionGroup(parent);
+			}
+			action->setActionGroup(group);
+		}
+		else
+		{
+			group = NULL;
+			action->setActionGroup(NULL);
+		}
+
+		//qDebug("action '%s' -> %p", TO_UTF8(action->text()), (void *)group);
+	}
+}
+
+//---------------------------------------------------------------------------
+
+MyAction::MyAction(QObject *parent): QAction(parent)
+{
+}
+
+bool MyAction::event(QEvent *e)
+{
+	if (e->type() == QEvent::Shortcut)
+	{
+		activate(Trigger);
+		return true;
+	}
+
+	return QAction::event(e);
+}
+
 //---------------------------------------------------------------------------
 
 BEGIN_METHOD(Menu_new, GB_OBJECT parent; GB_BOOLEAN hidden)
@@ -285,8 +338,9 @@ BEGIN_METHOD(Menu_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 			QObject::connect(menu->menu, SIGNAL(aboutToHide()), &CMenu::manager, SLOT(slotHidden()));
 		}
 
-		action = new QAction(menu->menu);
+		action = new MyAction(menu->menu);
 		action->setSeparator(true);
+		QObject::connect(action, SIGNAL(toggled(bool)), &CMenu::manager, SLOT(slotToggled(bool)));
 		QObject::connect(action, SIGNAL(destroyed()), &CMenu::manager, SLOT(slotDestroyed()));
 
 		menu->menu->addAction(action);
@@ -301,10 +355,11 @@ BEGIN_METHOD(Menu_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 		if (!menuBar)
 		{
 			menuBar = new QMenuBar(topLevel);
+			//menuBar->setAutoFillBackground(true);
 			window->menuBar = menuBar;
 		}
 
-		action = new QAction(menuBar);
+		action = new MyAction(menuBar);
 		menuBar->addAction(action);
 
 		action->setSeparator(true);
@@ -369,6 +424,7 @@ BEGIN_METHOD_VOID(Menu_free)
 	//if (!strcmp(THIS->widget.name, "mnuCut"))
 	//	BREAKPOINT();
 	GB.FreeString(&THIS->widget.name);
+	GB.FreeString(&THIS->save_text);
 
 	#ifdef DEBUG_MENU
 		qDebug("Menu_free: item = %p is freed!", THIS);
@@ -380,7 +436,12 @@ END_METHOD
 BEGIN_PROPERTY(Menu_Text)
 
 	if (READ_PROPERTY)
-		GB.ReturnNewZeroString(TO_UTF8(ACTION->text()));
+	{
+		if (THIS->save_text)
+			GB.ReturnString(THIS->save_text);
+		else
+			GB.ReturnNewZeroString(TO_UTF8(ACTION->text()));
+	}
 	else
 	{
 		QString text = QSTRING_PROP();
@@ -390,6 +451,8 @@ BEGIN_PROPERTY(Menu_Text)
 
 		if (!CMENU_is_toplevel(THIS))
 			((CMENU *)THIS->parent)->init_shortcut = FALSE;
+
+		GB.FreeString(&THIS->save_text);
 	}
 
 END_PROPERTY
@@ -437,7 +500,9 @@ BEGIN_PROPERTY(Menu_Checked)
 	else
 	{
 		if (READ_PROPERTY)
+		{
 			GB.ReturnBoolean(THIS->checked);
+		}
 		else
 		{
 			THIS->checked = VPROP(GB_BOOLEAN);
@@ -461,11 +526,26 @@ BEGIN_PROPERTY(Menu_Toggle)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Menu_Radio)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(THIS->radio);
+	else if (THIS->radio != VPROP(GB_BOOLEAN))
+	{
+		THIS->radio = VPROP(GB_BOOLEAN);
+		if (!CMENU_is_toplevel(THIS))
+			update_action_group(((CMENU *)THIS->parent)->menu);
+		update_check(THIS);
+	}
+
+END_PROPERTY
+
+
 static void send_click_event(CMENU *_object);
 
 BEGIN_PROPERTY(Menu_Value)
 
-	if (THIS->toggle)
+	if (THIS->toggle || THIS->radio)
 	{
 		Menu_Checked(_object, _param);
 		return;
@@ -674,6 +754,15 @@ BEGIN_PROPERTY(Menu_Action)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Menu_SaveText)
+
+	if (READ_PROPERTY)
+		GB.ReturnString(THIS->save_text);
+	else
+		GB.StoreString(PROP(GB_STRING), &THIS->save_text);
+
+END_PROPERTY
+
 /*BEGIN_PROPERTY(CMENU_tear_off)
 
 	if (!THIS->menu)
@@ -711,6 +800,7 @@ GB_DESC CMenuDesc[] =
 	GB_PROPERTY("Name", "s", Control_Name),
 	GB_PROPERTY("Caption", "s", Menu_Text),
 	GB_PROPERTY("Text", "s", Menu_Text),
+	GB_PROPERTY("_Text", "s", Menu_SaveText),
 	GB_PROPERTY("Enabled", "b", Menu_Enabled),
 	GB_PROPERTY("Checked", "b", Menu_Checked),
 	GB_PROPERTY("Tag", "v", Control_Tag),
@@ -718,6 +808,7 @@ GB_DESC CMenuDesc[] =
 	GB_PROPERTY("Shortcut", "s", Menu_Shortcut),
 	GB_PROPERTY("Visible", "b", Menu_Visible),
 	GB_PROPERTY("Toggle", "b", Menu_Toggle),
+	GB_PROPERTY("Radio", "b", Menu_Radio),
 	GB_PROPERTY("Value", "b", Menu_Value),
 	GB_PROPERTY("Action", "s", Menu_Action),
 	GB_PROPERTY_READ("Window", "Window", Menu_Window),
@@ -748,7 +839,7 @@ QHash<QAction *, CMENU *> CMenu::dict;
 
 static void send_click_event(CMENU *_object)
 {
-	if (THIS->toggle)
+	if (THIS->toggle && !THIS->radio)
 	{
 		THIS->checked = !THIS->checked;
 		update_check(THIS);
@@ -909,6 +1000,19 @@ void CMenu::unrefChildren(QWidget *w)
 	}
 }*/
 
+void CMenu::slotToggled(bool checked)
+{
+	CMENU *_object = dict[(QAction *)sender()];
+
+	if (!_object)
+		return;
+
+	if (!THIS->radio)
+		return;
+
+	THIS->checked = checked;
+}
+
 void CMenu::slotDestroyed(void)
 {
 	CMENU *_object = dict[(QAction *)sender()];
