@@ -65,9 +65,10 @@ typedef
 		QKeySequence *accel;
     CPICTURE *picture;
 		char *action;
+		char *save_text;
     unsigned deleted : 1;
     unsigned toggle : 1;
-    //unsigned noshortcut : 1;
+		unsigned radio : 1;
     unsigned exec : 1;
 		unsigned checked : 1;
 		unsigned disabled : 1;
@@ -79,6 +80,18 @@ typedef
 typedef
   QList<CMENU *> CMenuList;
 
+class MyAction : public QAction
+{
+	Q_OBJECT
+
+public:
+
+	MyAction(QObject *parent);
+
+protected:
+
+	virtual bool event(QEvent *);
+};
 
 class CMenu : public QObject
 {
@@ -96,6 +109,7 @@ public:
 public slots:
 
   void slotTriggered(QAction *);
+  void slotToggled(bool);
   void slotDestroyed();
   void slotShown();
   void slotHidden();
