@@ -69,6 +69,22 @@ void MyScrollView::showEvent(QShowEvent *e)
 	QScrollArea::showEvent(e);
 }
 
+void MyScrollView::doUpdateScrollbars()
+{
+	QEvent e(QEvent::LayoutRequest);
+	qApp->sendEvent(this, &e);
+}
+
+int MyScrollView::getScrollbar()
+{
+	int scroll = 0;
+	if (horizontalScrollBarPolicy() == Qt::ScrollBarAsNeeded)
+		scroll += 1;
+	if (verticalScrollBarPolicy() == Qt::ScrollBarAsNeeded)
+		scroll += 2;
+	return scroll;
+}
+
 /***************************************************************************
 
 	class MyContents
@@ -90,6 +106,7 @@ MyContents::MyContents(MyScrollView *scrollview)
 	THIS->widget.flag.shown = shown;
 	timer = false;
 	_mustfind = false;
+	_dirty = false;
 }
 
 void MyContents::autoResize(void)
@@ -97,164 +114,96 @@ void MyContents::autoResize(void)
 	CSCROLLVIEW *_object = (CSCROLLVIEW *)CWidget::get(sw);
 	int w, h;
 	int ww, hh;
-	bool cw, ch;
 	int oldw, oldh;
 	bool locked;
-	int i;
-	int x, y;
+	int sbsize;
 
-	if (!sw->isVisible())
+	if (!sw->isVisible() || !_dirty)
 		return;
-	
+
 	#ifdef DEBUG
 	qDebug("autoResize: (%s %p)", THIS->widget.name, THIS);
 	#endif
-	
+
+	if (sw->getScrollbar())
+		sbsize = qApp->style()->pixelMetric(QStyle::PM_ScrollBarExtent) + QMAX(0, qApp->style()->pixelMetric(QStyle::PM_ScrollView_ScrollBarSpacing));
+	else
+		sbsize = 0;
+
 	locked = THIS->arrangement.locked;
 	THIS->arrangement.locked = true;
 	ww = hh = -1;
 	oldw = width(); oldh = height();
-	
+
 	#ifdef DEBUG
-	qDebug("old = %d %d", oldw, oldh);
+	qDebug("current size = %d x %d", oldw, oldh);
 	#endif
 
-	//bh = sw->horizontalScrollBar()->blockSignals(true);
-	//bv = sw->verticalScrollBar()->blockSignals(true);
-	
-	x = sw->horizontalScrollBar()->value();
-	y = sw->verticalScrollBar()->value();
-	
 	sw->_noscroll = true;
-	sw->_scrollx = x;
-	sw->_scrolly = y;
-	
-	if (THIS->arrangement.mode)
-	{
-		ww = sw->width() - sw->frameWidth() * 2;
-		hh = sw->height() - sw->frameWidth() * 2;
-	
-		//qDebug("autoResize: (%d %d) (%d %d)", sw->viewport()->width(), sw->viewport()->height(), ww, hh);
-		#ifdef DEBUG
-		qDebug("resize first %d %d", ww, hh);
-		#endif
-		resize(ww, hh);
-		//sw->updateScrollBars();
-	}
-	else //if (_mustfind)  // _mustfind is not set to true if we unhide a scrollview child control
-	{
-		findRightBottom();
-		
-		/*w = h = 0;
-		
-		if (right)
-			w = right->x() + right->width();
-		if (bottom)
-			h = bottom->y() + bottom->height();	
-		
-		sw->setHorizontalScrollBarPolicy(sw->horizontalScrollBarPolicy());
-		ww = sw->viewport()->width();
-		hh = sw->viewport()->height(); //visibleHeight();	
+	this->removeEventFilter(sw);
 
-		if (w <= ww && h <= hh)
-		{
-			if (!sw->horizontalScrollBar()->isVisible() && !sw->verticalScrollBar()->isVisible())
-				goto __END;
-		}*/
-	}
+	ww = sw->width() - sw->frameWidth() * 2;
+	hh = sw->height() - sw->frameWidth() * 2;
 
-	//qDebug("autoResize: %s", THIS->widget.name);
-	
-	for(i = 0; i < 3; i++)
+	switch(THIS->arrangement.mode)
 	{
-		if (THIS->arrangement.mode)
-		{
-			//findRightBottom();
-			CCONTAINER_get_max_size(THIS, &w, &h);
-			//resize(w, h);
-			//sw->updateScrollBars();
-		}
-		else
-		{
+		case ARRANGE_NONE:
+
+			findRightBottom();
+
 			w = h = 0;
-			
+
 			if (right)
 				w = right->x() + right->width();
 			if (bottom)
-				h = bottom->y() + bottom->height();	
-		}
-		
-		#ifdef DEBUG
-		qDebug("want size %d %d", w, h);
-		#endif
-		
-		if (ww < 0)
-		{
-			//qDebug("#1: %d", i);
-			sw->setHorizontalScrollBarPolicy(sw->horizontalScrollBarPolicy());
-			ww = sw->viewport()->width();
-			hh = sw->viewport()->height(); //visibleHeight();	
-		}
-		
-		if (w < ww || THIS->arrangement.mode == ARRANGE_VERTICAL || THIS->arrangement.mode == ARRANGE_ROW)
-		{
-			w = ww;
-			cw = true;
-		}
-		else
-			cw = false;
-			
-		if (h < hh || THIS->arrangement.mode == ARRANGE_HORIZONTAL || THIS->arrangement.mode == ARRANGE_COLUMN)
-		{
-			h = hh;
-			ch = true;
-		}
-		else
-			ch = false;
-		
-		if (w != width() || h != height())
-		{
-			#ifdef DEBUG
-			qDebug("resize %d %d", w, h);
-			#endif
-		
-			//if (!qstrcmp(THIS->widget.name, "svwWorkspace"))
-			//	qDebug("resize %d %d", w, h);
-			
-			resize(w, h);
-			
-			//CCONTAINER_arrange(THIS);
-			
-			//qDebug("#2: %d", i);
-			sw->setHorizontalScrollBarPolicy(sw->horizontalScrollBarPolicy());
-			//sw->updateScrollBars();
-					
-			if (cw)
-				w = sw->viewport()->width();
-			if (ch)
-				h = sw->viewport()->height();
-				
-			if (w != width() || h != height())
+				h = bottom->y() + bottom->height();
+
+			if ((w > ww) && sw->horizontalScrollBarPolicy() == Qt::ScrollBarAsNeeded)
+				hh -= sbsize;
+
+			if ((h > hh) && sw->verticalScrollBarPolicy() == Qt::ScrollBarAsNeeded)
+				ww -= sbsize;
+
+			ww = MAX(w, ww);
+			hh = MAX(h, hh);
+
+			break;
+
+		case ARRANGE_VERTICAL:
+		case ARRANGE_ROW:
+
+			CCONTAINER_get_max_size(THIS, 0, 0, ww, 65536, &w, &h);
+			if (h > hh)
 			{
-				#ifdef DEBUG
-				qDebug("resize again %d %d", w, h);
-				#endif
-				
-				//if (!qstrcmp(THIS->widget.name, "svwWorkspace"))
-				//	qDebug("resize again %d %d", w, h);
-			
-				resize(w, h);
-				
-				//THIS->arrangement.locked = locked;
-				//CCONTAINER_arrange(THIS);
-				//CCONTAINER_arrange(THIS);
-				ww = -1;
-				continue;
+				ww -= sbsize;
+				CCONTAINER_get_max_size(THIS, 0, 0, ww, 65536, &w, &hh);
 			}
-		}
-		break;
+
+			break;
+
+		case ARRANGE_HORIZONTAL:
+		case ARRANGE_COLUMN:
+
+			CCONTAINER_get_max_size(THIS, 0, 0, 65536, hh, &w, &h);
+			if (w > ww)
+			{
+				hh -= sbsize;
+				CCONTAINER_get_max_size(THIS, 0, 0, 65536, hh, &ww, &h);
+			}
+
+			break;
+
+		case ARRANGE_FILL:
+
+			break;
 	}
 
+	#ifdef DEBUG
+	qDebug("new size = %d x %d", ww, hh);
+	#endif
+
+	resize(ww, hh);
+
 	THIS->arrangement.locked = locked;
 	if (width() != oldw || height() != oldh)
 	{
@@ -263,36 +212,17 @@ void MyContents::autoResize(void)
 		#endif
 		CCONTAINER_arrange(THIS);
 	}
-	
-	/*if (x != old_x)
-	{
-		sw->horizontalScrollBar()->blockSignals(bh);
-		sw->horizontalScrollBar()->setValue(x);
-	}
-	else
-	{
-		sw->horizontalScrollBar()->setValue(x);
-		sw->horizontalScrollBar()->blockSignals(bh);
-	}
-	
-	if (y != old_y)
-	{
-		sw->verticalScrollBar()->blockSignals(bv);
-		sw->verticalScrollBar()->setValue(y);
-	}
-	else
-	{
-		sw->verticalScrollBar()->setValue(y);
-		sw->verticalScrollBar()->blockSignals(bv);
-	}*/
 
 	sw->_noscroll = false;
-	if (x != sw->horizontalScrollBar()->value())
+	/*if (x != sw->horizontalScrollBar()->value())
 		sw->horizontalScrollBar()->setValue(x);
 	if (y != sw->verticalScrollBar()->value())
-		sw->verticalScrollBar()->setValue(y);
-	
+		sw->verticalScrollBar()->setValue(y);*/
+	this->installEventFilter(sw);
+	sw->doUpdateScrollbars();
+
 	timer = false;
+	_dirty = false;
 
 	#ifdef DEBUG
 	qDebug("autoResize: FIN: (%s %p) viewport: %d %d container: %d %d\n", THIS->widget.name, THIS, sw->viewport()->width(), sw->viewport()->height(), width(), height());
@@ -349,7 +279,7 @@ void MyContents::findRightBottom(void)
 	_mustfind = false;
 
 	#ifdef DEBUG
-	CSCROLLVIEW *_object = (CSCROLLVIEW *)CWidget::get(sw);
+	_object = CWidget::get(sw);
 	if (!qstrcmp(THIS->widget.name, "svwWorkspace"))
 	{
 		CWIDGET *wr = CWidget::get(right);
@@ -445,6 +375,7 @@ void MyContents::childEvent(QChildEvent *e)
 
 void MyContents::checkAutoResizeLater()
 {
+	_dirty = true;
 	if (timer)
 		return;
 	
@@ -455,9 +386,11 @@ void MyContents::checkAutoResizeLater()
 void CSCROLLVIEW_arrange(void *_object)
 {
 	if (THIS->container && THIS->container->isVisible())
-		//THIS->container->checkAutoResizeLater();
-		THIS->container->autoResize();
+	{
+		THIS->container->checkAutoResizeLater();
+		//THIS->container->autoResize();
 		//THIS->container->checkWidget(child->widget);
+	}
 }
 
 /***************************************************************************
@@ -481,7 +414,7 @@ BEGIN_METHOD(CSCROLLVIEW_new, GB_OBJECT parent)
 	CWIDGET_set_visible((CWIDGET *)THIS, true);
 	
 	// Border.Sunken by default
-	wid->setLineWidth(2);
+	wid->setLineWidth(3);
 	wid->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
 
 END_METHOD
@@ -569,15 +502,7 @@ BEGIN_PROPERTY(CSCROLLVIEW_scrollbar)
 	int scroll;
 
 	if (READ_PROPERTY)
-	{
-		scroll = 0;
-		if (WIDGET->horizontalScrollBarPolicy() == Qt::ScrollBarAsNeeded)
-			scroll += 1;
-		if (WIDGET->verticalScrollBarPolicy() == Qt::ScrollBarAsNeeded)
-			scroll += 2;
-
-		GB.ReturnInteger(scroll);
-	}
+		GB.ReturnInteger(WIDGET->getScrollbar());
 	else
 	{
 		scroll = VPROP(GB_INTEGER) & 3;
@@ -587,31 +512,11 @@ BEGIN_PROPERTY(CSCROLLVIEW_scrollbar)
 
 END_PROPERTY
 
-#if 0
-BEGIN_PROPERTY(CSCROLLVIEW_background)
-
-	if (READ_PROPERTY)
-		GB.ReturnInteger(THIS->container->palette().color(QPalette::Window).rgb() & 0xFFFFFF);
-	else
-	{
-		QPalette pal;
-		
-		if (VPROP(GB_INTEGER) != COLOR_DEFAULT)
-		{
-			pal = THIS->container->palette();
-			pal.setColor(QPalette::Window, QColor((QRgb)VPROP(GB_INTEGER)));
-		}
-		THIS->container->setPalette(pal);
-	}
-
-END_PROPERTY
-#endif
-
 BEGIN_PROPERTY(ScrollView_Border)
 
 	CWIDGET_border_simple(_object, _param);
 	if (!READ_PROPERTY)
-		THIS->container->autoResize();
+		THIS->container->checkAutoResizeLater();
 
 END_PROPERTY
 
@@ -687,7 +592,7 @@ void CScrollView::scrolled(void)
 	
 	//qDebug("scrolled: %d %d", WIDGET->horizontalScrollBar()->value(), WIDGET->verticalScrollBar()->value());
 	
-	if (WIDGET->_noscroll)
+	/*if (WIDGET->_noscroll)
 	{
 		//qDebug("no scroll!");
 		if (WIDGET->_scrollx != WIDGET->horizontalScrollBar()->value())
@@ -695,7 +600,7 @@ void CScrollView::scrolled(void)
 		if (WIDGET->_scrolly != WIDGET->verticalScrollBar()->value())
 			WIDGET->verticalScrollBar()->setValue(WIDGET->_scrolly);
 		return;
-	}
+	}*/
 
 	if (WIDGET->_scroll_sent)
 		return;
