@@ -76,7 +76,7 @@ static QImage _default_trayicon;
 MyTrayIcon::MyTrayIcon() : SystemTrayIcon()
 {
 	if (_default_trayicon.isNull())
-		_default_trayicon = QImage(_default_trayicon_data, DEFAULT_TRAYICON_WIDTH, DEFAULT_TRAYICON_HEIGHT, QImage::Format_ARGB32);
+		_default_trayicon = QImage(_default_trayicon_data, DEFAULT_TRAYICON_WIDTH, DEFAULT_TRAYICON_HEIGHT, QImage::Format_ARGB32_Premultiplied);
 
 	_icon = QPixmap(_default_trayicon);
 }
@@ -92,7 +92,8 @@ void MyTrayIcon::setIcon(QPixmap &icon)
 
 void MyTrayIcon::drawContents(QPainter *p)
 {
-	p->drawPixmap((width() - _icon.width()) / 2, (height() - _icon.height()) / 2, _icon);
+	//p->drawPixmap((width() - _icon.width()) / 2, (height() - _icon.height()) / 2, _icon);
+	p->drawPixmap(0, 0, width(), height(), _icon);
 }
 
 
@@ -115,7 +116,7 @@ static void destroy_widget(CTRAYICON *_object)
 {
 	if (WIDGET)
 	{
-		delete WIDGET;
+		WIDGET->deleteLater();
 		THIS->widget = NULL;
 		TRAYICON_count--;
 		MAIN_check_quit();
@@ -196,18 +197,20 @@ static void define_mask(CTRAYICON *_object)
 	WIDGET->setIcon(*p);
 	//WIDGET->setPaletteBackgroundColor(QColor(255, 0, 0));
 	WIDGET->resize(p->width(), p->height());
+	//qDebug("resize: %d %d", p->width(), p->height());
 
 	if (!THIS->icon)
 		delete p;
 	
-	#ifndef NO_X_WINDOW
+	/*#ifndef NO_X_WINDOW
 	// Needed, otherwise the icon does not appear in Gnome or XFCE notification area!
 	XSizeHints hints;
 	hints.flags = PMinSize;
 	hints.min_width = WIDGET->width();
 	hints.min_height = WIDGET->height();
 	XSetWMNormalHints(WIDGET->x11Display(), WIDGET->winId(), &hints);
-	#endif
+	qDebug("set hints: %ld %d %d", WIDGET->winId(), WIDGET->width(), WIDGET->height());
+	#endif*/
 }
 
 static void define_tooltip(CTRAYICON *_object)
@@ -302,8 +305,8 @@ BEGIN_METHOD_VOID(CTRAYICON_show)
 		#else
 		WIDGET->show();
 		#endif
-		define_mask(THIS);
-		define_tooltip(THIS);
+		//define_mask(THIS);
+		//define_tooltip(THIS);
 	}
 
 END_METHOD
