@@ -41,7 +41,6 @@
 #include "CClipboard.h"
 #include "CMenu.h"
 #include "CScrollView.h"
-#include "CProgress.h"
 #include "CDrawingArea.h"
 #include "CTextArea.h"
 
@@ -61,7 +60,6 @@
 #include <QWheelEvent>
 #include <QHash>
 #include <QAbstractScrollArea>
-#include <QProgressBar>
 #include <QAbstractEventDispatcher>
 #include <QListWidget>
 #include <QComboBox>
@@ -75,14 +73,18 @@ static QMap<int, int> _x11_to_qt_keycode;
 #endif
 
 CWIDGET *CWIDGET_active_control = 0;
+CWIDGET *CWIDGET_previous_control = 0;
 static bool _focus_change = false;
 static CWIDGET *_old_active_control = 0;
+int CCONTROL_last_event_type = 0;
 
 static CWIDGET *_hovered = 0;
 static CWIDGET *_official_hovered = 0;
 
 QSet<CWIDGET *> *_enter_leave_set = NULL;
 
+static QT_COLOR_FUNC _after_set_color = NULL;
+
 #define EXT(_ob) (((CWIDGET *)_ob)->ext)
 
 #define ENSURE_EXT(_ob) (EXT(_ob) ? EXT(_ob) : alloc_ext((CWIDGET *)(_ob)))
@@ -495,15 +497,11 @@ void CWIDGET_destroy(CWIDGET *_object)
 		return;
 	}
 
-	//qDebug("CWIDGET_destroy: %p (%p) :%p:%ld", object, object->widget, object->ob.klass, object->ob.ref);
-	//qDebug("CWIDGET_destroy: %s %p", GB.GetClassName(object), object);
+	//qDebug("CWIDGET_destroy: %s %p", GB.GetClassName(THIS), THIS);
 
 	CWIDGET_set_visible(THIS, false);
 	CWIDGET_set_flag(THIS, WF_DELETED);
 
-	if (qobject_cast<QProgressBar *>(WIDGET))
-		CPROGRESS_style_hack(THIS);
-	
 	WIDGET->deleteLater();
 }
 
@@ -557,6 +555,16 @@ static void arrange_parent(CWIDGET *_object)
 	CCONTAINER_arrange(parent);
 }
 
+void CWIDGET_check_visibility(CWIDGET *_object)
+{
+	if (!THIS->flag.resized)
+	{
+		THIS->flag.resized = TRUE;
+		//qDebug("CWIDGET_check_visibility: %s %s %d", GB.GetClassName(THIS), THIS->name, THIS->flag.visible);
+		CWIDGET_set_visible(THIS, THIS->flag.visible);
+	}
+}
+
 static void CWIDGET_after_geometry_change(void *_object, bool arrange)
 {
 	if (arrange)
@@ -567,10 +575,8 @@ static void CWIDGET_after_geometry_change(void *_object, bool arrange)
 			((MyDrawingArea *)((CWIDGET *)_object)->widget)->updateBackground();
 	}
 	
-	if (THIS->flag.ignore)
-		return;
-	
-  arrange_parent(THIS);
+	if (!THIS->flag.ignore)
+		arrange_parent(THIS);
 }
 
 void CWIDGET_move(void *_object, int x, int y)
@@ -624,6 +630,8 @@ void CWIDGET_resize(void *_object, int w, int h)
 	if (w < 0 && h < 0)
 		return;
 
+	CWIDGET_check_visibility(THIS);
+
 	CCONTAINER_decide(THIS, &decide_w, &decide_h);
 
 	if (w < 0 || decide_w)
@@ -692,6 +700,8 @@ void CWIDGET_move_resize(void *_object, int x, int y, int w, int h)
 		win->mustCenter = false;
   }
 
+	CWIDGET_check_visibility(THIS);
+
 	if (wid)
 	{
 		if (w < 0)
@@ -786,6 +796,19 @@ void CWIDGET_check_hovered()
 }
 #endif
 
+bool CWIDGET_is_design(CWIDGET *_object)
+{
+	return CWIDGET_test_flag(THIS, WF_DESIGN) || CWIDGET_test_flag(THIS, WF_DESIGN_LEADER);
+}
+
+//---------------------------------------------------------------------------
+
+BEGIN_METHOD_VOID(Control_new)
+
+	MAIN_CHECK_INIT();
+
+END_METHOD
+
 BEGIN_PROPERTY(Control_X)
 
 	if (READ_PROPERTY)
@@ -843,6 +866,18 @@ BEGIN_PROPERTY(Control_Height)
 
 END_PROPERTY
 
+void *CWIDGET_get_real_font(CWIDGET *_object)
+{
+	if (THIS->font)
+		return CFONT_create(*((CFONT *)THIS->font)->font);
+
+	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
+	if (parent)
+		return CWIDGET_get_real_font(parent);
+	else
+		return CFONT_create(qApp->font());
+}
+
 BEGIN_PROPERTY(Control_Font)
 
 	CFONT *font;
@@ -882,7 +917,7 @@ BEGIN_PROPERTY(Control_Design)
 
 	if (READ_PROPERTY)
 	{
-		GB.ReturnBoolean(CWIDGET_test_flag(_object, WF_DESIGN) || CWIDGET_test_flag(_object, WF_DESIGN_LEADER));
+		GB.ReturnBoolean(CWIDGET_is_design(THIS));
 		return;
 	}
 
@@ -892,7 +927,7 @@ BEGIN_PROPERTY(Control_Design)
 		//CWIDGET_set_flag(THIS, WF_DESIGN);
 	}
 	else if (CWIDGET_test_flag(_object, WF_DESIGN) || CWIDGET_test_flag(_object, WF_DESIGN_LEADER))
-		GB.Error("Cannot reset Design property");
+		GB.Error("Design property cannot be reset");
 
 END_PROPERTY
 
@@ -900,9 +935,9 @@ END_PROPERTY
 BEGIN_PROPERTY(Control_Enabled)
 
 	if (READ_PROPERTY)
-		GB.ReturnBoolean(QWIDGET(_object)->isEnabled());
+		GB.ReturnBoolean(WIDGET->isEnabled());
 	else
-		QWIDGET(_object)->setEnabled(VPROP(GB_BOOLEAN));
+		WIDGET->setEnabled(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
@@ -928,11 +963,11 @@ BEGIN_PROPERTY(Control_Expand)
 
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(THIS->flag.expand);
-	else
+	else if (THIS->flag.expand != VPROP(GB_BOOLEAN))
 	{
 		THIS->flag.expand = VPROP(GB_BOOLEAN);
+		CWIDGET_check_visibility(THIS);
 		arrange_parent(THIS);
-		//qApp->postEvent(WIDGET, new QEvent(EVENT_EXPAND));
 	}
 
 END_PROPERTY
@@ -942,11 +977,10 @@ BEGIN_PROPERTY(Control_Ignore)
 
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(THIS->flag.ignore);
-	else
+	else if (THIS->flag.ignore != VPROP(GB_BOOLEAN))
 	{
 		THIS->flag.ignore = VPROP(GB_BOOLEAN);
 		arrange_parent(THIS);
-		//qApp->postEvent(WIDGET, new QEvent(EVENT_EXPAND));
 	}
 
 END_PROPERTY
@@ -987,8 +1021,8 @@ BEGIN_METHOD(Control_ResizeScaled, GB_FLOAT w; GB_FLOAT h)
 
 	int w, h;
 
-	w = (int)(VARG(w) * MAIN_scale);
-	h = (int)(VARG(h) * MAIN_scale);
+	w = (int)(VARG(w) * MAIN_scale + 0.5);
+	h = (int)(VARG(h) * MAIN_scale + 0.5);
 	
 	if (w == 0) w = 1;
 	if (h == 0) h = 1;
@@ -1013,6 +1047,10 @@ void CWIDGET_set_visible(CWIDGET *_object, bool v)
 	bool arrange = false;
 	
 	THIS->flag.visible = v;
+
+	if (!THIS->flag.resized)
+		return;
+
 	if (THIS->flag.visible)
 	{
 		arrange = !WIDGET->isVisible();
@@ -1202,6 +1240,7 @@ static void set_focus(void *_object)
 
 	if (win->opened && QWIDGET(win)->isVisible())
 	{
+		//qDebug("set focus on %s for %s", THIS->name, ((CWIDGET *)win)->name);
 		WIDGET->setFocus();
 	}
 	else if ((CWIDGET *)win != THIS)
@@ -1322,94 +1361,6 @@ BEGIN_PROPERTY(Control_NoTabFocus)
 
 END_PROPERTY
 
-#if 0
-
-/*
-static QColor get_background(CWIDGET *_object, QWidget *wid)
-{
-	QPalette pal(wid->palette());
-	QColorGroup::ColorRole role = (QColorGroup::ColorRole)OBJECT(CWIDGET)->background;
-
-	return pal.color(QPalette::Active, role);
-}
-
-static void test_color(CWIDGET *_object, QWidget *wid)
-{
-	QColor b, f, bp, fp;
-
-	if (!wid->ownPalette() || !wid->parentWidget())
-		return;
-
-	f = wid->paletteForegroundColor();
-	fp = wid->parentWidget()->paletteForegroundColor();
-
-	if (f != fp)
-		return;
-
-	b = get_background(_object, wid);
-	bp = get_background(CWidget::get(wid->parentWidget()), wid->parentWidget());
-
-	if (b != bp)
-		return;
-
-	wid->unsetPalette();
-}
-*/
-
-BEGIN_PROPERTY(CWIDGET_background)
-
-	QWidget *wid = QWIDGET(_object);
-	QPalette pal(wid->palette());
-	QColorGroup::ColorRole role = (QColorGroup::ColorRole)OBJECT(CWIDGET)->background;
-
-	//qDebug("bm = %d (%d %d)", wid->backgroundMode(), QWidget::PaletteButton, QWidget::PaletteBase);
-
-	if (READ_PROPERTY)
-		GB.ReturnInteger(pal.color(QPalette::Active, role).rgb() & 0xFFFFFF);
-	else
-	{
-		pal.setColor(role, QColor((QRgb)VPROP(GB_INTEGER)));
-		wid->setPalette(pal);
-		//test_color((CWIDGET *)_object, wid);
-	}
-
-#if 0
-	if (READ_PROPERTY)
-		GB.ReturnInteger((int)(WIDGET->paletteBackgroundColor().rgb() & 0xFFFFFF));
-	else
-		WIDGET->setPaletteBackgroundColor(QColor((QRgb)PROPERTY(int)));
-#endif
-
-END_PROPERTY
-
-
-BEGIN_PROPERTY(CWIDGET_foreground)
-
-	QWidget *wid;
-
-	if (GB.Is(_object, GB.FindClass("Container")))
-		wid = ((CCONTAINER *)_object)->container;
-	else
-		wid = QWIDGET(_object);
-
-	if (READ_PROPERTY)
-	{
-		GB.ReturnInteger((int)(wid->paletteForegroundColor().rgb() & 0xFFFFFF));
-		return;
-	}
-	else
-	{
-		QPalette pal(QWIDGET(_object)->palette());
-
-		pal.setColor(QColorGroup::Foreground, QColor((QRgb)VPROP(GB_INTEGER)));
-		pal.setColor(QColorGroup::Text, QColor((QRgb)VPROP(GB_INTEGER)));
-		wid->setPalette(pal);
-		//test_color((CWIDGET *)_object, wid);
-	}
-
-END_PROPERTY
-#endif
-
 
 static QWidget *get_color_widget(CWIDGET *_object)
 {
@@ -1423,6 +1374,13 @@ static QWidget *get_color_widget(CWIDGET *_object)
 	return WIDGET;
 }
 
+QT_COLOR_FUNC CWIDGET_after_set_color(QT_COLOR_FUNC func)
+{
+	QT_COLOR_FUNC old = _after_set_color;
+	_after_set_color = func;
+	return old;
+}
+
 
 void CWIDGET_reset_color(CWIDGET *_object)
 {
@@ -1454,17 +1412,17 @@ void CWIDGET_reset_color(CWIDGET *_object)
 			if (bg != COLOR_DEFAULT)
 			{
 				if (cb->isEditable())
-					palette.setColor(QPalette::Base, QColor((QRgb)bg));
+					palette.setColor(QPalette::Base, TO_QCOLOR(bg));
 				else
-					palette.setColor(QPalette::Button, QColor((QRgb)bg));
+					palette.setColor(QPalette::Button, TO_QCOLOR(bg));
 			}
 
 			if (fg != COLOR_DEFAULT)
 			{
 				if (cb->isEditable())
-					palette.setColor(QPalette::Text, QColor((QRgb)fg));
+					palette.setColor(QPalette::Text, TO_QCOLOR(fg));
 				else
-					palette.setColor(QPalette::ButtonText, QColor((QRgb)fg));
+					palette.setColor(QPalette::ButtonText, TO_QCOLOR(fg));
 			}
 
 			w->setPalette(palette);
@@ -1475,12 +1433,12 @@ void CWIDGET_reset_color(CWIDGET *_object)
 
 			if (bg != COLOR_DEFAULT)
 			{
-				palette.setColor(QPalette::Base, QColor((QRgb)bg));
+				palette.setColor(QPalette::Base, TO_QCOLOR(bg));
 			}
 
 			if (fg != COLOR_DEFAULT)
 			{
-				palette.setColor(QPalette::Text, QColor((QRgb)fg));
+				palette.setColor(QPalette::Text, TO_QCOLOR(fg));
 			}
 
 			w->setPalette(palette);
@@ -1490,14 +1448,13 @@ void CWIDGET_reset_color(CWIDGET *_object)
 			palette = QPalette();
 		
 			if (bg != COLOR_DEFAULT)
-				palette.setColor(w->backgroundRole(), QColor((QRgb)bg));
+				palette.setColor(w->backgroundRole(), TO_QCOLOR(bg));
 			
 			if (fg != COLOR_DEFAULT)
-				palette.setColor(w->foregroundRole(), QColor((QRgb)fg));
+				palette.setColor(w->foregroundRole(), TO_QCOLOR(fg));
 		
-			w->setPalette(palette);
-
 			w->setAutoFillBackground(!THIS->flag.noBackground && (THIS->flag.fillBackground || ((THIS_EXT && THIS_EXT->bg != COLOR_DEFAULT) && w->backgroundRole() == QPalette::Window)));
+			w->setPalette(palette);
 		}
 
 	}
@@ -1507,6 +1464,9 @@ void CWIDGET_reset_color(CWIDGET *_object)
 	if (GB.Is(THIS, CLASS_TextArea))
 		CTEXTAREA_set_foreground(THIS);
 	
+	if (_after_set_color)
+		(*_after_set_color)(THIS);
+
 	if (!GB.Is(THIS, CLASS_Container))
 		return;
 	
@@ -1535,7 +1495,7 @@ GB_COLOR CWIDGET_get_background(CWIDGET *_object, bool handle_proxy)
 }
 
 
-int CWIDGET_get_real_background(CWIDGET *_object)
+GB_COLOR CWIDGET_get_real_background(CWIDGET *_object)
 {
 	GB_COLOR bg = CWIDGET_get_background(THIS);
 	
@@ -1559,7 +1519,7 @@ GB_COLOR CWIDGET_get_foreground(CWIDGET *_object, bool handle_proxy)
 }
 
 
-/*int CWIDGET_get_real_foreground(CWIDGET *_object, bool return_default)
+GB_COLOR CWIDGET_get_real_foreground(CWIDGET *_object)
 {
 	GB_COLOR fg = CWIDGET_get_foreground(THIS);
 	
@@ -1569,12 +1529,10 @@ GB_COLOR CWIDGET_get_foreground(CWIDGET *_object, bool handle_proxy)
 	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
 	
 	if (parent)
-		return CWIDGET_get_real_foreground(parent, return_default);
-	else if (return_default)
-		return QApplication::palette().color(QPalette::WindowText).rgb() & 0xFFFFFF;
+		return CWIDGET_get_real_foreground(parent);
 	else
-		return COLOR_DEFAULT;
-}*/
+		return QApplication::palette().color(QPalette::WindowText).rgb() & 0xFFFFFF;
+}
 
 
 BEGIN_PROPERTY(Control_Background)
@@ -1584,7 +1542,12 @@ BEGIN_PROPERTY(Control_Background)
 		if (READ_PROPERTY)
 			GB.GetProperty(THIS_EXT->proxy, "Background");
 		else
-			GB.SetProperty(THIS_EXT->proxy, "Background", GB_T_INTEGER, VPROP(GB_INTEGER));
+		{
+			GB_VALUE value;
+			value.type = GB_T_INTEGER;
+			value._integer.value = VPROP(GB_INTEGER);
+			GB.SetProperty(THIS_EXT->proxy, "Background", &value);
+		}
 		
 		return;
 	}
@@ -1608,8 +1571,13 @@ BEGIN_PROPERTY(Control_Foreground)
 		if (READ_PROPERTY)
 			GB.GetProperty(THIS_EXT->proxy, "Foreground");
 		else
-			GB.SetProperty(THIS_EXT->proxy, "Foreground", GB_T_INTEGER, VPROP(GB_INTEGER));
-		
+		{
+			GB_VALUE value;
+			value.type = GB_T_INTEGER;
+			value._integer.value = VPROP(GB_INTEGER);
+			GB.SetProperty(THIS_EXT->proxy, "Foreground", &value);
+		}
+
 		return;
 	}
 
@@ -1838,13 +1806,13 @@ BEGIN_PROPERTY(CWIDGET_border_full)
 
 	if (READ_PROPERTY)
 	{
-		if (wid->frameStyle() == (QFrame::Box + QFrame::Plain))
+		if (wid->frameStyle() == (QFrame::Box + QFrame::Plain) && wid->lineWidth() == 1)
 			border = BORDER_PLAIN;
 		else if (wid->frameStyle() == (QFrame::StyledPanel + QFrame::Sunken))
 			border = BORDER_SUNKEN;
 		else if (wid->frameStyle() == (QFrame::StyledPanel + QFrame::Raised))
 			border = BORDER_RAISED;
-		else if (wid->frameStyle() == (QFrame::Box + QFrame::Sunken))
+		else if (wid->frameStyle() == (QFrame::StyledPanel + QFrame::Plain))
 			border = BORDER_ETCHED;
 		else
 			border = BORDER_NONE;
@@ -1853,15 +1821,13 @@ BEGIN_PROPERTY(CWIDGET_border_full)
 	}
 	else
 	{
-		lw = 1;
-
 		switch (VPROP(GB_INTEGER))
 		{
-			case BORDER_PLAIN: border = QFrame::Box + QFrame::Plain; break;
+			case BORDER_PLAIN: border = QFrame::Box + QFrame::Plain; lw = 1; break;
 			case BORDER_SUNKEN: border = QFrame::StyledPanel + QFrame::Sunken; lw = 2; break;
 			case BORDER_RAISED: border = QFrame::StyledPanel + QFrame::Raised; lw = 2; break;
-			case BORDER_ETCHED: border = QFrame::Box + QFrame::Sunken; break;
-			default: border = QFrame::NoFrame; break;
+			case BORDER_ETCHED: border = QFrame::StyledPanel + QFrame::Plain; lw = 2; break;
+			default: border = QFrame::NoFrame; lw = 0; break;
 		}
 
 		wid->setFrameStyle(border);
@@ -2273,6 +2239,8 @@ void CWidget::setName(CWIDGET *object, const char *name)
 }
 #endif
 
+#define CLEAN_POINTER(_ptr) if ((_ptr) == THIS) _ptr = NULL
+
 void CWidget::destroy()
 {
 	QWidget *w = (QWidget *)sender();
@@ -2282,7 +2250,7 @@ void CWidget::destroy()
 		return;
 
 	//qDebug("CWidget::destroy: (%s %p) %s [%p]", GB.GetClassName(THIS), THIS, THIS->name, _hovered);
-	
+
 	if (!_post_check_hovered)
 	{
 		CWIDGET *top = (CWIDGET *)CWidget::getTopLevel(THIS);
@@ -2294,21 +2262,13 @@ void CWidget::destroy()
 		}
 	}
 	
-	if (_hovered == THIS)
-		_hovered = NULL;
-	
-	if (_official_hovered == THIS)
-		_official_hovered = NULL;
-	
-	if (_post_check_hovered_window == THIS)
-		_post_check_hovered_window = NULL;
-	
-	if (CWIDGET_active_control == THIS)
-		CWIDGET_active_control = NULL;
-	
-	if (_old_active_control == THIS)
-		_old_active_control = NULL;
-	
+	CLEAN_POINTER(_hovered);
+	CLEAN_POINTER(_official_hovered);
+	CLEAN_POINTER(_post_check_hovered_window);
+	CLEAN_POINTER(CWIDGET_active_control);
+	CLEAN_POINTER(CWIDGET_previous_control);
+	CLEAN_POINTER(_old_active_control);
+
 	if (THIS_EXT)
 	{
 		CACTION_register(THIS, THIS_EXT->action, NULL);
@@ -2371,7 +2331,7 @@ static void post_focus_change(void *)
 		}
 		
 		_old_active_control = current;
-		CWINDOW_activate(_old_active_control);
+		CWINDOW_activate(current);
 		
 		control = current;
 		while (control)
@@ -2398,7 +2358,8 @@ void CWIDGET_handle_focus(CWIDGET *control, bool on)
 	if (on == (CWIDGET_active_control == control))
 		return;
 	
-	//qDebug("CWIDGET_handle_focus: %p %s %d", control, control->name, on);
+	if (CWIDGET_active_control && !_focus_change)
+		CWIDGET_previous_control = CWIDGET_active_control;
 	CWIDGET_active_control = on ? control : NULL;
 	handle_focus_change();
 }
@@ -2431,6 +2392,8 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	void *jump;
 	bool parent_got_it;
 
+	CCONTROL_last_event_type = type;
+
 	//if (widget->isA("MyMainWindow"))
 	//	getDesignDebug(widget);
 	switch (type)
@@ -2558,15 +2521,14 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		//qDebug("Menu event! %p %d", control, EVENT_Menu);
 		if (GB.CanRaise(control, EVENT_Menu))
 		{
-			static int old = MENU_popup_count;
+			int old = MENU_popup_count;
 
 			((QContextMenuEvent *)event)->accept();
 
-			GB.Raise(control, EVENT_Menu, 0);
-			if (MENU_popup_count != old)
+			if (GB.Raise(control, EVENT_Menu, 0) || MENU_popup_count != old)
 				return true;
-			else
-				goto __NEXT;
+			//else
+			//	goto __NEXT;
 		}
 
 		if (EXT(control) && EXT(control)->popup)
@@ -2713,13 +2675,13 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		GB.Unref(POINTER(&control));
 		
 		if (!control)
-			return true;
-		
+			goto __MOUSE_RETURN_TRUE;
+
 		if (control->flag.grab && event_id == EVENT_MouseUp)
 			MyApplication::eventLoop->exit();
 		
 		if (cancel)
-			return true;
+			goto __MOUSE_RETURN_TRUE;
 		
 		if (EXT(control) && EXT(control)->proxy_for)
 		{
@@ -2727,7 +2689,13 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			goto __MOUSE_TRY_PROXY;
 		}
 		
+		CMOUSE_reset_translate();
 		goto __NEXT;
+
+	__MOUSE_RETURN_TRUE:
+
+		CMOUSE_reset_translate();
+		return true;
 	}
 	
 	__TABLET:
@@ -2835,6 +2803,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			goto __TABLET_TRY_PROXY;
 		}
 		
+		CMOUSE_reset_translate();
 		return false; // We fill the information, and then expect Qt to generate a Mouse event from the Tablet event
 	}
 	
@@ -2846,7 +2815,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		if (!real || !original)
 			goto _DESIGN;
 		#endif
-			
+
 		if (control->flag.no_keyboard)
 			goto _DESIGN;
 
@@ -3172,7 +3141,7 @@ void CWIDGET_iconset(QIcon &icon, const QPixmap &pixmap, int size)
 	
 	if (size > 0)
 	{
-		img = pixmap.toImage().convertToFormat(QImage::Format_ARGB32);
+		img = pixmap.toImage().convertToFormat(QImage::Format_ARGB32_Premultiplied);
 		size = ((size + 1) & ~3);
 		img = img.scaled(size, size, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
 		normal = QPixmap::fromImage(img);
@@ -3195,6 +3164,7 @@ GB_DESC CControlDesc[] =
 
 	GB_HOOK_CHECK(CWIDGET_check),
 
+	GB_METHOD("_new", NULL, Control_new, NULL),
 	GB_METHOD("_free", NULL, Control_Delete, NULL),
 
 	GB_METHOD("Move", NULL, Control_Move, "(X)i(Y)i[(Width)i(Height)i]"),
