@@ -75,7 +75,8 @@ typedef
 			unsigned tablet_pressed : 1;
 			unsigned has_action : 1;
 			unsigned drop : 1;
-			unsigned _reserved : 5;
+			unsigned resized : 1;
+			unsigned _reserved : 4;
 			} flag;
 		int level;
 		char *name;
@@ -111,6 +112,9 @@ enum {
 
 extern GB_DESC CControlDesc[];
 extern CWIDGET *CWIDGET_active_control;
+extern CWIDGET *CWIDGET_previous_control;
+
+extern int CCONTROL_last_event_type;
 
 #else
 
@@ -129,10 +133,12 @@ DECLARE_PROPERTY(Control_Tag);
 DECLARE_PROPERTY(CWIDGET_border_full);
 DECLARE_PROPERTY(CWIDGET_border_simple);
 DECLARE_PROPERTY(CWIDGET_scrollbar);
+DECLARE_PROPERTY(Control_Background);
 DECLARE_PROPERTY(Control_Design);
 DECLARE_PROPERTY(Control_Enabled);
 DECLARE_PROPERTY(Control_Font);
 DECLARE_PROPERTY(Control_Action);
+DECLARE_PROPERTY(Control_Mouse);
 
 #define CWIDGET_set_flag(wid, _f) (((CWIDGET *)wid)->flag.f |= _f)
 #define CWIDGET_clear_flag(wid, _f) (((CWIDGET *)wid)->flag.f &= ~_f)
@@ -182,14 +188,19 @@ void CWIDGET_init_name(CWIDGET *_object);
 void CWIDGET_set_name(CWIDGET *_object, const char *name);
 int CWIDGET_check(void *object);
 QString CWIDGET_Utf8ToQString(GB_STRING *str);
-void CWIDGET_destroy(CWIDGET *);
+void CWIDGET_destroy(CWIDGET *_object);
 void CWIDGET_update_design(CWIDGET *_object);
 void CWIDGET_iconset(QIcon &icon, const QPixmap &p, int size = 0);
 void CWIDGET_set_color(CWIDGET *_object, int bg, int fg, bool handle_proxy = false);
 void CWIDGET_reset_color(CWIDGET *_object);
+QT_COLOR_FUNC CWIDGET_after_set_color(QT_COLOR_FUNC func);
 GB_COLOR CWIDGET_get_background(CWIDGET *_object, bool handle_proxy = false);
 GB_COLOR CWIDGET_get_foreground(CWIDGET *_object, bool handle_proxy = false);
-int CWIDGET_get_real_background(CWIDGET *_object);
+
+GB_COLOR CWIDGET_get_real_background(CWIDGET *_object);
+GB_COLOR CWIDGET_get_real_foreground(CWIDGET *_object);
+void *CWIDGET_get_real_font(CWIDGET *_object);
+
 void *CWIDGET_get_parent(void *_object);
 int CWIDGET_get_handle(void *_object);
 bool CWIDGET_is_visible(void *_object);
@@ -210,6 +221,9 @@ void CACTION_register(void *control, const char *old, const char *key);
 void CACTION_raise(void *control);
 bool CWIDGET_get_allow_focus(void *_object);
 void CWIDGET_set_allow_focus(void *_object, bool f);
+bool CWIDGET_is_design(CWIDGET *_object);
+void CWIDGET_check_visibility(CWIDGET *_object);
+
 
 #ifndef DO_NOT_DECLARE_EVENTS
 #ifndef __CWIDGET_CPP
