@@ -86,6 +86,7 @@ typedef
 		unsigned showMenuBar : 1;
 		unsigned sticky : 1;
 		unsigned mustCenter : 1;
+		unsigned noTakeFocus : 1;
 		}
 	CWINDOW;
 
@@ -106,6 +107,7 @@ extern GB_DESC CWindowsDesc[];
 extern GB_DESC CFormDesc[];
 
 extern CWINDOW *CWINDOW_Main;
+extern int CWINDOW_MainDesktop;
 extern CWINDOW *CWINDOW_Current;
 extern CWINDOW *CWINDOW_Active;
 extern CWINDOW *CWINDOW_LastActive;
@@ -254,8 +256,8 @@ void CWINDOW_set_cancel_button(CWINDOW *win, QPushButton *button, bool on);
 void CWINDOW_define_mask(CWINDOW *_object);
 void CWINDOW_ensure_active_window();
 bool CWINDOW_must_quit();
-bool CWINDOW_close_all();
-void CWINDOW_delete_all();
+bool CWINDOW_close_all(bool main);
+void CWINDOW_delete_all(bool main);
 //void CWINDOW_fix_menubar(CWINDOW *window);
 
 #endif
