@@ -56,7 +56,7 @@
 #include "csvgimage.h"
 #include "cpaint_impl.h"
 
-class ClipInfo
+/*class ClipInfo
 {
 public:
 	QPainterPath *path;
@@ -64,7 +64,7 @@ public:
 	
 	ClipInfo() { path = NULL; rect = NULL; }
 	~ClipInfo() { delete path; delete rect; }
-};
+};*/
 
 typedef
 	struct {
@@ -73,9 +73,9 @@ typedef
 		int fillRule;
 		QTransform *init;
 		float bx, by;
-		QPainterPath *clip;
-		GB_RECT *clipRect;
-		QList<ClipInfo *> *clipStack;
+		//QPainterPath *clip;
+		//GB_RECT *clipRect;
+		//QList<ClipInfo *> *clipStack;
 	}
 	QT_PAINT_EXTRA;
 	
@@ -86,7 +86,7 @@ typedef
 
 #define PAINTER(d) EXTRA(d)->painter
 #define PATH(d) EXTRA(d)->path
-#define CLIP(d) EXTRA(d)->clip
+//#define CLIP(d) EXTRA(d)->clip
 
 static inline qreal to_deg(float angle)
 {
@@ -97,8 +97,8 @@ static bool init_painting(GB_PAINT *d, QPaintDevice *device)
 {
 	QPen pen;
 	
-	d->width = device->width();
-	d->height = device->height();
+	d->area.width = device->width();
+	d->area.height = device->height();
 	d->resolutionX = device->physicalDpiX();
 	d->resolutionY = device->physicalDpiY();
 	
@@ -151,7 +151,7 @@ static QColor get_color(GB_PAINT *d, GB_COLOR col)
 	return CCOLOR_make(col);
 }
 
-static void begin_clipping(GB_PAINT *d)
+/*static void begin_clipping(GB_PAINT *d)
 {
 	if (CLIP(d))
 	{
@@ -171,7 +171,10 @@ static void end_clipping(GB_PAINT *d)
 {
 	if (CLIP(d))
 		PAINTER(d)->setClipping(false);
-}
+}*/
+
+#define begin_clipping(_d)
+#define end_clipping(_d)
 
 //---------------------------------------------------------------------------
 
@@ -233,8 +236,8 @@ static int Begin(GB_PAINT *d)
 		if (wid->isCached())
 			PAINTER(d)->initFrom(wid);
 		
-		d->width = wid->width();
-		d->height = wid->height();
+		d->area.width = wid->width();
+		d->area.height = wid->height();
 		return FALSE;
 	}
 	else if (GB.Is(device, CLASS_Printer))
@@ -287,27 +290,27 @@ static void End(GB_PAINT *d)
 		PAINTER(d)->end(); // ??
 	}
 
-	if (dx->clipStack)
+	/*if (dx->clipStack)
 	{
 		while (!dx->clipStack->isEmpty())
 			delete dx->clipStack->takeLast();
 		delete dx->clipStack;
-	}
+	}*/
 	
 	delete dx->init;
 	delete dx->path;
-	delete dx->clip;
+	//delete dx->clip;
 	delete dx->painter;
 }
 
 static void Save(GB_PAINT *d)
 {
-	QT_PAINT_EXTRA *dx = EXTRA(d);
-	ClipInfo *ci;
+	//QT_PAINT_EXTRA *dx = EXTRA(d);
+	//ClipInfo *ci;
 	
 	PAINTER(d)->save();
 	
-	if (!dx->clipStack)
+	/*if (!dx->clipStack)
 		dx->clipStack = new QList<ClipInfo *>;
 	
 	ci = new ClipInfo;
@@ -319,16 +322,16 @@ static void Save(GB_PAINT *d)
 		*ci->rect = *dx->clipRect;
 	}
 	
-	dx->clipStack->append(ci);
+	dx->clipStack->append(ci);*/
 }
 
 static void Restore(GB_PAINT *d)
 {
-	QT_PAINT_EXTRA *dx = EXTRA(d);
+	//QT_PAINT_EXTRA *dx = EXTRA(d);
 	
 	PAINTER(d)->restore();
 	
-	if (dx->clipStack && !dx->clipStack->isEmpty())
+	/*if (dx->clipStack && !dx->clipStack->isEmpty())
 	{
 		ClipInfo *ci = dx->clipStack->takeLast();
 		
@@ -345,7 +348,7 @@ static void Restore(GB_PAINT *d)
 			dx->clipRect = NULL;
 		
 		delete ci;
-	}
+	}*/
 }
 		
 static void Antialias(GB_PAINT *d, int set, int *antialias)
@@ -366,7 +369,17 @@ static void apply_font(QFont &font, void *object = 0)
 static void Font(GB_PAINT *d, int set, GB_FONT *font)
 {
 	if (set)
-		PAINTER(d)->setFont(*((CFONT *)(*font))->font);
+	{
+		QFont f(*((CFONT *)(*font))->font);
+		PAINTER(d)->setFont(f);
+
+		// Strange bug of QT. Sometimes the font does not apply (cf. DrawTextShadow)
+		if (f != PAINTER(d)->font())
+		{
+			f.fromString(f.toString());
+			PAINTER(d)->setFont(f);
+		}
+	}
 	else
 		*font = CFONT_create(PAINTER(d)->font(), apply_font);
 }
@@ -401,36 +414,39 @@ static void init_path(GB_PAINT *d)
 	if (!PATH(_d)) \
 		EXTRA(_d)->path = new QPainterPath();
 
-static void delete_clip_rect(GB_PAINT *d)
+/*static void delete_clip_rect(GB_PAINT *d)
 {
 	if (EXTRA(d)->clipRect)
 	{
 		delete EXTRA(d)->clipRect;
 		EXTRA(d)->clipRect = NULL;
 	}
-}
+}*/
 	
 static void Clip(GB_PAINT *d, int preserve)
 {
 	CHECK_PATH(d);
+
+	PAINTER(d)->setClipPath(*PATH(d), PAINTER(d)->hasClipping() ? Qt::IntersectClip : Qt::ReplaceClip);
 	
-	QPainterPath path = PAINTER(d)->worldTransform().map(*PATH(d));
+	/*QPainterPath path = PAINTER(d)->worldTransform().map(*PATH(d));
 	
 	if (CLIP(d))
 		path = CLIP(d)->intersected(path);
 	
 	delete EXTRA(d)->clip;
 	EXTRA(d)->clip = new QPainterPath(path);
-	delete_clip_rect(d);
+	delete_clip_rect(d);*/
 	
 	PRESERVE_PATH(d, preserve);
 }
 
 static void ResetClip(GB_PAINT *d)
 {
-	delete CLIP(d);
+	/*delete CLIP(d);
 	EXTRA(d)->clip = 0;
-	delete_clip_rect(d);
+	delete_clip_rect(d);*/
+	PAINTER(d)->setClipping(false);
 }
 
 static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext, const QTransform &transform)
@@ -451,7 +467,7 @@ static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext, const QTransfo
 
 static void ClipExtents(GB_PAINT *d, GB_EXTENTS *ext)
 {
-	GB_RECT *rect = EXTRA(d)->clipRect;
+	/*GB_RECT *rect = EXTRA(d)->clipRect;
 	if (rect)
 	{
 		ext->x1 = (float)rect->x;
@@ -459,22 +475,23 @@ static void ClipExtents(GB_PAINT *d, GB_EXTENTS *ext)
 		ext->x2 = (float)(rect->x + rect->w);
 		ext->y2 = (float)(rect->y + rect->h);
 	}
-	else
-		get_path_extents(CLIP(d), ext, PAINTER(d)->transform());
+	else*/
+	QPainterPath path = PAINTER(d)->clipPath();
+	get_path_extents(&path, ext, QTransform()); //PAINTER(d)->transform());
 }
 
 static void Fill(GB_PAINT *d, int preserve)
 {
 	CHECK_PATH(d);
 	
-	if (!CLIP(d))
+	//if (!CLIP(d))
 		PAINTER(d)->fillPath(*PATH(d), PAINTER(d)->brush());
-	else
+	/*else
 	{
 		QPainterPath path = PAINTER(d)->worldTransform().inverted().map(*CLIP(d));
 		path = path.intersected(*PATH(d));
 		PAINTER(d)->fillPath(path, PAINTER(d)->brush());
-	}
+	}*/
 	
 	PRESERVE_PATH(d, preserve);
 }
@@ -485,9 +502,9 @@ static void Stroke(GB_PAINT *d, int preserve)
 	
 	if (PAINTER(d)->pen().widthF() > 0.0)
 	{
-		if (!CLIP(d))
+		//if (!CLIP(d))
 			PAINTER(d)->strokePath(*PATH(d), PAINTER(d)->pen());
-		else
+		/*else
 		{
 			QPainterPathStroker stroker;
 			QPen pen = PAINTER(d)->pen();
@@ -502,7 +519,7 @@ static void Stroke(GB_PAINT *d, int preserve)
 			QPainterPath path = PAINTER(d)->worldTransform().inverted().map(*CLIP(d));
 			path = path.intersected(stroker.createStroke(*PATH(d)));
 			PAINTER(d)->fillPath(path, PAINTER(d)->brush());
-		}
+		}*/
 	}
 	
 	PRESERVE_PATH(d, preserve);
@@ -522,7 +539,30 @@ static int PathContains(GB_PAINT *d, float x, float y)
 	return PATH(d)->contains(point);
 }
 
-		
+static void PathOutline(GB_PAINT *d, GB_PAINT_OUTLINE_CB cb)
+{
+	QPainterPath *p = PATH(d);
+	int i, j;
+
+	if (!p)
+		return;
+
+	QList<QPolygonF> qoutline = p->toSubpathPolygons(QTransform());
+
+	for (i = 0; i < qoutline.count(); i++)
+	{
+		QPolygonF qpolygon = qoutline.at(i);
+
+		for (j = 0; j < qpolygon.count(); j++)
+		{
+			QPointF qpoint = qpolygon.at(j);
+			(*cb)(j == 0 ? GB_PAINT_PATH_MOVE : GB_PAINT_PATH_LINE, qpoint.x(), qpoint.y());
+		}
+	}
+}
+
+#define DASH_ZERO 0.0009765625
+
 static void Dash(GB_PAINT *d, int set, float **dashes, int *count)
 {
 	QPen pen = PAINTER(d)->pen();
@@ -534,8 +574,15 @@ static void Dash(GB_PAINT *d, int set, float **dashes, int *count)
 		else
 		{
 			QVector<qreal> dv;
+			qreal d;
+
 			for (int i = 0; i < *count; i++)
-				dv << (qreal)(*dashes)[i];
+			{
+				d = (*dashes)[i];
+				if (d == 0.0)
+					d = DASH_ZERO;
+				dv << (qreal)d;
+			}
 			pen.setStyle(Qt::CustomDashLine);
 			pen.setDashPattern(dv);
 		}
@@ -546,10 +593,16 @@ static void Dash(GB_PAINT *d, int set, float **dashes, int *count)
 		if (pen.style() == Qt::CustomDashLine)
 		{
 			QVector<qreal> dv = pen.dashPattern();
+			float d;
 			*count = dv.count();
 			GB.Alloc(POINTER(dashes), sizeof(float) * *count);
 			for (int i = 0; i < *count; i++)
-				(*dashes)[i] = (float)dv[i];
+			{
+				d = (float)dv[i];
+				if (d <= DASH_ZERO)
+					d = 0.0;
+				(*dashes)[i] = d;
+			}
 		}
 		else
 		{
@@ -776,6 +829,7 @@ static void Ellipse(GB_PAINT *d, float x, float y, float width, float height, fl
 	
 	PATH(d)->arcTo(rect, to_deg(angle), to_deg(length));
 	if (pie)
+		//PATH(d)->lineTo(x + width / 2, y + height / 2);
 		PATH(d)->closeSubpath();
 }
 
@@ -787,17 +841,17 @@ static void Rectangle(GB_PAINT *d, float x, float y, float width, float height)
 
 static void ClipRect(GB_PAINT *d, int x, int y, int w, int h)
 {
-	GB_RECT *rect;
+	//GB_RECT *rect;
 	ResetClip(d);
 	Rectangle(d, x, y, w, h);
 	Clip(d, FALSE);
 	
-	rect = new GB_RECT;
+	/*rect = new GB_RECT;
 	rect->x = x;
 	rect->y = y;
 	rect->w = w;
 	rect->h = h;
-	EXTRA(d)->clipRect = rect;
+	EXTRA(d)->clipRect = rect;*/
 }
 	
 static void GetCurrentPoint(GB_PAINT *d, float *x, float *y)
@@ -1296,6 +1350,7 @@ GB_PAINT_DESC PAINT_Interface =
 	Stroke,
 	PathExtents,
 	PathContains,
+	PathOutline,
 	Dash,
 	DashOffset,
 	FillRule,
@@ -1484,7 +1539,7 @@ void MyPaintEngine::drawTextItem(const QPointF &p, const QTextItem &textItem)
 
 MyPaintEngine MyPaintDevice::engine;
 
-MyPaintDevice::MyPaintDevice() : QPaintDevice() //QImage(1, 1, QImage::Format_ARGB32)
+MyPaintDevice::MyPaintDevice() : QPaintDevice()
 {
 }
 
@@ -1503,7 +1558,7 @@ int MyPaintDevice::metric(PaintDeviceMetric m) const
 		case PdmHeight: return d->height();
 		case PdmWidthMM: return d->widthMM();
 		case PdmHeightMM: return d->heightMM();
-		#if QT_VERSION <= 0x040600
+		#if QT_VERSION <= QT_VERSION_CHECK(4, 6, 0)
 		case PdmNumColors: return d->numColors();
 		#else
 		case PdmNumColors: return d->colorCount();
