@@ -164,8 +164,10 @@ static void update_duplex(CPRINTER *_object)
 
 BEGIN_METHOD_VOID(Printer_new)
 
+	if (MAIN_CHECK_INIT())
+		return;
+
 	THIS->printer = new QPrinter(QPrinter::HighResolution);
-	THIS->num_copies = 1;
 	THIS->page_count = 1;
 
 END_METHOD
@@ -357,7 +359,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Printer_GrayScale)
 
 	if (READ_PROPERTY)
-		GB.ReturnBoolean(!PRINTER->colorMode() == QPrinter::GrayScale);
+		GB.ReturnBoolean(PRINTER->colorMode() == QPrinter::GrayScale);
 	else
 		PRINTER->setColorMode(VPROP(GB_BOOLEAN) ? QPrinter::GrayScale : QPrinter::Color);
 
@@ -365,17 +367,21 @@ END_PROPERTY
 
 BEGIN_PROPERTY(Printer_NumCopies)
 
-	if (READ_PROPERTY)
-		GB.ReturnInteger(THIS->num_copies);
-	else
+#if QT_VERSION >= QT_VERSION_CHECK(4, 7, 0)
+	if (PRINTER->supportsMultipleCopies())
 	{
-		int val = VPROP(GB_INTEGER);
-		
-		if (val >= 1 && val <= 32767)
-		{
-			PRINTER->setNumCopies(val);
-			THIS->num_copies = val;
-		}
+		if (READ_PROPERTY)
+			GB.ReturnInteger(PRINTER->copyCount());
+		else
+			PRINTER->setCopyCount(VPROP(GB_INTEGER));
+	}
+	else
+#endif
+	{
+		if (READ_PROPERTY)
+			GB.ReturnInteger(PRINTER->numCopies());
+		else
+			PRINTER->setNumCopies(VPROP(GB_INTEGER));
 	}
 
 END_PROPERTY
