@@ -61,6 +61,15 @@ typedef
 	}
 	HIGHLIGHT_NAME;
 
+typedef
+	struct
+	{
+		const char *cmd;
+		void *label;
+		bool select;
+	}
+	COMMAND;
+
 static HIGHLIGHT_NAME _highlight_name[] = {
 	{ HIGHLIGHT_HTML, "_DoHtml" },
 	{ HIGHLIGHT_CSS, "_DoCss" },
@@ -83,11 +92,12 @@ static int _highlight_tag = 0;
 static bool _highlight_show_limit = false;
 static QString _highlight_text = "";
 static GHighlightArray *_highlight_data = NULL;
+static int _highlight_line = -1;
 
 static QT_PICTURE _breakpoint_picture = 0;
 static QT_PICTURE _bookmark_picture = 0;
 
-static void highlightCustom(GEditor *master, uint &state, bool &alternate, int &tag, GString &s, GHighlightArray *data, bool &proc)
+static void highlightCustom(GEditor *master, int line, uint &state, bool &alternate, int &tag, GString &s, GHighlightArray *data, bool &proc)
 {
 	void *_object = QT.GetObject((QWidget *)master);
 
@@ -97,6 +107,7 @@ static void highlightCustom(GEditor *master, uint &state, bool &alternate, int &
 	_highlight_text = s.getString();
 	_highlight_show_limit = proc;
 	_highlight_data = data;
+	_highlight_line = line;
 
 	GB.NewArray(_highlight_data, sizeof(GHighlight), 0);
 
@@ -119,7 +130,13 @@ static void highlightCustom(GEditor *master, uint &state, bool &alternate, int &
 
 ****************************************************************************/
 
-BEGIN_PROPERTY(CHIGHLIGHT_state)
+BEGIN_PROPERTY(Highlight_Line)
+
+	GB.ReturnInteger(_highlight_line);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(Highlight_State)
 
 	if (READ_PROPERTY)
 		GB.ReturnInteger(_highlight_state);
@@ -128,7 +145,7 @@ BEGIN_PROPERTY(CHIGHLIGHT_state)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CHIGHLIGHT_alternate)
+BEGIN_PROPERTY(Highlight_AlternateState)
 
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(_highlight_alternate);
@@ -137,7 +154,7 @@ BEGIN_PROPERTY(CHIGHLIGHT_alternate)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CHIGHLIGHT_tag)
+BEGIN_PROPERTY(Highlight_Tag)
 
 	if (READ_PROPERTY)
 		GB.ReturnInteger(_highlight_tag);
@@ -146,7 +163,7 @@ BEGIN_PROPERTY(CHIGHLIGHT_tag)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CHIGHLIGHT_show_limit)
+BEGIN_PROPERTY(Highlight_ShowLimit)
 
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(_highlight_show_limit);
@@ -155,7 +172,7 @@ BEGIN_PROPERTY(CHIGHLIGHT_show_limit)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CHIGHLIGHT_text)
+BEGIN_PROPERTY(Highlight_Text)
 
 	if (READ_PROPERTY)
 		GB.ReturnNewZeroString(TO_UTF8(_highlight_text));
@@ -164,7 +181,7 @@ BEGIN_PROPERTY(CHIGHLIGHT_text)
 
 END_PROPERTY
 
-BEGIN_METHOD(CHIGHLIGHT_add, GB_INTEGER state; GB_INTEGER len)
+BEGIN_METHOD(Highlight_Add, GB_INTEGER state; GB_INTEGER len)
 
 	GHighlight *h;
 
@@ -220,6 +237,15 @@ BEGIN_PROPERTY(Editor_Length)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Editor_EndOfLine)
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(DOC->getEndOfLine());
+	else
+		DOC->setEndOfLine(VPROP(GB_INTEGER));
+
+END_PROPERTY
+
 BEGIN_METHOD_VOID(Editor_HighlightAll)
 
 	DOC->colorizeAll();
@@ -343,8 +369,9 @@ BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 			
 			if (c == '\t')
 			{
-				int l = 8 - col % 8;
-				print_text(THIS, "        ", l);
+				//int l = 8 - col % 8;
+				//print_text(THIS, "        ", l);
+				WIDGET->insert("\t");
 			}
 			else if (c == '\r')
 			{
@@ -355,11 +382,11 @@ BEGIN_METHOD(CEDITOR_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 				WIDGET->cursorGoto(line, DOC->lineLength(line), false);
 				WIDGET->insert("\n");
 			}
-			else if (c == 12) // CTRL+L
+			else if (c == '\f') // CTRL+L
 			{
 				DOC->clear();
 			}
-			else if (c == 7)
+			else if (c == '\a')
 			{
 				WIDGET->flash();
 			}
@@ -1233,6 +1260,37 @@ BEGIN_PROPERTY(Editor_LineOffset)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(Editor_Mouse)
+
+	QT.MouseProperty(_object, _param);
+
+	if (!READ_PROPERTY)
+		WIDGET->saveCursor();
+
+END_PROPERTY
+
+#if 0
+BEGIN_METHOD(Editor_Command, GB_STRING command)
+
+	static COMMAND CommandList[] =
+	{
+		{ "move-left", &&__MOVE_LEFT },
+		{ "move-left-word", &&__MOVE_LEFT_WORD },
+		{ "select-left", &&__MOVE_LEFT, TRUE },
+		{ "select-left-word", &&__MOVE_LEFT_WORD, TRUE },
+		{ "move-right", &&__MOVE_RIGHT },
+		{ "move-right-word", &&__MOVE_RIGHT_WORD },
+		{ "select-right", &&__MOVE_RIGHT, TRUE },
+		{ "select-right-word", &&__MOVE_RIGHT_WORD, TRUE },
+		{ "move-up", &&__MOVE_UP },
+		{ "move-right-word", &&__MOVE_RIGHT_WORD },
+		{ "select-right", &&__MOVE_RIGHT, TRUE },
+		{ "select-right-word", &&__MOVE_RIGHT_WORD, TRUE },
+	};
+
+END_METHOD
+#endif
+
 #if 0
 BEGIN_PROPERTY(Editor_ShowLimits)
 
@@ -1340,12 +1398,13 @@ GB_DESC CHighlightDesc[] =
 	GB_CONSTANT("Preprocessor", "i", HIGHLIGHT_PREPROCESSOR),
 	GB_STATIC_PROPERTY_READ("Alternate", "i", Highlight_Alternate),
 
-	GB_STATIC_PROPERTY("State", "i", CHIGHLIGHT_state),
-	GB_STATIC_PROPERTY("Tag", "i", CHIGHLIGHT_tag),
-	GB_STATIC_PROPERTY("ShowLimit", "b", CHIGHLIGHT_show_limit),
-	GB_STATIC_PROPERTY("Text", "s", CHIGHLIGHT_text),
-	GB_STATIC_PROPERTY("AlternateState", "b", CHIGHLIGHT_alternate),
-	GB_STATIC_METHOD("Add", NULL, CHIGHLIGHT_add, "(State)i[(Count)i]"),
+	GB_STATIC_PROPERTY_READ("Line", "i", Highlight_Line),
+	GB_STATIC_PROPERTY("State", "i", Highlight_State),
+	GB_STATIC_PROPERTY("Tag", "i", Highlight_Tag),
+	GB_STATIC_PROPERTY("ShowLimit", "b", Highlight_ShowLimit),
+	GB_STATIC_PROPERTY("Text", "s", Highlight_Text),
+	GB_STATIC_PROPERTY("AlternateState", "b", Highlight_AlternateState),
+	GB_STATIC_METHOD("Add", NULL, Highlight_Add, "(State)i[(Count)i]"),
 
 	GB_END_DECLARE
 };
@@ -1434,17 +1493,14 @@ GB_DESC CEditorDesc[] =
 	GB_CONSTANT("ShowLimits", "i", GEditor::ShowProcedureLimits),
 	GB_CONSTANT("BlendedLimits", "i", GEditor::BlendedProcedureLimits),
 	GB_CONSTANT("BackgroundLimits", "i", GEditor::ChangeBackgroundAtLimit),
-	//GB_CONSTANT("DrawWithRelief", "i", GEditor::DrawWithRelief),
 	GB_CONSTANT("ShowModifiedLines", "i", GEditor::ShowModifiedLines),
 	GB_CONSTANT("ShowCurrentLine", "i", GEditor::ShowCurrentLine),
 	GB_CONSTANT("ShowLineNumbers", "i", GEditor::ShowLineNumbers),
 	GB_CONSTANT("HighlightBraces", "i", GEditor::HighlightBraces),
 	GB_CONSTANT("HighlightImmediately", "i", GEditor::HighlightImmediately),
 	GB_CONSTANT("ShowDots", "i", GEditor::ShowDots),
-	//GB_CONSTANT("ShowCursorPosition", "i", GEditor::ShowCursorPosition),
 	GB_CONSTANT("HideMargin", "i", GEditor::HideMargin),
 	GB_CONSTANT("NoFolding", "i", GEditor::NoFolding),
-	//GB_CONSTANT("BlinkCursor", "i", GEditor::BlinkCursor),
 
 	GB_METHOD("_new", NULL, CEDITOR_new, "(Parent)Container;"),
 	GB_METHOD("_free", NULL, CEDITOR_free, NULL),
@@ -1484,6 +1540,8 @@ GB_DESC CEditorDesc[] =
 	GB_METHOD("Undo", NULL, CEDITOR_undo, NULL),
 	GB_METHOD("Redo", NULL, CEDITOR_redo, NULL),
 
+	//GB_METHOD("Command", NULL, Editor_command, "(Command)s"),
+
 	GB_PROPERTY_SELF("Styles", ".Editor.Styles"),
 
 	GB_METHOD("Select", NULL, CEDITOR_select, "(StartLine)i(StartColumn)i(EndLine)i(EndColumn)i"),
@@ -1507,6 +1565,7 @@ GB_DESC CEditorDesc[] =
 	GB_METHOD("HighlightAll", NULL, Editor_HighlightAll, NULL),
 	GB_PROPERTY("Text", "s", Editor_Text),
 	GB_PROPERTY_READ("Length", "i", Editor_Length),
+	GB_PROPERTY("EndOfLine", "i", Editor_EndOfLine),
 
 	GB_PROPERTY("Highlight", "i", CEDITOR_highlight),
 	GB_PROPERTY("KeywordsUseUpperCase", "b", CEDITOR_keywords_ucase),
@@ -1544,6 +1603,8 @@ GB_DESC CEditorDesc[] =
 	GB_PROPERTY("ScrollY", "i", Editor_ScrollY),
 	GB_METHOD("Scroll", NULL, Editor_Scroll, "(X)i(Y)i"),
 
+	GB_PROPERTY("Mouse", "i", Editor_Mouse),
+
 	GB_EVENT("Cursor", NULL, NULL, &EVENT_Cursor),
 	GB_EVENT("Scroll", NULL, NULL, &EVENT_Scroll),
 	GB_EVENT("Change", NULL, NULL, &EVENT_Change),
