@@ -84,7 +84,7 @@ class GCommand;
 class GEditor;
 
 typedef
-	void (*GHighlightCallback)(GEditor *master, uint &state, bool &alternate, int &tag, GString &s, GHighlightArray *highlight, bool &proc);
+	void (*GHighlightCallback)(GEditor *master, int line, uint &state, bool &alternate, int &tag, GString &s, GHighlightArray *highlight, bool &proc);
 
 
 class GDocument
@@ -96,21 +96,23 @@ private:
 	GArray<GCommand> undoList;
 	GArray<GCommand> redoList;
 	int undoLevel;
-	bool textHasChanged;
 	int highlightMode;
-	bool keywordsUseUpperCase;
 	GHighlightCallback highlightCallback;
 	uint oldCount;
-	bool readOnly;
-	bool blockUndo;
 	GEditor *selector;
 	GEditor *_currentView;
+	GString _eol;
 	int xs, ys, xs2, ys2;
 	int tabWidth;
 	int colorizeFrom;
 	int _disableColorize;
 	int _disableColorizeStart;
 	int _currentLine;
+	unsigned _eol_mode : 2;
+	unsigned readOnly : 1;
+	unsigned blockUndo : 1;
+	unsigned keywordsUseUpperCase : 1;
+	unsigned textHasChanged : 1;
 
 	void init();
 	void clearUndo();
@@ -129,7 +131,7 @@ public:
 		Custom = 2
 	};
 
-	static void highlightGambas(GEditor *master, uint &state, bool &alternate, int &tag, GString &s, GHighlightArray *data, bool &proc);
+	static void highlightGambas(GEditor *master, int line, uint &state, bool &alternate, int &tag, GString &s, GHighlightArray *data, bool &proc);
 
 	GArray<GLine> lines;
 	GArray<GEditor> views;
@@ -148,6 +150,10 @@ public:
 	void setText(const GString & text);
 	int getLength() const;
 
+	int getEndOfLine() const { return _eol_mode; }
+	void setEndOfLine(int mode);
+	GString getEndOfLineText() const { return _eol; }
+
 	void subscribe(GEditor *view);
 	void unsubscribe(GEditor *view);
 
@@ -212,6 +218,8 @@ public:
 	void hideSelection();
 	void eraseSelection(bool insertMode);
 
+	bool insideUndo() const { return undoLevel; }
+
 	void emitTextChanged();
 
 private:
@@ -223,6 +231,7 @@ private:
 	void modifyLine(GLine *l, int y);
 	void disableColorize();
 	void enableColorize();
+	void invalidateHighlight();
 };
 
 #endif
