@@ -109,7 +109,7 @@ void GEditor::reset()
 	x1m = x2m = 0;
 	y1m = y2m = -1;
 	lastx = -1;
-	cursor = false;
+	_cursor = false;
 	lineNumberLength = 0;
 	center = false;
 	largestLine = 0;
@@ -149,6 +149,7 @@ GEditor::GEditor(QWidget *parent)
 	setMouseTracking(true);
 	viewport()->setMouseTracking(true);
 	viewport()->setCursor(Qt::ibeamCursor);
+	saveCursor();
   viewport()->setBackgroundRole(QPalette::Base);
 	viewport()->setPaletteBackgroundColor(defaultColors[GLine::Background]);
 	viewport()->setFocusProxy(this);
@@ -618,7 +619,7 @@ void GEditor::drawTextWithTab(QPainter &p, int sx, int x, int y, const QString &
 	
 	for(;;) //while (pos < s.length())
 	{
-		tnp = s.find('\t', tp);
+		tnp = s.indexOf('\t', tp);
 		if (tnp < 0)
 		{
 			p.drawText(x, y, s.mid(tp));
@@ -804,7 +805,7 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 		p.setPen(styles[GLine::Normal].color);
 		//p.drawText(x, y, l->s.mid(pos).getString());
 		drawTextWithTab(p, xs, x, y, l->s.mid(pos).getString());
-		if (show_dots)
+		if (ps >= 0 && pos >= ps && show_dots)
 			paintDottedSpaces(p, row, pos, QMIN(xmin + lmax - pos, (int)l->s.length() - pos));
 	}
 }
@@ -826,7 +827,7 @@ void GEditor::paintShowString(QPainter &p, GLine *l, int x, int y, int xmin, int
 		{
 			if (pos >= (int)l->s.length())
 				break;
-			pos = l->s.find(_showString, pos, !_showStringIgnoreCase);
+			pos = l->s.indexOf(_showString, pos, !_showStringIgnoreCase);
 			if (pos < 0)
 				break;
 			ps = lineWidth(row, pos);
@@ -1086,7 +1087,7 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	}
 
 	// Margin
-	if (margin && (margin > ur.left()))
+	if (!getFlag(HideMargin) && (margin > ur.left()))
 	{
 		//if (!l->flag)
 		//	p.fillRect(0, 0, margin, _cellh, color); //styles[GLine::Background].color);
@@ -1187,7 +1188,7 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	}
 
 	// Text cursor
-	if (cursor && realRow == y)
+	if (_cursor && realRow == y)
 	{
 		QColor color = styles[GLine::Normal].color;
 		int xc = lineWidth(realRow, x);
@@ -1284,14 +1285,14 @@ void GEditor::checkMatching()
 	if (x > 0)
 	{
 		c = str.at(x - 1);
-		pos = look.find(c);
+		pos = look.indexOf(c);
 		x1m = x - 1;
 	}
 	
 	if (pos < 0 && x < len)
 	{
 		c = str.at(x);
-		pos = look.find(c);
+		pos = look.indexOf(c);
 		x1m = x;
 	}
 	
@@ -1545,7 +1546,7 @@ bool GEditor::cursorGoto(int ny, int nx, bool mark)
 	else if (ny >= numLines())
 	{
 		ny = numLines() - 1;
-		nx = QMIN(nx, lineLength(ny));
+		nx = lineLength(ny);
 	}
 	else
 		ny = checkFolded(ny);
@@ -1593,6 +1594,9 @@ bool GEditor::cursorGoto(int ny, int nx, bool mark)
 		if (mark)
 			doc->endSelection(y, x);
 		
+		if (oy != y && !doc->insideUndo())
+			_cutBuffer.clear();
+		
 		emit cursorMoved();
 	}
 	else if (center)
@@ -1834,8 +1838,12 @@ void GEditor::del(bool ctrl)
 	
 	if (x >= lineLength(y))
 	{
-		if (!_insertMode && y < (numLines() - 1))
+		if (y < (numLines() - 1))
+		{
+			if (_insertMode)
+				doc->insert(y, x, QString(), TRUE);
 			doc->remove(y, x, y + 1, 0);
+		}
 	}
 	else
 	{
@@ -1898,6 +1906,31 @@ void GEditor::backspace(bool ctrl)
 	doc->end();
 }
 
+void GEditor::selectCurrentLine()
+{
+	cursorGoto(y, 0, FALSE);
+	cursorGoto(y + 1, 0, TRUE);
+}
+
+void GEditor::deleteCurrentLine()
+{
+	bool im;
+
+	if (doc->hasSelection())
+	{
+		doc->eraseSelection(_insertMode);
+		return;
+	}
+
+	im = _insertMode;
+	_insertMode = FALSE;
+	doc->begin();
+	selectCurrentLine();
+	del(FALSE);
+	doc->end();
+	_insertMode = im;
+}
+
 void GEditor::tab(bool back)
 {
 	QString ins;
@@ -1984,6 +2017,17 @@ void GEditor::copy(bool mouse)
 
 void GEditor::cut()
 {
+	if (!doc->hasSelection())
+	{
+		doc->begin();
+		selectCurrentLine();
+		_cutBuffer += doc->getSelectedText(_insertMode);
+		QApplication::clipboard()->setText(_cutBuffer.getString(), QClipboard::Clipboard);
+		doc->eraseSelection(_insertMode);
+		doc->end();
+		return;
+	}
+
 	copy(false);
 	doc->eraseSelection(_insertMode);
 }
@@ -1991,8 +2035,9 @@ void GEditor::cut()
 void GEditor::paste(bool mouse)
 {
 	QString text;
+	GString gtext;
 	QString subType("plain");
-	int i, i2, xs;
+	int i, i2, xs, len;
 	QString tab;
 
 	text = QApplication::clipboard()->text(subType, mouse ? QClipboard::Selection : QClipboard::Clipboard);
@@ -2005,26 +2050,25 @@ void GEditor::paste(bool mouse)
 
 	for (i = 0; i < text.length(); i++)
 	{
-		if ((text[i] < ' ' || text[i].isSpace()) && text[i] != '\n')
+		if ((text[i] < ' ' || text[i].isSpace()) && text[i] != '\n' && text[i] != '\r')
 			text[i] = ' ';
 	}
 
 	if (_insertMode)
 	{
+		gtext = text;
 		doc->begin();
 		i = 0;
 		while (i < (int)text.length())
 		{
-			i2 = text.find('\n', i);
-			if (i2 < 0) 
-				i2 = text.length();
+			i2 = gtext.findNextLine(i, len);
 			xs = x;
-			insert(text.mid(i, i2 - i));
+			insert(text.mid(i, len));
 			x = xs;
 			y++;
 			if (y >= doc->numLines())
 				insert("\n");
-			i = i2 + 1; 
+			i = i2;
 		}
 		doc->end();
 	}
@@ -2078,6 +2122,10 @@ void GEditor::keyPressEvent(QKeyEvent *e)
 				cursorPageUp(shift, alt); return;
 			case Qt::Key_Next:
 				cursorPageDown(shift, alt); return;
+			case Qt::Key_Return:
+				if (ctrl)
+					expand(shift);
+				return;
 		}
 
 		if (!ctrl)
@@ -2123,8 +2171,13 @@ void GEditor::keyPressEvent(QKeyEvent *e)
 			case Qt::Key_Down:
 				cursorDown(shift, ctrl, alt); return;
 			case Qt::Key_Enter: 
-			case Qt::Key_Return:
 				newLine(); return;
+			case Qt::Key_Return:
+				if (ctrl)
+					expand(shift);
+				else
+					newLine();
+				return;
 			case Qt::Key_Home:
 				cursorHome(shift, ctrl, alt); return;
 			case Qt::Key_End:
@@ -2162,6 +2215,8 @@ void GEditor::keyPressEvent(QKeyEvent *e)
 				redo(); return;
 			case Qt::Key_A:
 				selectAll(); return;
+			case Qt::Key_D:
+				deleteCurrentLine(); return;
 		}
 	}
 
@@ -2274,7 +2329,7 @@ bool GEditor::updateCursor()
 {
 	if (contentsX() + lastx >= margin)
 	{
-		viewport()->setCursor(Qt::IBeamCursor);
+		viewport()->setCursor(_saveCursor);
 		return false;
 	}
 	else
@@ -2447,14 +2502,14 @@ void GEditor::ensureCursorVisible()
 void GEditor::startBlink()
 {
 	blinkTimer->start(QApplication::cursorFlashTime() / 2, false);
-	cursor = true;
+	_cursor = true;
 	updateLine(y);
 }
 
 void GEditor::stopBlink()
 {
 	blinkTimer->stop();
-	cursor = false;
+	_cursor = false;
 	updateLine(y);
 }
 
@@ -2462,7 +2517,7 @@ void GEditor::blinkTimerTimeout()
 {
 	if (!doc->isReadOnly())
 	{
-		cursor = !cursor;
+		_cursor = !_cursor;
 		updateLine(y);
 	}
 }
@@ -2625,19 +2680,6 @@ void GEditor::docTextChanged()
 		docTextChangedLater();
 }
 
-#if 0
-void GEditor::baptizeVisible()
-{
-	doc->baptizeUntil(lastVisibleRow());
-}
-
-void GEditor::baptizeVisible(int x, int y)
-{
-	doc->baptizeUntil(lastVisibleRow(y));
-	//viewport()->update();
-}
-#endif
-
 void GEditor::inputMethodEvent(QInputMethodEvent *e)
 {
 	//qDebug("inputMethodEvent: %s\n", (const char *)e->commitString().toUtf8());
@@ -2650,14 +2692,6 @@ void GEditor::inputMethodEvent(QInputMethodEvent *e)
 
 	insert(e->commitString());
 	
-	/*if (QApplication::keypadNavigationEnabled() && 
-					hasFocus() && !hasEditFocus()
-					&& !e->preeditString().isEmpty()) 
-	{
-		setEditFocus(true);
-		selectAll();        // so text is replaced rather than appended to
-	}*/
-
 	#if 0
     int priorState = d->undoState;
     d->removeSelectedText();
@@ -3290,3 +3324,30 @@ void GEditor::setLineOffset(int l)
 	_firstLineNumber = l;
 	update();
 }
+
+void GEditor::expand(bool shift)
+{
+	bool e;
+
+	e = isFolded(y);
+
+	if (shift)
+	{
+		if (e)
+			unfoldAll();
+		else
+			foldAll();
+	}
+	else
+	{
+		if (e)
+			unfoldLine(y);
+		else
+			foldLine(y);
+	}
+}
+
+void GEditor::saveCursor()
+{
+	_saveCursor = viewport()->cursor();
+}
