@@ -78,7 +78,7 @@ private:
 	int largestLine;
 	int x, y, xx;
 	int nx, ny;
-	bool cursor;
+	bool _cursor;
 	QTimer *blinkTimer;
 	QTimer *scrollTimer;
 	int x1m, x2m, y1m, y2m;
@@ -104,6 +104,8 @@ private:
 	bool _border;
 	bool _ensureCursorVisibleLater;
 	int _firstLineNumber;
+	QCursor _saveCursor;
+	GString _cutBuffer;
 	
 	int lastx;
 	bool left;
@@ -239,7 +241,10 @@ public:
 	void selectAll();
 	void movePreviousSameIndent(bool shift);
 	void moveNextSameIndent(bool shift);
-	
+	void expand(bool shift);
+	void selectCurrentLine();
+	void deleteCurrentLine();
+
 	bool getInsertMode() const { return _insertMode; }
 	void setInsertMode(bool mode);
 	
@@ -289,6 +294,8 @@ public:
 	GString getSelectedText() { return doc->getSelectedText(_insertMode); }
 	void hideSelection() { doc->hideSelection(); }
 
+	void saveCursor();
+
 signals:
 
 	void cursorMoved();
