@@ -25,6 +25,7 @@
 #define __GB_QT_H
 
 #include "gambas.h"
+#include "gb.image.h"
 
 #ifdef OS_MACOSX
 //#define NO_X_WINDOW 1
@@ -41,6 +42,7 @@
 #include <QBrush>
 #include <QPen>
 #include <QTransform>
+#include <QColor>
 
 #define QT_INTERFACE_VERSION 1
 
@@ -61,6 +63,8 @@
 #define QSTRING_ARG(_arg) (QString::fromUtf8((const char *)(VARG(_arg).addr + VARG(_arg).start), VARG(_arg).len))
 #define QSTRING_PROP() (QString::fromUtf8((const char *)(VPROP(GB_STRING).addr + VPROP(GB_STRING).start), VPROP(GB_STRING).len))
 
+#define TO_QCOLOR(_col) QColor::fromRgba((QRgb)((_col) ^ 0xFF000000))
+
 typedef
 	struct {
 		GB_BASE ob;
@@ -107,6 +111,9 @@ typedef
 typedef
 	void (*QT_FONT_FUNC)(QFont &, void *);
 
+typedef
+	void (*QT_COLOR_FUNC)(void *);
+
 typedef
 	struct {
 		intptr_t version;
@@ -131,6 +138,9 @@ typedef
 		void (*Link)(QObject *, void *);
 		void *(*GetLink)(QObject *);
 		QPainter *(*GetCurrentPainter)();
+		uint (*GetBackgroundColor)(void *);
+		void (*MouseProperty)(void *, void *);
+		QT_COLOR_FUNC (*AfterSetColor)(QT_COLOR_FUNC);
 		void *_null;
 		}
 	QT_INTERFACE;
