@@ -58,6 +58,7 @@
 
 #include "CFont.h"
 #include "CScreen.h"
+#include "CStyle.h"
 #include "CWidget.h"
 #include "CWindow.h"
 #include "CButton.h"
@@ -84,7 +85,6 @@
 #include "CWatch.h"
 #include "CScrollView.h"
 #include "CDrawingArea.h"
-#include "CProgress.h"
 #include "CMessage.h"
 #include "CSlider.h"
 #include "CScrollBar.h"
@@ -120,8 +120,8 @@ int MAIN_scale = 6;
 #ifndef NO_X_WINDOW
 int MAIN_x11_last_key_code = 0;
 #endif
-
 bool MAIN_debug_busy = false;
+bool MAIN_init = false;
 
 GB_CLASS CLASS_Control;
 GB_CLASS CLASS_Container;
@@ -319,6 +319,11 @@ void MAIN_process_events(void)
 	_no_destroy--;
 }
 
+void MAIN_init_error()
+{
+	GB.Error("GUI is not initialized");
+}
+
 /** MyApplication **********************************************************/
 
 bool MyApplication::_tooltip_disable = false;
@@ -328,6 +333,24 @@ QEventLoop *MyApplication::eventLoop = 0;
 MyApplication::MyApplication(int &argc, char **argv)
 : QApplication(argc, argv)
 {
+	if (isSessionRestored())
+	{
+		bool ok;
+		int desktop;
+
+		if (argc >= 2 && ::strcmp(argv[argc - 2], "-session-desktop") == 0)
+		{
+			desktop = QString(argv[argc - 1]).toInt(&ok);
+			if (ok)
+				CWINDOW_MainDesktop = desktop;
+
+			//qDebug("session desktop: %d", CWINDOW_MainDesktop);
+
+			argc -= 2;
+		}
+	}
+
+	QObject::connect(this, SIGNAL(commitDataRequest(QSessionManager &)), SLOT(commitDataRequested(QSessionManager &)));
 }
 
 void MyApplication::initClipboard()
@@ -480,6 +503,45 @@ void MyApplication::setTooltipEnabled(bool b)
 	setEventFilter(b);
 }
 
+void MyApplication::commitDataRequested(QSessionManager &session)
+{
+	QStringList cmd;
+
+	if (CAPPLICATION_Restart)
+	{
+		int i;
+		char **str;
+
+		str = (char **)GB.Array.Get(CAPPLICATION_Restart, 0);
+		for (i = 0; i < GB.Array.Count(CAPPLICATION_Restart); i++)
+		{
+			if (str[i])
+				cmd += str[i];
+			else
+				cmd += "";
+		}
+	}
+	else
+		cmd += arguments().at(0);
+
+	cmd += "-session";
+	cmd += sessionId();
+
+	if (CWINDOW_Main)
+	{
+		cmd += "-session-desktop";
+		cmd += QString::number(X11_window_get_desktop(CWINDOW_Main->widget.widget->winId()));
+		/*cmd += "-session-data";
+		cmd += QString::number(CWINDOW_Main->x) + ","
+		       + QString::number(CWINDOW_Main->y) + ","
+					 + QString::number(CWINDOW_Main->w) + ","
+					 + QString::number(CWINDOW_Main->h) + ","
+					 + QString::number(QApplication::desktop()->screenNumber(CWINDOW_Main->widget.widget));*/
+	}
+
+	session.setRestartCommand(cmd);
+}
+
 #ifndef NO_X_WINDOW
 
 static void x11_set_event_filter(void (*filter)(XEvent *))
@@ -745,23 +807,32 @@ static void *_old_hook_main;
 
 static void hook_main(int *argc, char ***argv)
 {
+	char *env;
+
+	env = getenv("GB_X11_INIT_THREADS");
+	if (env && atoi(env))
+		XInitThreads();
+
 	new MyApplication(*argc, *argv);
-	
+
 	QT_Init();
 	init_lang(GB.System.Language(), GB.System.IsRightToLeft());
-	
+
+	MAIN_init = true;
+
 	//_old_handler = XSetErrorHandler(X11_error_handler);
-	
+
 	CALL_HOOK_MAIN(_old_hook_main, argc, argv);
 }
 
 
 static void hook_quit()
 {
-	CWINDOW_close_all();
-	CWINDOW_delete_all();
+	CWINDOW_close_all(true);
+	CWINDOW_delete_all(true);
 
-	qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
+	qApp->sendPostedEvents(); //processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
+	qApp->sendPostedEvents(0, QEvent::DeferredDelete);
 }
 
 
@@ -769,7 +840,8 @@ static void hook_loop()
 {
 	//qDebug("**** ENTERING EVENT LOOP");
 	
-	qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
+	qApp->sendPostedEvents();
+	//qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 
 	in_event_loop = true;
 
@@ -786,7 +858,12 @@ static void hook_wait(int duration)
 {
 	MAIN_in_wait++;
 	if (duration > 0)
-		qApp->processEvents(QEventLoop::AllEvents, duration);
+	{
+		if (CKEY_is_valid())
+			fprintf(stderr, "gb.qt4: warning: calling the event loop during a keyboard event handler is ignored\n");
+		else
+			qApp->processEvents(QEventLoop::AllEvents, duration);
+	}
 	else
 		qApp->processEvents(QEventLoop::ExcludeUserInputEvents, duration);
 	MAIN_in_wait--;
@@ -862,70 +939,6 @@ static void hook_error(int code, char *error, char *where)
 	//qApp->exit();
 }
 
-#if 0
-static int hook_image(CIMAGE **pimage, GB_IMAGE_INFO *info) //void **pdata, int width, int height, int format)
-{
-	CIMAGE *image = *pimage;
-	QImage *img;
-
-	if (!image)
-	{
-		img = new QImage(info->width, info->height, GB_IMAGE_TRANSPARENT(info->format) ? QImage::Format_ARGB32 : QImage::Format_RGB32);
-
-		if (info->data)
-			GB.Image.Convert(img->bits(), GB_IMAGE_BGRA, info->data, info->format, info->width, info->height);
-
-		GB.New(POINTER(&image), GB.FindClass("Image"), NULL, NULL);
-		delete image->image;
-		image->image = img;
-
-		*pimage = image;
-	}
-	else
-	{
-		info->data = image->image->bits();
-		info->width = image->image->width();
-		info->height = image->image->height();
-		info->format = image->image->hasAlphaChannel() ? GB_IMAGE_BGRA : GB_IMAGE_BGRX;
-	}
-
-	return 0;
-}
-
-static int hook_picture(CPICTURE **ppicture, GB_PICTURE_INFO *info)
-{
-	CPICTURE *picture = *ppicture;
-	QImage *img;
-
-	if (!picture)
-	{
-		if (info->format == GB_IMAGE_BGRA || info->format == GB_IMAGE_BGRX)
-			img = new QImage((uchar *)info->data, info->width, info->height, info->format == GB_IMAGE_BGRA ? QImage::Format_ARGB32 : QImage::Format_RGB32);
-		else
-		{
-			img = new QImage(info->width, info->height, GB_IMAGE_TRANSPARENT(info->format) ? QImage::Format_ARGB32 : QImage::Format_RGB32);
-			GB.Image.Convert(img->bits(), GB_IMAGE_BGRA, info->data, info->format, info->width, info->height);
-		}
-
-		GB.New(POINTER(&picture), GB.FindClass("Picture"), NULL, NULL);
-		delete picture->pixmap;
-		*picture->pixmap = QPixmap::fromImage(*img);
-		delete img;
-
-		*ppicture = picture;
-	}
-	else
-	{
-		info->data = NULL;
-		info->format = GB_IMAGE_BGRA;
-		info->width = picture->pixmap->width();
-		info->height = picture->pixmap->height();
-	}
-
-	return 0;
-}
-#endif
-
 static void QT_InitWidget(QWidget *widget, void *object, int fill_bg)
 {
 	((CWIDGET *)object)->flag.fillBackground = fill_bg;	
@@ -1018,7 +1031,7 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CTabStripContainerChildrenDesc, CTabStripContainerDesc, CTabStripDesc,
 	CScrollViewDesc,
 	CDrawingAreaDesc,
-	CProgressDesc, CSliderDesc, CSpinBoxDesc, CMovieBoxDesc, CScrollBarDesc,
+	CSliderDesc, CSpinBoxDesc, CMovieBoxDesc, CScrollBarDesc,
 	CWindowMenusDesc, CWindowControlsDesc, CWindowDesc, CWindowsDesc, CFormDesc,
 	CDialogDesc,
 	#ifndef NO_X_WINDOW
@@ -1055,6 +1068,9 @@ void *GB_QT4_1[] EXPORT = {
 	(void *)QT_Link,
 	(void *)QT_GetLink,
 	(void *)PAINT_get_current,
+	(void *)CWIDGET_get_background,
+	(void *)Control_Mouse,
+	(void *)CWIDGET_after_set_color,
 	NULL
 };
 
@@ -1107,7 +1123,7 @@ int EXPORT GB_INIT(void)
 	GB.Component.Load("gb.image");
 	GB.Component.Load("gb.gui.base");
 	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
-  IMAGE.SetDefaultFormat(GB_IMAGE_BGRA);
+  IMAGE.SetDefaultFormat(GB_IMAGE_BGRP);
 	DRAW_init();
 	
 	CLASS_Control = GB.FindClass("Control");
