@@ -30,6 +30,7 @@
 #include <QApplication>
 #include <QTimerEvent>
 #include <QEventLoop>
+#include <QSessionManager>
 
 #define DO_NOT_USE_QT_INTERFACE
 #include "gb.qt.h"
@@ -43,6 +44,7 @@ extern int MAIN_in_message_box;
 extern int MAIN_loop_level;
 extern int MAIN_scale;
 extern bool MAIN_debug_busy;
+extern bool MAIN_init;
 #ifndef NO_X_WINDOW
 extern int MAIN_x11_last_key_code;
 #endif
@@ -103,10 +105,12 @@ public:
   
   static QEventLoop *eventLoop;
 	
+
 public slots:
 	
 	void linkDestroyed(QObject *);
 	void clipboardHasChanged();
+	void commitDataRequested(QSessionManager &);
   
 private:
   static bool _tooltip_disable;
@@ -137,6 +141,9 @@ private:
 void MAIN_check_quit(void);
 void MAIN_update_scale(void);
 void MAIN_process_events(void);
+void MAIN_init_error(void);
+
+#define MAIN_CHECK_INIT() (MAIN_init ? 0 : (MAIN_init_error(), 1))
 
 const char *QT_ToUTF8(const QString &str);
 void QT_RegisterAction(void *object, const char *key, int on);
