@@ -23,6 +23,9 @@
 
 #define __CWEBSETTINGS_CPP
 
+#include <errno.h>
+#include <unistd.h>
+
 #include <QNetworkDiskCache>
 #include <QNetworkProxy>
 
@@ -189,6 +192,18 @@ BEGIN_PROPERTY(WebSettingsCache_Path)
 		GB.ReturnString(_cache_path);
 	else
 	{
+		QString path = QSTRING_PROP();
+		QString root = QString(GB.System.Home());
+
+		if (root.at(root.length() - 1) != '/')
+			root += '/';
+		root += ".cache/";
+		if (!path.startsWith(root))
+		{
+			GB.Error("Cache directory must be located inside ~/.cache");
+			return;
+		}
+
 		GB.StoreString(PROP(GB_STRING), &_cache_path);
 		set_cache(_cache_enabled);
 	}
@@ -204,6 +219,45 @@ BEGIN_PROPERTY(WebSettingsCache_Enabled)
 
 END_PROPERTY
 
+static int _clear_error;
+static char *_clear_path;
+
+static void remove_file(const char *path)
+{
+	if (rmdir(path) == 0)
+		return;
+
+	if (errno == ENOTDIR)
+	{
+		if (unlink(path) == 0)
+			return;
+	}
+
+	if (_clear_error == 0)
+	{
+		_clear_error = errno;
+		_clear_path = GB.NewZeroString(path);
+	}
+}
+
+BEGIN_METHOD_VOID(WebSettingsCache_Clear)
+
+	if (!_cache_path || !*_cache_path)
+		return;
+
+	_clear_error = 0;
+	GB.BrowseDirectory(_cache_path, NULL, remove_file);
+
+	if (_clear_error)
+	{
+		GB.Error("Unable to remove '&1': &2", _clear_path, strerror(_clear_error));
+		GB.FreeString(&_clear_path);
+	}
+
+	set_cache(_cache_enabled);
+
+END_METHOD
+
 
 /***************************************************************************/
 
@@ -326,7 +380,7 @@ END_PROPERTY
 
 /***************************************************************************/
 
-GB_DESC CWebViewSettingsDesc[] =
+GB_DESC WebViewSettingsDesc[] =
 {
   GB_DECLARE(".WebView.Settings", 0),
 	
@@ -337,7 +391,7 @@ GB_DESC CWebViewSettingsDesc[] =
 	GB_END_DECLARE
 };
 
-GB_DESC CWebSettingsFontsDesc[] =
+GB_DESC WebSettingsFontsDesc[] =
 {
 	GB_DECLARE(".WebSettings.Fonts", 0), GB_VIRTUAL_CLASS(),
 
@@ -356,7 +410,7 @@ GB_DESC CWebSettingsFontsDesc[] =
 	GB_END_DECLARE
 };
 
-GB_DESC CWebSettingsIconDatabaseDesc[] =
+GB_DESC WebSettingsIconDatabaseDesc[] =
 {
   GB_DECLARE(".WebSettings.IconDatabase", 0), GB_VIRTUAL_CLASS(),
 	
@@ -367,17 +421,18 @@ GB_DESC CWebSettingsIconDatabaseDesc[] =
 	GB_END_DECLARE
 };
 
-GB_DESC CWebSettingsCacheDesc[] =
+GB_DESC WebSettingsCacheDesc[] =
 {
   GB_DECLARE(".WebSettings.Cache", 0), GB_VIRTUAL_CLASS(),
 	
 	GB_STATIC_PROPERTY("Enabled", "b", WebSettingsCache_Enabled),
 	GB_STATIC_PROPERTY("Path", "s", WebSettingsCache_Path),
+	GB_STATIC_METHOD("Clear", NULL, WebSettingsCache_Clear, NULL),
 	
 	GB_END_DECLARE
 };
 
-GB_DESC CWebSettingsProxyDesc[] =
+GB_DESC WebSettingsProxyDesc[] =
 {
   GB_DECLARE(".WebSettings.Proxy", 0), GB_VIRTUAL_CLASS(),
 	
@@ -390,7 +445,7 @@ GB_DESC CWebSettingsProxyDesc[] =
 	GB_END_DECLARE
 };
 
-GB_DESC CWebSettingsDesc[] =
+GB_DESC WebSettingsDesc[] =
 {
   GB_DECLARE("WebSettings", 0),
 	
