@@ -28,6 +28,7 @@
 #include <QNetworkProxy>
 #include <QWebPage>
 #include <QWebFrame>
+#include <QWebHistory>
 
 #include "ccookiejar.h"
 #include "cwebsettings.h"
@@ -57,8 +58,12 @@ DECLARE_EVENT(EVENT_NEW_FRAME);
 DECLARE_EVENT(EVENT_AUTH);
 DECLARE_EVENT(EVENT_DOWNLOAD);
 
+#define HISTORY (WIDGET->history())
+
+
 static QNetworkAccessManager *_network_access_manager = 0;
 static CWEBVIEW *_network_access_manager_view = 0;
+static QT_COLOR_FUNC _old_after_set_color;
 
 /*
 static WEBVIEW_ACTION _actions[] = 
@@ -161,6 +166,41 @@ static QWebPage::WebAction get_action(const char *name)
 }
 */
 
+static void after_set_color(void *_object)
+{
+	if (!GB.Is(THIS, CLASS_WebView))
+	{
+		if (_old_after_set_color)
+			(*_old_after_set_color)(THIS);
+		return;
+	}
+
+	if (QT.GetBackgroundColor(THIS) == GB_COLOR_DEFAULT)
+	{
+		QPalette palette = WIDGET->palette();
+		WIDGET->page()->setPalette(palette);
+		WIDGET->setAttribute(Qt::WA_OpaquePaintEvent, true);
+	}
+	else
+	{
+		qDebug("after_set_color");
+		QPalette palette = WIDGET->palette();
+		palette.setBrush(QPalette::Base, Qt::transparent);
+		WIDGET->page()->setPalette(palette);
+		WIDGET->setAttribute(Qt::WA_OpaquePaintEvent, false);
+	}
+}
+
+static void stop_view(void *_object)
+{
+	//fprintf(stderr, "stop_view\n");
+	THIS->stopping = TRUE;
+	WIDGET->stop();
+	THIS->stopping = FALSE;
+}
+
+//-------------------------------------------------------------------------
+
 BEGIN_METHOD(WebView_new, GB_OBJECT parent)
 
   MyWebView *wid = new MyWebView(QT.GetContainer(VARG(parent)));
@@ -204,10 +244,16 @@ BEGIN_METHOD_VOID(WebView_free)
 
 END_METHOD
 
+BEGIN_METHOD_VOID(WebView_init)
+
+	_old_after_set_color = QT.AfterSetColor(after_set_color);
+
+END_METHOD
+
 BEGIN_METHOD_VOID(WebView_exit)
 
 	delete _network_access_manager;
-	
+
 END_METHOD
 
 BEGIN_PROPERTY(WebView_Url)
@@ -215,7 +261,10 @@ BEGIN_PROPERTY(WebView_Url)
 	if (READ_PROPERTY)
 		GB.ReturnNewZeroString(TO_UTF8(WIDGET->url().toString()));
 	else
-		WIDGET->setUrl(QUrl(QSTRING_PROP()));
+	{
+		stop_view(THIS);
+		WIDGET->setUrl(QSTRING_PROP());
+	}
 
 END_PROPERTY
 
@@ -298,16 +347,20 @@ BEGIN_METHOD_VOID(WebView_Forward)
 
 END_METHOD
 
-BEGIN_METHOD_VOID(WebView_Reload)
+BEGIN_METHOD(WebView_Reload, GB_BOOLEAN bypass)
 
-	WIDGET->stop();
-	WIDGET->reload();
+	bool bypass = VARGOPT(bypass, false);
+	stop_view(THIS);
+	if (bypass)
+		WIDGET->page()->triggerAction(QWebPage::ReloadAndBypassCache);
+	else
+		WIDGET->reload();
 
 END_METHOD
 
 BEGIN_METHOD_VOID(WebView_Stop)
 
-	WIDGET->stop();
+	stop_view(THIS);
 
 END_METHOD
 
@@ -539,12 +592,59 @@ BEGIN_PROPERTY(WebView_Document)
 
 END_PROPERTY
 
+//-------------------------------------------------------------------------
 
-/***************************************************************************/
+BEGIN_PROPERTY(WebViewHistory_Count)
 
-GB_DESC CWebViewAuthDesc[] =
+	GB.ReturnInteger(HISTORY->count());
+
+END_PROPERTY
+
+BEGIN_PROPERTY(WebViewHistory_Max)
+
+	GB.ReturnInteger(HISTORY->count() - 1);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(WebViewHistory_Index)
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(HISTORY->currentItemIndex());
+	else
+	{
+		int index = VPROP(GB_INTEGER);
+		if (index < 0 || index >= HISTORY->count())
+		{
+			GB.Error(GB_ERR_ARG);
+			return;
+		}
+		HISTORY->goToItem(HISTORY->itemAt(index));
+	}
+
+END_PROPERTY
+
+BEGIN_PROPERTY(WebViewHistory_MaxSize)
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(HISTORY->maximumItemCount());
+	else
+		HISTORY->setMaximumItemCount(VPROP(GB_INTEGER));
+
+END_PROPERTY
+
+BEGIN_METHOD_VOID(WebViewHistory_Clear)
+
+	int max = HISTORY->maximumItemCount();
+	HISTORY->setMaximumItemCount(0);
+	HISTORY->setMaximumItemCount(max);
+
+END_METHOD
+
+//-------------------------------------------------------------------------
+
+GB_DESC WebViewAuthDesc[] =
 {
-  GB_DECLARE(".WebView.Auth", sizeof(CWEBVIEW)), GB_VIRTUAL_CLASS(),
+  GB_DECLARE_VIRTUAL(".WebView.Auth"),
 	
 	GB_PROPERTY_READ("Url", "s", WebViewAuth_Url),
 	GB_PROPERTY_READ("Realm", "s", WebViewAuth_Realm),
@@ -554,14 +654,28 @@ GB_DESC CWebViewAuthDesc[] =
 	GB_END_DECLARE
 };
 
-GB_DESC CWebViewDesc[] =
+GB_DESC WebViewHistoryDesc[] =
+{
+	GB_DECLARE_VIRTUAL(".WebView.History"),
+
+	GB_PROPERTY_READ("Count", "i", WebViewHistory_Count),
+	GB_PROPERTY_READ("Max", "i", WebViewHistory_Max),
+	GB_PROPERTY("Index", "i", WebViewHistory_Index),
+	GB_PROPERTY("MaxSize", "i", WebViewHistory_MaxSize),
+	GB_METHOD("Clear", NULL, WebViewHistory_Clear, NULL),
+
+	GB_END_DECLARE
+};
+
+GB_DESC WebViewDesc[] =
 {
   GB_DECLARE("WebView", sizeof(CWEBVIEW)), GB_INHERITS("Control"),
 	
   GB_METHOD("_new", NULL, WebView_new, "(Parent)Container;"),
   GB_METHOD("_free", NULL, WebView_free, NULL),
+  GB_METHOD("_init", NULL, WebView_init, NULL),
   GB_METHOD("_exit", NULL, WebView_exit, NULL),
-	
+
 	GB_PROPERTY("Url", "s", WebView_Url),
 	GB_PROPERTY("Status", "s", WebView_Status),
 
@@ -584,12 +698,13 @@ GB_DESC CWebViewDesc[] =
 	
 	GB_PROPERTY_SELF("Settings", ".WebView.Settings"),
 	GB_PROPERTY_SELF("Auth", ".WebView.Auth"),
+	GB_PROPERTY_SELF("History", ".WebView.History"),
 
 	GB_METHOD("Back", NULL, WebView_Back, NULL),
 	GB_METHOD("Forward", NULL, WebView_Forward, NULL),
-	GB_METHOD("Reload", NULL, WebView_Reload, NULL),
+	GB_METHOD("Reload", NULL, WebView_Reload, "[(BypassCache)b]"),
 	GB_METHOD("Stop", NULL, WebView_Stop, NULL),
-	
+
 	GB_PROPERTY("NewView", "WebView", WebView_NewView),
 
 	GB_PROPERTY("Cookies", "Cookie[]", WebView_Cookies),
@@ -679,23 +794,35 @@ CWebView CWebView::manager;
 void CWebView::loadFinished(bool ok)
 {
 	GET_SENDER();
+
+	//fprintf(stderr, "loadFinished %d (%d)\n", ok, THIS->stopping);
+
 	THIS->progress = 1;
+
 	if (ok)
 		GB.Raise(THIS, EVENT_LOAD, 0);
-	else
+	else if (!THIS->stopping)
 		GB.RaiseLater(THIS, EVENT_ERROR);
 }
 
 void CWebView::loadProgress(int progress)
 {
 	GET_SENDER();
-	THIS->progress = progress / 100.0;
+
+	double v = progress / 100.0;
+	if (THIS->progress == v)
+		return;
+
+	THIS->progress = v;
 	GB.Raise(THIS, EVENT_PROGRESS, 0);
 }
 
 void CWebView::loadStarted()
 {
 	GET_SENDER();
+
+	//fprintf(stderr, "loadStarted\n");
+
 	THIS->progress = 0;
 	_network_access_manager_view = THIS;
 	GB.Raise(THIS, EVENT_PROGRESS, 0);
@@ -764,6 +891,7 @@ void CWebView::iconChanged()
 void CWebView::urlChanged(const QUrl &)
 {
 	QWebFrame *frame = (QWebFrame *)sender();
+	//fprintf(stderr, "urlChanged: %p\n", frame);
 	void *_object = QT.GetObject(frame->page()->view());
 	GB.Raise(THIS, EVENT_CLICK, 1, GB_T_OBJECT, CWEBFRAME_get(frame));
 }
@@ -784,11 +912,13 @@ void CWebView::handleUnsupportedContent(QNetworkReply *reply)
 	void *_object = QT.GetObject(((QWebPage*)sender())->view());
 	CWEBDOWNLOAD *download;
 	
-	if (reply->error() == QNetworkReply::NoError) 
+	if (reply->error() == QNetworkReply::NoError)
 	{
 		download = WEB_create_download(reply);
 		
 		if (GB.Raise(THIS, EVENT_DOWNLOAD, 1, GB_T_OBJECT, download) || !download->path || !*download->path)
 			WEB_remove_download(download);
 	}
+
+	delete reply;
 }
