@@ -39,6 +39,8 @@
 #include "cwebview.h"
 #include "main.h"
 
+GB_CLASS CLASS_WebView;
+
 extern "C" {
 
 GB_INTERFACE GB EXPORT;
@@ -46,28 +48,30 @@ QT_INTERFACE QT;
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
-	CWebDownloadDesc,
-	CWebDownloadsDesc,
-	CWebHitTestDesc,
-	CCookieDesc,
-	CWebSettingsIconDatabaseDesc,
-	CWebSettingsCacheDesc,
-	CWebSettingsFontsDesc,
-	CWebSettingsProxyDesc,
-	CWebSettingsDesc,
+	WebDownloadDesc,
+	WebDownloadsDesc,
+	WebHitTestDesc,
+	CookieDesc,
+	WebSettingsIconDatabaseDesc,
+	WebSettingsCacheDesc,
+	WebSettingsFontsDesc,
+	WebSettingsProxyDesc,
+	WebSettingsDesc,
 	WebElementStyleDesc,
 	WebElementDesc,
-	CWebFrameChildrenDesc,
-	CWebFrameDesc,
-	CWebViewSettingsDesc,
-  CWebViewAuthDesc,
-	CWebViewDesc,
+	WebFrameChildrenDesc,
+	WebFrameDesc,
+	WebViewSettingsDesc,
+  WebViewAuthDesc,
+	WebViewHistoryDesc,
+	WebViewDesc,
   NULL
 };
 
 int EXPORT GB_INIT(void)
 {
   GB.GetInterface("gb.qt4", QT_INTERFACE_VERSION, &QT);
+	CLASS_WebView = GB.FindClass("WebView");
   return 0;
 }
 
