@@ -39,6 +39,8 @@
 
 //static bool _accept_statement = FALSE;
 
+static bool _must_drop_vargs = FALSE;
+
 static short get_nparam(PATTERN *tree, int count, int *pindex, uint64_t *byref)
 {
 	PATTERN pattern;
@@ -156,6 +158,14 @@ static void push_string(int index, bool trans)
 	}
 }
 
+bool TRANS_string(PATTERN pattern)
+{
+	if (!PATTERN_is_string(pattern))
+		return TRUE;
+	push_string(PATTERN_index(pattern), FALSE);
+	return FALSE;
+}
+
 
 void TRANS_class(int index)
 {
@@ -225,6 +235,9 @@ static void trans_identifier(int index, bool point, PATTERN next)
 			is_static = TYPE_is_static(sym->global.type);
 			is_func = type == TK_FUNCTION;
 
+			if (is_func && PATTERN_is_point(next))
+				goto __CLASS;
+
 			if (!is_static && TYPE_is_static(JOB->func->type))
 				THROW("Dynamic symbols cannot be used in static function");
 
@@ -257,18 +270,24 @@ static void trans_subr(int subr, short nparam)
 {
 	SUBR_INFO *info = &COMP_subr_info[subr];
 
-	if (subr == SUBR_VarPtr)
-	{
-		if (CODE_check_varptr())
-			THROW("VarPtr() argument must be a dynamic, a static or a local variable");
-	}
-	
 	if (nparam < info->min_param)
 		THROW("Not enough arguments to &1()", info->name);
 	else if (nparam > info->max_param)
 		THROW("Too many arguments to &1()", info->name);
 
-	CODE_subr(info->opcode, nparam, info->optype, FALSE /* output */, (info->max_param == info->min_param));
+	if (subr == SUBR_VarPtr)
+	{
+		if (CODE_check_varptr())
+			THROW("VarPtr() argument must be a dynamic, a static or a local variable");
+	}
+	else if (subr == SUBR_IsMissing)
+	{
+		JOB->func->use_is_missing = TRUE;
+		if (CODE_check_ismissing())
+			THROW("IsMissing() requires a function argument");
+	}
+
+	CODE_subr(info->opcode, nparam, info->optype, info->max_param == info->min_param);
 }
 
 
@@ -325,6 +344,12 @@ static void trans_call(short nparam, uint64_t byref)
 	}
 		
 	CODE_call_byref(nparam, byref);
+
+	if (_must_drop_vargs)
+	{
+		CODE_drop_vargs();
+		_must_drop_vargs = FALSE;
+	}
 }
 
 static void trans_expr_from_tree(TRANS_TREE *tree, int count)
@@ -424,6 +449,11 @@ static void trans_expr_from_tree(TRANS_TREE *tree, int count)
 			{
 				CODE_push_inf(TRUE);
 			}
+			else if (PATTERN_is(pattern, RS_3PTS))
+			{
+				_must_drop_vargs = TRUE;
+				CODE_push_vargs();
+			}
 			else
 			{
 				op = PATTERN_index(pattern);
@@ -667,7 +697,8 @@ bool TRANS_affectation(bool dup)
 	int niv = 0;
 	bool equal = FALSE;
 	bool stat = FALSE;
-	int op;
+	int op = RS_NONE;
+	int id = RS_NONE;
 
 	for(;;)
 	{
@@ -718,6 +749,7 @@ bool TRANS_affectation(bool dup)
 			TRANS_in_affectation++;
 			TRANS_new();
 			TRANS_in_affectation--;
+			id = RS_NEW;
 			stat = TRUE;
 		}
 		else
@@ -726,10 +758,12 @@ bool TRANS_affectation(bool dup)
 			{
 				if (TRANS_is(st->id))
 				{
+					id = st->id;
 					TRANS_in_affectation++;
 					(*st->func)();
 					TRANS_in_affectation--;
 					stat = TRUE;
+					break;
 				}
 			}
 		}
@@ -760,6 +794,9 @@ bool TRANS_affectation(bool dup)
 	if (dup)
 		CODE_dup();
 
+	if (id == RS_EXEC || id == RS_SHELL)
+		CODE_dup();
+
 	JOB->current = left;
 	TRANS_reference();
 
@@ -768,6 +805,12 @@ bool TRANS_affectation(bool dup)
 
 	JOB->current = after;
 
+	if (id == RS_EXEC || id == RS_SHELL)
+	{
+		TRANS_subr(TS_SUBR_CHECK_EXEC, 1);
+		CODE_drop();
+	}
+
 	return TRUE;
 }
 
