@@ -45,7 +45,7 @@
 
 #include <dlfcn.h>
 
-#if defined(OS_LINUX) || defined(OS_OPENBSD) || defined(OS_FREEBSD) || defined(OS_CYGWIN)
+#if defined(OS_LINUX) || defined(OS_GNU) || defined(OS_OPENBSD) || defined(OS_FREEBSD) || defined(OS_CYGWIN)
 	#define lt_dlinit() (0)
 	#define lt_dlhandle void *
 	#define lt_dlopenext(_path) dlopen(_path, RTLD_LAZY)
@@ -64,6 +64,7 @@
 #include "gb_arch.h"
 #include "gb_common_swap.h"
 #include "gb_array.h"
+#include "gb_table.h"
 #include "gambas.h"
 
 static char _root[PATH_MAX + 1] = { 0 };
@@ -83,6 +84,8 @@ static bool _no_include_warning = TRUE;
 
 static char **_components = NULL;
 
+static TABLE *_classes = NULL;
+
 static void analyze(const char *comp, bool include);
 
 #if HAVE_GETOPT_LONG
@@ -284,6 +287,22 @@ static int sort_symbol(const int *a, const int *b)
 	return strcmp(_sort_symbol[*a].name, _sort_symbol[*b].name);
 }
 
+static void add_class(const char *name, bool has_static)
+{
+	int index;
+
+	if (out_list)
+	{
+		if (!TABLE_add_symbol(_classes, name, strlen(name), &index))
+		{
+			fputs(name, out_list);
+			if (has_static)
+				fputc('!', out_list);
+			fputc('\n', out_list);
+		}
+	}
+}
+
 static void analyze_class(GB_DESC *desc)
 {
 	const char *name = desc->name;
@@ -295,9 +314,7 @@ static void analyze_class(GB_DESC *desc)
 	int *sort;
 	GB_DESC *p;
 	int i;
-
-	if (out_list) // && name[0] != '.')
-		fprintf(out_list, "%s\n", name);
+	bool has_static = FALSE;
 
 	desc++;
 
@@ -307,7 +324,7 @@ static void analyze_class(GB_DESC *desc)
 
 		if (hook == (intptr_t)GB_INHERITS_ID)
 			parent = (char *)desc->val1;
-		else if ((hook == (intptr_t)GB_NOT_CREATABLE_ID))
+		else if (hook == (intptr_t)GB_NOT_CREATABLE_ID)
 			nocreate = TRUE;
 		else if (hook == (intptr_t)GB_AUTO_CREATABLE_ID)
 			autocreate = TRUE;
@@ -353,12 +370,19 @@ static void analyze_class(GB_DESC *desc)
 	qsort(sort, nsymbol, sizeof(int), (int (*)(const void *, const void *))sort_symbol);
 
 	for (i = 0; i < nsymbol; i++)
-		dump_symbol(&desc[sort[i]]);
+	{
+		p = &desc[sort[i]];
+		if (strchr("PCMR", *p->name) && p->name[1] != '_')
+			has_static = TRUE;
+		dump_symbol(p);
+	}
 
 	FREE(&sort);
 	
 	if (_format)
 		newline();
+
+	add_class(name, has_static);
 }
 
 
@@ -453,7 +477,7 @@ static bool analyze_native_component(const char *path)
 		//ret = TRUE;
 	}
 
-	// Do not close shared libraries,except on openbsd that seems to feel better with
+	// Do not close shared libraries, except on openbsd that seems to feel better with
 	#ifdef OS_OPENBSD
 	lt_dlclose(lib);
 	#endif
@@ -467,15 +491,34 @@ static bool analyze_gambas_component(const char *path)
 	ARCH *arch;
 	ARCH_FIND find;
 	bool ret = TRUE;
+	char *buffer;
+	char *line;
 
 	if (_verbose)
 		fprintf(stderr, "Loading gambas component: %s\n", path);
 
 	arch = ARCH_open(path);
 
+	if (!ARCH_find(arch, ".component", 0, &find))
+	{
+		ALLOC(&buffer, find.len + 1);
+		memcpy(buffer, &arch->addr[find.pos], find.len);
+		buffer[find.len] = 0;
+
+		for (line = strtok(buffer, "\n"); line; line = strtok(NULL, "\n"))
+		{
+			if (strncmp(line, "Include=", 8))
+				continue;
+			analyze_include(&line[8]);
+			break;
+		}
+
+		FREE(&buffer);
+	}
+
 	if (ARCH_find(arch, ".info", 0, &find))
 	{
-		warning(".info file not found in component archive.");
+		warning("'.info' file not found in component archive.");
 		goto __RETURN;
 	}
 
@@ -483,11 +526,20 @@ static bool analyze_gambas_component(const char *path)
 
 	if (ARCH_find(arch, ".list", 0, &find))
 	{
-		warning(".list file not found in component archive.");
+		warning("'.list' file not found in component archive.");
 		goto __RETURN;
 	}
 
-	fwrite(&arch->addr[find.pos], 1, find.len, out_list);
+	ALLOC(&buffer, find.len + 1);
+	memcpy(buffer, &arch->addr[find.pos], find.len);
+	buffer[find.len] = 0;
+
+	for (line = strtok(buffer, "\n"); line; line = strtok(NULL, "\n"))
+		add_class(line, FALSE);
+
+	FREE(&buffer);
+
+	//fwrite(&arch->addr[find.pos], 1, find.len, out_list);
 	ret = FALSE;
 
 __RETURN:
@@ -564,11 +616,14 @@ static void analyze(const char *comp, bool include)
 			error(FALSE, "Cannot write file: %s", path_list);
 			return;
 		}
+
+		TABLE_create(&_classes, sizeof(SYMBOL), TF_IGNORE_CASE);
 	}
 	
 	fflush(stdout);
 	ok = TRUE;
 
+
 	if (native)
 	{
 		snprintf(_buffer, sizeof(_buffer), LIB_PATTERN, _lib_path, name);
@@ -588,6 +643,8 @@ static void analyze(const char *comp, bool include)
 
 	if (!include)
 	{
+		TABLE_delete(&_classes);
+
 		fclose(out_info);
 		fclose(out_list);
 	
@@ -598,8 +655,8 @@ static void analyze(const char *comp, bool include)
 		}
 		else if (_verbose)
 		{
-			fprintf(stderr, "wrote %s\n", path_info);
-			fprintf(stderr, "wrote %s\n", path_list);
+			fprintf(stderr, "Wrote %s\n", path_info);
+			fprintf(stderr, "Wrote %s\n", path_list);
 		}
 
 		STR_free(path_info);
@@ -747,7 +804,7 @@ int main(int argc, char **argv)
 				printf(
 					"\nGenerate component description files.\n"
 					"\nUsage: gbi" GAMBAS_VERSION_STRING " [options] [components]\n"
-					"Options:"
+					"\nOptions:"
 					#if HAVE_GETOPT_LONG
 					"\n"
 					#if DO_PRELOADING
@@ -779,7 +836,11 @@ int main(int argc, char **argv)
 	if (_analyze)
 	{
 		if (_verbose)
-			fprintf(stderr, "LD_PRELOAD=%s\n", getenv("LD_PRELOAD"));
+		{
+			char *env = getenv("LD_PRELOAD");
+			if (env)
+				fprintf(stderr, "LD_PRELOAD=%s\n", env);
+		}
 
 		analyze(argv[optind], FALSE);
 		/*if (strcmp(argv[optind], "gb.qt4") == 0)
