@@ -131,7 +131,7 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_Is,
 	(void *)GB_Ref,
 	(void *)GB_Unref,
-	(void *)GB_UnrefKeep,
+	//(void *)GB_UnrefKeep,
 	(void *)GB_Detach,
 	(void *)GB_Attach,
 	(void *)OBJECT_parent,
@@ -199,6 +199,7 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_TempFile,
 	(void *)GB_CopyFile,
 	(void *)GB_BrowseProject,
+	(void *)GB_BrowseDirectory,
 	(void *)GB_StatFile,
 
 	(void *)GB_Store,
@@ -220,6 +221,7 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_Eval,
 
 	(void *)GB_Alloc,
+	(void *)GB_AllocZero,
 	(void *)GB_Free,
 	(void *)GB_Realloc,
 
@@ -248,6 +250,7 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_SystemPath,
 	(void *)FILE_init,
 	(void *)GB_SystemDebug,
+	(void *)PROJECT_get_home,
 
 	(void *)GB_ArrayNew,
 	(void *)GB_ArrayCount,
@@ -314,6 +317,9 @@ const void *GAMBAS_DebugApi[] =
 	(void *)DEBUG_get_object_access_type,
 	(void *)DEBUG_find_class,
 	(void *)CARRAY_get_array_bounds,
+	(void *)GB_DebugBreakOnError,
+	(void *)DEBUG_enter_eval,
+	(void *)DEBUG_leave_eval,
 	NULL
 };
 
@@ -634,7 +640,7 @@ static CLASS_DESC *get_desc(CLASS *class, const char *name)
 		return class->table[index].desc;
 }
 
-void GB_GetProperty(void *object, const char *name)
+bool GB_GetProperty(void *object, const char *name)
 {
 	CLASS_DESC *desc;
 	CLASS *class;
@@ -656,7 +662,7 @@ void GB_GetProperty(void *object, const char *name)
 	desc = get_desc(class, name);
 
 	if (!desc)
-		return;
+		return TRUE;
 
 	type = CLASS_DESC_get_type(desc);
 
@@ -665,7 +671,7 @@ void GB_GetProperty(void *object, const char *name)
 		if (!object)
 		{
 			error(E_DYNAMIC, class, name);
-			return;
+			return TRUE;
 		}
 	}
 	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_PROPERTY_READ || type == CD_STATIC_VARIABLE || type == CD_CONSTANT)
@@ -673,13 +679,13 @@ void GB_GetProperty(void *object, const char *name)
 		if (object)
 		{
 			error(E_STATIC, class, name);
-			return;
+			return TRUE;
 		}
 	}
 	else
 	{
 		error(E_NPROPERTY, class, name);
-		return;
+		return TRUE;
 	}
 
 	if (type == CD_VARIABLE)
@@ -695,7 +701,7 @@ void GB_GetProperty(void *object, const char *name)
 			if (EXEC_call_native(desc->property.read, object, desc->property.type, 0))
 			{
 				EXEC_set_native_error(TRUE);
-				return;
+				return TRUE;
 			}
 		}
 		else
@@ -714,28 +720,102 @@ void GB_GetProperty(void *object, const char *name)
 			RP->type = T_VOID;
 		}
 	}
+
+	return FALSE;
 }
 
-void GB_SetProperty(void *object, const char *property, ...)
+bool GB_SetProperty(void *object, const char *name, GB_VALUE *val)
 {
-	static bool init = FALSE;
-	static GB_FUNCTION func;
-	
-	va_list args;
-	
-	if (!init)
-	{
-		GB_GetFunction(&func, (void *)CLASS_find("Object"), "SetProperty", NULL, NULL);
-		init = TRUE;
-	}
-	
-	GB_Push(2, GB_T_OBJECT, object, GB_T_STRING, property, strlen(property));
+	VALUE *value = (VALUE *)val;
+	CLASS_DESC *desc;
+	CLASS *class;
+	char type;
 
-	va_start(args, property);
-	push(1, args);
-	va_end(args);
-	
-	GB_Call(&func, 3, TRUE);
+	if (OBJECT_is_class(object))
+	{
+		class = (CLASS *)object;
+		object = NULL;
+	}
+	else
+	{
+		class = OBJECT_class(object);
+	}
+
+	desc = get_desc(class, name);
+
+	if (!desc)
+		return TRUE;
+
+	type = CLASS_DESC_get_type(desc);
+
+	if (type == CD_PROPERTY || type == CD_VARIABLE)
+	{
+		if (!object)
+		{
+			if (!class->auto_create)
+			{
+				error(E_DYNAMIC, class, name);
+				return TRUE;
+			}
+
+			object = EXEC_auto_create(class, TRUE);
+		}
+	}
+	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_VARIABLE)
+	{
+		if (object)
+		{
+			error(E_STATIC, class, name);
+			return TRUE;
+		}
+	}
+	else if (type == CD_PROPERTY_READ  || type == CD_STATIC_PROPERTY_READ)
+	{
+		error(E_NWRITE, class, name);
+		return TRUE;
+	}
+	else
+	{
+		error(E_NPROPERTY, class, name);
+		return TRUE;
+	}
+
+	if (type == CD_VARIABLE)
+		VALUE_write(value, (char *)object + desc->variable.offset, desc->variable.type);
+	else if (type == CD_STATIC_VARIABLE)
+		VALUE_write(value, (char *)class->stat + desc->variable.offset, desc->variable.type);
+	else
+	{
+		if (desc->property.native)
+		{
+			VALUE_conv(value, desc->property.type);
+
+			if (EXEC_call_native(desc->property.write, object, 0, value))
+			{
+				EXEC_set_native_error(TRUE);
+				return TRUE;
+			}
+		}
+		else
+		{
+			*SP = *value;
+			BORROW(SP);
+			SP++;
+
+			EXEC.class = desc->property.class;
+			EXEC.object = object;
+			EXEC.nparam = 1;
+			EXEC.native = FALSE;
+			EXEC.index = (int)(intptr_t)desc->property.write;
+			//EXEC.func = &class->load->func[(long)desc->property.write];
+
+			EXEC_function();
+
+			/*VALUE_write(value, OBJECT_get_prop_addr(object, desc), desc->property.type);*/
+		}
+	}
+
+	return FALSE;
 }
 
 
@@ -750,11 +830,11 @@ bool GB_CanRaise(void *object, int event_id)
 
 	LIST_for_each(obs, OBJECT_event(object)->observer)
 	{
-		if (OBJECT_active_parent(obs) && obs->event && obs->event[event_id])
+		if (OBJECT_parent(obs) && obs->event && obs->event[event_id])
 			return TRUE;
 	}
 	
-	if (!OBJECT_active_parent(object))
+	if (!OBJECT_parent(object))
 		return FALSE;
 	
 	event_tab = OBJECT_event(object)->event;
@@ -931,7 +1011,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 	
 		parent = OBJECT_active_parent(object);
 		if (parent)
-		{	
+		{
 			func_id = get_event_func_id(OBJECT_event(object)->event, event_id);
 			if (func_id)
 			{
@@ -943,7 +1023,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 						printf("parent is locked!\n");
 					fflush(NULL);
 				#endif
-			
+
 				if (!OBJECT_is_valid(parent))
 					OBJECT_detach(object);
 				else
@@ -1048,7 +1128,7 @@ bool GB_GetFunction(GB_FUNCTION *_func, void *object, const char *name, const ch
 			tsign = TYPE_transform_signature(&tsign, sign, nparam);
 		}
 
-		error = TYPE_compare_signature(desc->method.signature, desc->method.npmax, tsign, nparam);
+		error = TYPE_compare_signature(desc->method.signature, desc->method.npmax, tsign, nparam, FALSE);
 
 		if (nparam)
 			FREE(&tsign);
@@ -1183,12 +1263,6 @@ int GB_NParam(void)
 }
 
 
-/*bool GB_IsProperty(void)
-{
-	return FALSE; //EXEC_unknown_property;
-}*/
-
-
 const char *GB_GetUnknown(void)
 {
 	return EXEC_unknown_name;
@@ -1262,8 +1336,10 @@ static void print_stack_backtrace()
 	trace_size = backtrace(trace, 16);
 	messages = backtrace_symbols(trace, trace_size);
 	//printf("[bt] Execution path:\n");
+	fprintf(stderr, "[ ");
 	for (i=0; i<trace_size; ++i)
-		fprintf(stderr, "[bt] %s\n", messages[i]);
+		fprintf(stderr, "%s ", messages[i]);
+	fprintf(stderr, "] ");
 }
 
 #endif
@@ -1279,7 +1355,7 @@ void GB_Ref(void *object)
 	if (object)
 	{
 		#if DEBUG_REF
-		//print_stack_backtrace();
+		print_stack_backtrace();
 		#endif
 		OBJECT_REF(object);
 	}
@@ -1300,7 +1376,7 @@ void GB_Unref(void **object)
 	if (*object)
 	{
 		#if DEBUG_REF
-		//print_stack_backtrace();
+		print_stack_backtrace();
 		#endif
 		OBJECT_UNREF(*object);
 	}
@@ -1311,7 +1387,7 @@ void GB_Unref(void **object)
 }
 
 
-void GB_UnrefKeep(void **object, int delete)
+/*void GB_UnrefKeep(void **object, int delete)
 {
 	#if TRACE_MEMORY
 	CLASS *save = CP;
@@ -1336,7 +1412,7 @@ void GB_UnrefKeep(void **object, int delete)
 	#if TRACE_MEMORY
 	CP = save;
 	#endif
-}
+}*/
 
 
 void GB_Detach(void *object)
@@ -1951,6 +2027,11 @@ void GB_Alloc(void **addr, int len)
 	ALLOC(addr, len);
 }
 
+void GB_AllocZero(void **addr, int len)
+{
+	ALLOC_ZERO(addr, len);
+}
+
 void GB_Free(void **addr)
 {
 	FREE(addr);
@@ -2020,7 +2101,7 @@ bool GB_HashTableGet(GB_HASHTABLE hash, const char *key, int len, void **data)
 	if (len <= 0)
 		len = strlen(key);
 
-	pdata = (void **)HASH_TABLE_lookup((HASH_TABLE *)hash, key, len);
+	pdata = (void **)HASH_TABLE_lookup((HASH_TABLE *)hash, key, len, FALSE);
 	if (pdata)
 	{
 		*data = *pdata;
@@ -2040,7 +2121,7 @@ void GB_HashTableEnum(GB_HASHTABLE hash, GB_HASHTABLE_ENUM_FUNC func)
 
 	for(;;)
 	{
-		data = (void **)HASH_TABLE_next((HASH_TABLE *)hash, &iter);
+		data = (void **)HASH_TABLE_next((HASH_TABLE *)hash, &iter, FALSE);
 		if (!data)
 			break;
 
@@ -2305,7 +2386,7 @@ char *GB_RealFileName(const char *name, int len)
 	return real;
 }
 
-static GB_BROWSE_CALLBACK _browse_project_func;
+static GB_BROWSE_PROJECT_CALLBACK _browse_project_func;
 
 static void project_file_found(const char *path)
 {
@@ -2315,7 +2396,7 @@ static void project_file_found(const char *path)
 	(*_browse_project_func)(path, info.size);
 }
 
-void GB_BrowseProject(GB_BROWSE_CALLBACK func)
+void GB_BrowseProject(GB_BROWSE_PROJECT_CALLBACK func)
 {
 	ARCHIVE *arch = NULL;
 	
@@ -2329,6 +2410,11 @@ void GB_BrowseProject(GB_BROWSE_CALLBACK func)
 		ARCHIVE_browse(arch, func);
 }
 
+void GB_BrowseDirectory(const char *dir, GB_BROWSE_CALLBACK before, GB_BROWSE_CALLBACK after)
+{
+	FILE_recursive_dir(dir, before, after, 0, FALSE);
+}
+
 
 const char *GB_CurrentComponent()
 {
@@ -2344,6 +2430,11 @@ void *GB_DebugGetExec(void)
 	return &EXEC_current;
 }
 
+void GB_DebugBreakOnError(bool b)
+{
+	EXEC_break_on_error = b;
+}
+
 bool GB_SystemDebug(void)
 {
 	return EXEC_debug;
