@@ -47,10 +47,9 @@ bool GB_Stopped(void);
 
 int GB_NParam(void);
 const char *GB_GetUnknown(void);
-//bool GB_IsProperty(void);
 
-void GB_GetProperty(void *object, const char *property);
-void GB_SetProperty(void *object, const char *property, ...);
+bool GB_GetProperty(void *object, const char *property);
+bool GB_SetProperty(void *object, const char *name, GB_VALUE *value);
 
 bool GB_Serialize(const char *path, GB_VALUE *value);
 bool GB_UnSerialize(const char *path, GB_VALUE *value);
@@ -115,7 +114,8 @@ char *GB_TempFile(const char *pattern);
 bool GB_CopyFile(const char *src, const char *dst);
 //int GB_FindFile(const char *dir, int recursive, int follow, void (*found)(const char *));
 bool GB_StatFile(const char *path, GB_FILE_STAT *info, bool follow);
-void GB_BrowseProject(GB_BROWSE_CALLBACK func);
+void GB_BrowseProject(GB_BROWSE_PROJECT_CALLBACK func);
+void GB_BrowseDirectory(const char *dir, GB_BROWSE_CALLBACK before, GB_BROWSE_CALLBACK after);
 
 int GB_IsMissing(int param);
 
@@ -167,6 +167,7 @@ bool GB_CollectionGet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *v
 bool GB_CollectionEnum(GB_COLLECTION col, GB_COLLECTION_ITER *iter, GB_VARIANT *value, char **key, int *len);
 
 void GB_Alloc(void **addr, int len);
+void GB_AllocZero(void **addr, int len);
 void GB_Free(void **addr);
 void GB_Realloc(void **addr, int len);
 
@@ -203,6 +204,7 @@ int GB_toupper(int c);
 
 void *GB_DebugGetClass(const char *name);
 void *GB_DebugGetExec(void);
+void GB_DebugBreakOnError(bool);
 
 #define GB_PrintString PRINT_string
 
