@@ -531,7 +531,7 @@ BEGIN_METHOD_VOID(Array_free)
 
 	if (UNLIKELY(THIS->ref != NULL))
 	{
-		GB_Unref(&THIS->ref);
+		OBJECT_UNREF(THIS->ref);
 		return;
 	}
 	
@@ -654,14 +654,10 @@ BEGIN_METHOD(Array_Extract, GB_INTEGER start; GB_INTEGER length)
 END_METHOD
 
 
-BEGIN_METHOD(Array_Resize, GB_INTEGER size)
-
-	int size = VARG(size);
+void CARRAY_resize(CARRAY *_object, int size)
+{
 	int count = THIS->count;
 
-	if (check_not_multi(THIS))
-		return;
-
 	if (size > count)
 	{
 		ARRAY_add_many_void(&THIS->data, size - count);
@@ -673,6 +669,17 @@ BEGIN_METHOD(Array_Resize, GB_INTEGER size)
 	}
 
 	THIS->count = size;
+}
+
+
+BEGIN_METHOD(Array_Resize, GB_INTEGER size)
+
+	int size = VARG(size);
+
+	if (check_not_multi(THIS))
+		return;
+
+	CARRAY_resize(THIS, size);
 	
 END_METHOD
 
@@ -1100,9 +1107,9 @@ END_METHOD
 BEGIN_METHOD(Array_String_join, GB_STRING sep; GB_STRING esc)
 
 	char *sep = ",";
-	int lsep = 1;
+	uint lsep = 1;
 	char *esc = "";
-	int lesc = 0;
+	uint lesc = 0;
 	char escl, NO_WARNING(escr);
 	int i;
 	char **data = (char **)THIS->data;
@@ -1129,7 +1136,7 @@ BEGIN_METHOD(Array_String_join, GB_STRING sep; GB_STRING esc)
 		}
 	}
 
-	if (!lesc)
+	if (lesc == 0)
 	{
 		max = 0;
 		for (i = 0; i < THIS->count; i++)
@@ -1138,42 +1145,84 @@ BEGIN_METHOD(Array_String_join, GB_STRING sep; GB_STRING esc)
 			max -= lsep;
 		
 		STRING_start_len(max);
+
+		for (i = 0; i < THIS->count; i++)
+		{
+			p = data[i];
+			l = STRING_length(data[i]);
+
+			if (i)
+				STRING_make(sep, lsep);
+			if (l)
+				STRING_make(p, l);
+		}
+	}
+	else if (*sep && escr == *sep)
+	{
+		STRING_start();
+
+		for (i = 0; i < THIS->count; i++)
+		{
+			p = data[i];
+			l = STRING_length(data[i]);
+
+			if (i)
+				STRING_make(sep, lsep);
+
+			if (l == 0)
+				continue;
+
+			for(;;)
+			{
+				p2 = index(p, escr);
+				if (p2)
+				{
+					STRING_make(p, p2 - p);
+					STRING_make_char(escl);
+					STRING_make_char(escr);
+					p = p2 + 1;
+				}
+				else
+				{
+					STRING_make(p, l + data[i] - p);
+					break;
+				}
+			}
+		}
 	}
 	else
-		STRING_start();
-	
-	for (i = 0; i < THIS->count; i++)
 	{
-		p = data[i];
-		l = STRING_length(data[i]);
-		
-		if (i)
-			STRING_make(sep, lsep);
-		if (lesc)
+		STRING_start();
+
+		for (i = 0; i < THIS->count; i++)
 		{
-			STRING_make(&escl, 1);
-			if (l)
+			p = data[i];
+			l = STRING_length(data[i]);
+
+			if (i)
+				STRING_make(sep, lsep);
+
+			if (l == 0)
+				continue;
+
+			STRING_make_char(escl);
+			for(;;)
 			{
-				for(;;)
+				p2 = index(p, escr);
+				if (p2)
 				{
-					p2 = index(p, escr);
-					if (p2)
-					{
-						STRING_make(p, p2 - p + 1);
-						STRING_make_char(escr);
-						p = p2 + 1;
-					}
-					else
-					{
-						STRING_make(p, l + data[i] - p);
-						break;
-					}
+					STRING_make(p, p2 - p + 1);
+					STRING_make_char(escr);
+					p = p2 + 1;
+				}
+				else
+				{
+					STRING_make(p, l + data[i] - p);
+					break;
 				}
 			}
 			STRING_make_char(escr);
 		}
-		else if (l)
-			STRING_make(p, l);
 	}
 
 	GB_ReturnString(STRING_end_temp());
@@ -1970,176 +2019,6 @@ GB_DESC NATIVE_TemplateArrayOfStruct[ARRAY_OF_STRUCT_TEMPLATE_NDESC] =
 
 #ifndef GBX_INFO
 
-
-static CARRAY *_array;
-static bool _novoid;
-static char *_entry;
-static const char *_ptr;
-static int _lptr;
-
-static void add_char_real(const char *p)
-{
-	//fprintf(stderr, "add_char: %p %c\n", p, p ? *p : 0);
-	
-	/*if (p && p == (_ptr + _lptr))
-	{
-		_lptr++;
-		return;
-	}*/
-	
-	if (_lptr)
-	{
-		//fprintf(stderr, "STRING_add: '%.*s'\n", _lptr, _ptr);
-		_entry = STRING_add(_entry, _ptr, _lptr);
-	}
-	
-	_ptr = p;
-	_lptr = p ? 1 : 0;
-}
-
-#define add_char(_p) \
-({ \
-	if ((_p) && (_p) == (_ptr + _lptr)) \
-		_lptr++; \
-	else \
-		add_char_real(_p); \
-})
-
-static void add_entry()
-{
-	add_char_real(NULL);
-	
-	if (!_entry)
-	{
-		if (!_novoid)
-			ARRAY_add_void_size(&_array->data);
-	}
-	else
-	{
-		*((char **)ARRAY_add_size(&_array->data)) = _entry;
-		_entry = NULL;
-	}
-	
-	//fprintf(stderr, "** add_entry\n");
-}
-
-
-void CARRAY_split(CARRAY *_object, const char *str, int lstr, const char *sep, const char *esc, bool no_void, bool keep_esc)
-{
-	int i;
-	char c;
-	bool escape;
-	char escl, escr;
-	bool lsep;
-	
-	if (sep == NULL || *sep == 0)
-		sep = ",";
-
-	lsep = sep[1];
-
-	clear(THIS);
-	
-	_array = _object;
-	_entry = NULL;
-	_novoid = no_void;
-	_ptr = NULL;
-	_lptr = 0;
-	
-	if (esc == NULL || *esc == 0)
-	{
-		i = lstr;
-		
-		if (!lsep)
-		{
-			char csep = sep[0];
-			
-			while (i--)
-			{
-				if (*str == csep)
-					add_entry();
-				else
-				{
-					//add_char(str);
-					if (!_lptr) _ptr = str;
-					_lptr++;
-				}
-					
-				str++;
-			}		
-		}
-		else
-		{
-			while (i--)
-			{
-				if (index(sep, *str))
-					add_entry();
-				else
-				{
-					//add_char(str);
-					if (!_lptr) _ptr = str;
-					_lptr++;
-				}
-					
-				str++;
-			}
-		}
-		
-		add_entry();
-	}
-	else
-	{
-		escl = *esc++;
-		if (*esc)
-			escr = *esc;
-		else
-			escr = escl;
-
-		escape = FALSE;
-		
-		for (i = 0; i < lstr; i++)
-		{
-			c = *str;
-			
-			if (escape)
-			{
-				if (c != escr)
-					add_char(str);
-				else if ((i < (lstr - 1)) && str[1] == escr)
-				{
-					add_char(str);
-					str++;
-					i++;
-				}
-				else
-				{
-					escape = FALSE;
-					if (keep_esc)
-						add_char(str);
-				}
-			}
-			else if (c == escl)
-			{
-				escape = TRUE;
-				if (keep_esc)
-					add_char(str);
-			}
-			else if (c == *sep || (lsep && index(&sep[1], c)))
-			{
-				add_entry();
-			}
-			else
-				add_char(str);
-				
-			str++;
-		}
-		
-		add_entry();
-	}
-	
-	_object->count = ARRAY_count(_object->data);
-	_array = NULL;
-}
-
 /* Gambas API */
 
 void GB_ArrayNew(GB_ARRAY *array, TYPE type, int size)
