@@ -49,23 +49,6 @@ static void error(int code, CLASS *class, const char *name)
 	GB_Error((char *)(intptr_t)code, CLASS_get_name(class), name);
 }
 
-static CLASS_DESC *get_desc(CLASS *class, const char *name)
-{
-	int index;
-
-	index = CLASS_find_symbol(class, name);
-
-	//fprintf(stderr, "CLASS_find_symbol('%s', '%s') = %ld\n", CLASS_get_name(class), name, index);
-
-	if (index == NO_SYMBOL)
-	{
-		error(E_NSYMBOL, class, name);
-		return NULL;
-	}
-	else
-		return class->table[index].desc;
-}
-
 //---- Components ---------------------------------------------------------
 
 BEGIN_METHOD(Components_get, GB_STRING name)
@@ -150,7 +133,7 @@ BEGIN_METHOD(Classes_get, GB_STRING name)
 		return;
 	}
 
-	if (!class->state)
+	if (!CLASS_is_loaded(class))
 	{
 		GB_Error("Class is not loaded");
 		return;
@@ -170,33 +153,6 @@ BEGIN_METHOD(Class_Load, GB_STRING name)
 
 END_METHOD
 
-/*
-BEGIN_METHOD_VOID(Classes_next)
-
-	TABLE *table = CLASS_get_table();
-	int *index = (int *)GB_GetEnum();
-	CLASS *class;
-
-	for(;;)
-	{
-		if (*index >= TABLE_count(table))
-		{
-			GB_StopEnum();
-			break;
-		}
-
-		class = ((CLASS_SYMBOL *)TABLE_get_symbol(table, *index))->class;
-		(*index)++;
-
-		if (class->state)
-		{
-			GB_ReturnObject(class);
-			break;
-		}
-	}
-
-END_METHOD
-*/
 
 BEGIN_METHOD_VOID(Classes_next)
 
@@ -218,7 +174,7 @@ BEGIN_METHOD_VOID(Classes_next)
 			break;
 		}
 
-		if (class->state)
+		if (CLASS_is_loaded(class))
 		{
 			GB_ReturnObject(class);
 			break;
@@ -533,99 +489,14 @@ END_METHOD
 BEGIN_METHOD(Object_SetProperty, GB_OBJECT object; GB_STRING property; GB_VARIANT value)
 
 	const char *name;
-	CLASS_DESC *desc;
-	CLASS *class;
-	char type;
 	void *object = VARG(object);
-	VALUE *value = (VALUE *)ARG(value);
 
 	if (GB_CheckObject(object))
 		return;
 
-	if (OBJECT_is_class(object))
-	{
-		class = (CLASS *)object;
-		object = NULL;
-	}
-	else
-	{
-		class = OBJECT_class(object);
-	}
-
 	name = GB_ToZeroString(ARG(property));
-	desc = get_desc(class, name);
 
-	if (!desc)
-		return;
-
-	type = CLASS_DESC_get_type(desc);
-
-	if (type == CD_PROPERTY || type == CD_VARIABLE)
-	{
-		if (!object)
-		{
-			if (!class->auto_create)
-			{
-				error(E_DYNAMIC, class, name);
-				return;
-			}
-			
-			object = EXEC_auto_create(class, TRUE);
-		}
-	}
-	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_VARIABLE)
-	{
-		if (object)
-		{
-			error(E_STATIC, class, name);
-			return;
-		}
-	}
-	else if (type == CD_PROPERTY_READ  || type == CD_STATIC_PROPERTY_READ)
-	{
-		error(E_NWRITE, class, name);
-		return;
-	}
-	else
-	{
-		error(E_NPROPERTY, class, name);
-		return;
-	}
-
-	if (type == CD_VARIABLE)
-		VALUE_write(value, (char *)object + desc->variable.offset, desc->variable.type);
-	else if (type == CD_STATIC_VARIABLE)
-		VALUE_write(value, (char *)class->stat + desc->variable.offset, desc->variable.type);
-	else
-	{
-		if (desc->property.native)
-		{
-			VALUE_conv(value, desc->property.type);
-	
-			if (EXEC_call_native(desc->property.write, object, 0, value))
-			{
-				EXEC_set_native_error(TRUE);
-				return;
-			}
-		}
-		else
-		{
-			*SP = *value;
-			BORROW(SP);
-			SP++;
-	
-			EXEC.class = desc->property.class;
-			EXEC.object = object;
-			EXEC.nparam = 1;
-			EXEC.native = FALSE;
-			EXEC.index = (int)(intptr_t)desc->property.write;
-			//EXEC.func = &class->load->func[(long)desc->property.write];
-	
-			EXEC_function();
-	
-			/*VALUE_write(value, OBJECT_get_prop_addr(object, desc), desc->property.type);*/
-		}
-	}
+	GB_SetProperty(object, name, (GB_VALUE *)ARG(value));
 
 END_METHOD
 
@@ -906,7 +777,7 @@ END_METHOD
 
 GB_DESC NATIVE_Symbol[] =
 {
-	GB_DECLARE(".Symbol", 0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE_VIRTUAL(".Symbol"),
 
 	GB_PROPERTY_READ("Name", "s", Symbol_Name),
 	GB_PROPERTY_READ("Kind", "i", Symbol_Kind),
@@ -922,7 +793,7 @@ GB_DESC NATIVE_Symbol[] =
 
 GB_DESC NATIVE_Components[] =
 {
-	GB_DECLARE("Components", 0),  GB_VIRTUAL_CLASS(),
+	GB_DECLARE_STATIC("Components"),
 
 	GB_STATIC_METHOD("_next", "Component", Components_next, NULL),
 	GB_STATIC_METHOD("_get", "Component", Components_get, "(Name)s"),
@@ -933,7 +804,8 @@ GB_DESC NATIVE_Components[] =
 
 GB_DESC NATIVE_Component[] =
 {
-	GB_DECLARE("Component", 0),  GB_VIRTUAL_CLASS(),
+	GB_DECLARE("Component", sizeof(COMPONENT)),
+	GB_NOT_CREATABLE(),
 
 	//GB_STATIC_METHOD("_get", "Component", library_get, "(Name)s"),
 	GB_STATIC_METHOD("Load", "Component", Component_Load, "(Name)s"),
@@ -947,7 +819,7 @@ GB_DESC NATIVE_Component[] =
 
 GB_DESC NATIVE_Classes[] =
 {
-	GB_DECLARE("Classes", 0),  GB_VIRTUAL_CLASS(),
+	GB_DECLARE_STATIC("Classes"),
 
 	GB_STATIC_METHOD("_next", "Class", Classes_next, NULL),
 	GB_STATIC_METHOD("_get", "Class", Classes_get, "(Name)s"),
@@ -958,7 +830,8 @@ GB_DESC NATIVE_Classes[] =
 
 GB_DESC NATIVE_Class[] =
 {
-	GB_DECLARE("Class", 0),  GB_VIRTUAL_CLASS(),
+	GB_DECLARE("Class", sizeof(CLASS)),
+	GB_NOT_CREATABLE(),
 
 	GB_STATIC_METHOD("Load", "Class", Class_Load, "(Name)s"),
 
@@ -987,7 +860,7 @@ GB_DESC NATIVE_Class[] =
 
 GB_DESC NATIVE_Object[] =
 {
-	GB_DECLARE("Object", 0),  GB_VIRTUAL_CLASS(),
+	GB_DECLARE_STATIC("Object"),
 
 	GB_STATIC_METHOD("GetProperty", "v", Object_GetProperty, "(Object)o(Property)s"),
 	GB_STATIC_METHOD("SetProperty", NULL, Object_SetProperty, "(Object)o(Property)s(Value)v"),
