@@ -44,6 +44,7 @@
 #include "gbx_project.h"
 #include "gbx_string.h"
 #include "gbx_date.h"
+#include "gbx_watch.h"
 
 #include "gbx_c_file.h"
 
@@ -59,8 +60,10 @@ static char _buffer[16];
 
 static void callback_read(int fd, int type, CFILE *file)
 {
-	//fprintf(stderr, "callback_read\n");
-	GB_Raise(file, EVENT_Read, 0);
+	if (!STREAM_read_ahead(CSTREAM_stream(file)))
+		GB_Raise(file, EVENT_Read, 0);
+	else
+		WATCH_little_sleep();
 }
 
 
@@ -101,7 +104,6 @@ CFILE *CFILE_create(STREAM *stream, int mode)
 	return file;
 }
 
-
 static CFILE *create_default_stream(FILE *file, int mode)
 {
 	CFILE *ob;
@@ -148,9 +150,6 @@ BEGIN_METHOD_VOID(File_free)
 
 	STREAM_close(&THIS->ob.stream);
 
-	if (THIS->watch_fd >= 0)
-		GB_Watch(THIS->watch_fd, GB_WATCH_NONE, NULL, 0);
-
 END_METHOD
 
 
