@@ -62,6 +62,7 @@
 #include "gbx_c_process.h"
 
 //#define DEBUG_ME
+//#define DEBUG_CHILD
 
 char *CPROCESS_shell = NULL;
 
@@ -85,6 +86,7 @@ static int _last_child_error_errno = 0;
 
 static void init_child(void);
 static void exit_child(void);
+static void wait_child(CPROCESS *process);
 
 enum {
 	CHILD_NO_ERROR,
@@ -157,21 +159,39 @@ static void callback_write(int fd, int type, CPROCESS *process)
 	#ifdef DEBUG_ME
 	fprintf(stderr, "callback_write: %d %p\n", fd, process);
 	#endif
-	
+
 	if (process->to_string)
 	{
-		int n = read(fd, COMMON_buffer, 256);
+		int n;
+
+		for(;;)
+		{
+			n = read(fd, COMMON_buffer, 256);
+			if (n >= 0 || errno != EINTR)
+				break;
+		}
+
 		if (n > 0)
+		{
 			process->result = STRING_add(process->result, COMMON_buffer, n);
+			return;
+		}
 	}
-	else if (GB_CanRaise(process, EVENT_Read) && !STREAM_is_closed(CSTREAM_stream(process)) && !STREAM_eof(CSTREAM_stream(process))) //process->running &&
-		GB_Raise(process, EVENT_Read, 0);
-	else
-		close_fd(&process->out);
+
+	if (GB_CanRaise(process, EVENT_Read))
+	{
+		if (!STREAM_read_ahead(CSTREAM_stream(process)))
+		{
+			GB_Raise(process, EVENT_Read, 0);
+			return;
+		}
+	}
+
+	close_fd(&process->out);
 }
 
 
-static bool callback_error(int fd, int type, CPROCESS *process)
+static void callback_error(int fd, int type, CPROCESS *process)
 {
 	char buffer[256];
 	int n;
@@ -180,24 +200,17 @@ static bool callback_error(int fd, int type, CPROCESS *process)
 	fprintf(stderr, "callback_error: %d %p\n", fd, process);
 	#endif
 
-	n = read(fd, buffer, sizeof(buffer));
-
-	if (n <= 0) /* || !process->running)*/
+	if (GB_CanRaise(process, EVENT_Error))
 	{
-		/*close(process->err);*/
-		return TRUE;
+		n = read(fd, buffer, sizeof(buffer));
+		if (n > 0)
+		{
+			GB_Raise(process, EVENT_Error, 1, GB_T_STRING, buffer, n);
+			return;
+		}
 	}
 
-	/*printf("callback_error: (%d) %.*s\n", n, n, buffer);*/
-
-	if (GB_CanRaise(process, EVENT_Error))
-		GB_Raise(process, EVENT_Error, 1, GB_T_STRING, buffer, n);
-	else
-		close_fd(&process->err);
-
-	return FALSE;
-
-	/*fprintf(stderr, "<< Write\n"); fflush(stderr);*/
+	close_fd(&process->err);
 }
 
 
@@ -295,11 +308,10 @@ static void throw_last_child_error()
 static void stop_process_after(CPROCESS *_object)
 {
 	STREAM *stream;
-	int64_t len, len2;
 	bool do_exit_process = FALSE;
 
 	#ifdef DEBUG_ME
-	fprintf(stderr, "stop_process_after: %p\n", _object);
+	fprintf(stderr, "stop_process_after: %p  out = %d err = %d\n", _object, THIS->out, THIS->err);
 	#endif
 
 	if (WIFEXITED(THIS->status) && WEXITSTATUS(THIS->status) == 255)
@@ -310,9 +322,9 @@ static void stop_process_after(CPROCESS *_object)
 	}
 
 	/* Vidage du tampon d'erreur */
-	if (THIS->err >= 0)
+	while (THIS->err >= 0)
 	{
-		while (callback_error(THIS->err, 0, THIS) == 0);
+		callback_error(THIS->err, 0, THIS);
 		do_exit_process = TRUE;
 	}
 
@@ -320,19 +332,8 @@ static void stop_process_after(CPROCESS *_object)
 	if (THIS->out >= 0)
 	{
 		stream = CSTREAM_stream(THIS);
-		if (!STREAM_is_closed(stream))
-		{
-			while (!STREAM_eof(stream))
-			{
-				STREAM_lof(stream, &len);
-				callback_write(THIS->out, 0, THIS);
-				if (STREAM_is_closed(stream))
-					break;
-				STREAM_lof(stream, &len2);
-				if (len == len2)
-					break;
-			}
-		}
+		while (THIS->out >= 0 && !STREAM_is_closed(stream))
+			callback_write(THIS->out, 0, THIS);
 
 		do_exit_process = TRUE;
 	}
@@ -340,33 +341,32 @@ static void stop_process_after(CPROCESS *_object)
 	if (do_exit_process)
 		exit_process(THIS);
 
+	/*printf("** stop_process_after\n");*/
+	//GB_Unref((void **)&_object); /* Ref du post */
+}
+
+
+static void stop_process(CPROCESS *_object)
+{
+	if (!THIS->running)
+		return;
+
+	#ifdef DEBUG_ME
+	fprintf(stderr, "stop_process: %p\n", THIS);
+	#endif
+
+	/* Remove from running process list */
+
+	stop_process_after(THIS);
+	remove_process_from_running_list(THIS);
+
 	#ifdef DEBUG_ME
 	fprintf(stderr, "Raising Kill event for %p: parent = %p  can raise = %d\n", THIS, OBJECT_parent(THIS), GB_CanRaise(THIS, EVENT_Kill));
 	#endif
 	GB_Raise(THIS, EVENT_Kill, 0);
 
 	OBJECT_detach((OBJECT *)THIS);
-
-	/*printf("** stop_process_after\n");*/
-	//GB_Unref((void **)&_object); /* Ref du post */
-}
-
-
-static void stop_process(CPROCESS *process)
-{
-	if (!process->running)
-		return;
-
-	#ifdef DEBUG_ME
-	fprintf(stderr, "stop_process: %p\n", process);
-	#endif
-
-	/* Remove from running process list */
-
-	remove_process_from_running_list(process);
-	stop_process_after(process);
-
-	OBJECT_UNREF(process);
+	OBJECT_UNREF(_object);
 
 	//if (!_running_process_list)
 	if (_running_process <= _ignore_process)
@@ -387,7 +387,7 @@ static void abort_child(int error)
 	#ifdef DEBUG_ME
 	fprintf(stderr, "abort_child: %d %d\n", error, save_errno);
 	#endif
-	
+
 	snprintf(path, sizeof(path), FILE_TEMP_DIR "/%d.child", (int)getuid(), (int)getppid(), (int)getpid());
 	
 	fd = open(path, O_CREAT | O_WRONLY, 0600);
@@ -398,7 +398,7 @@ static void abort_child(int error)
 		close(fd);
 	}
 	
-	exit(255);
+	_exit(255);
 }
 
 static void init_child_tty(int fd)
@@ -418,13 +418,13 @@ static void init_child_tty(int fd)
 	terminal.c_lflag |= ISIG | ICANON | IEXTEN; // | ECHO;
 	terminal.c_lflag &= ~ECHO;
 	
-	#ifdef DEBUG_ME
+	#ifdef DEBUG_CHILD
 	fprintf(stderr, "init_child_tty: %s\n", isatty(fd) ? ttyname(fd) : "not a tty!");
 	#endif
 	
 	if (tcsetattr(fd, TCSANOW, &terminal))
 	{
-		#ifdef DEBUG_ME
+		#ifdef DEBUG_CHILD
 		int save_errno = errno;
 		fprintf(stderr, "init_child_tty: errno = %d\n", errno);
 		errno = save_errno;
@@ -433,9 +433,64 @@ static void init_child_tty(int fd)
 	}
 }
 
+const char *CPROCESS_search_program_in_path(char *name)
+{
+	char *search;
+	char *p, *p2;
+	int len;
+	const char *path;
+
+	if (strchr(name, '/'))
+	{
+		if (access(name, X_OK) == 0)
+			return name;
+		else
+			return NULL;
+	}
+
+	search = getenv("PATH");
+	if (!search || !*search)
+		search = "/usr/bin:/bin";
+
+	search = STRING_new_zero(search);
+
+	path = NULL;
+
+	//fprintf(stderr, "search_program_in_path: '%s' in '%s'\n", name, search);
+
+	p = search;
+	for(;;)
+	{
+		p2 = strchr(p, ':');
+		if (p2)
+			len = p2 - p;
+		else
+			len = strlen(p);
+
+		if (len > 0)
+		{
+			p[len] = 0;
+			//fprintf(stderr, "trying: %s\n", p);
+			path = FILE_cat(p, name, NULL);
+			if (access(path, X_OK) == 0)
+				break;
+			path = NULL;
+		}
+
+		if (!p2)
+			break;
+
+		p = p2 + 1;
+	}
+
+	STRING_free(&search);
+	//fprintf(stderr, "--> %s\n", path);
+	return path;
+}
+
 static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 {
-	static const char *shell[] = { "/bin/sh", "-c", NULL, NULL };
+	static const char *shell[] = { NULL, "-c", NULL, NULL };
 
 	int fdin[2], fdout[2], fderr[2];
 	pid_t pid;
@@ -443,16 +498,15 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	CARRAY *array;
 	int i, n;
 	sigset_t sig, old;
-	bool pwd;
 	int save_errno;
 
 	// for virtual terminal
 	int fd_master = -1;
-	int fd_slave;
 	char *slave = NULL;
 	//struct termios termios_stdin;
 	//struct termios termios_check;
 	struct termios termios_master;
+	const char *exec;
 
 	if (mode & PM_SHELL)
 	{
@@ -464,11 +518,15 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 
 		if (CPROCESS_shell)
 			argv[0] = CPROCESS_shell;
+		else
+			argv[0] = "/bin/sh";
 		
 		argv[2] = (char *)cmd;
 
 		if (argv[2] == NULL || *argv[2] == 0)
 			return;
+
+		exec = argv[0];
 		
 		process->process_group = TRUE;
 	}
@@ -484,18 +542,19 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		memcpy(argv, array->data, sizeof(*argv) * n);
 		argv[n] = NULL;
 
+		exec = CPROCESS_search_program_in_path(argv[0]);
+		if (!exec)
+		{
+			IFREE(argv);
+			THROW(E_NEXIST);
+		}
+
 		for (i = 0; i < n; i++)
 		{
 			if (!argv[i])
 				argv[i] = "";
 		}
 
-		if (*argv[0] == '/' && !FILE_exist(argv[0]))
-		{
-			FREE(&argv);
-			THROW(E_NEXIST);
-		}
-		
 		#ifdef DEBUG_ME
 		{
 			int i;
@@ -518,6 +577,7 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	if (mode & PM_TERM)
 	{
 		#ifdef OS_OPENBSD
+		int fd_slave;
 		if (openpty(&fd_master, &fd_slave, NULL, NULL, NULL) < 0)
 			goto __ABORT_ERRNO;
 		#else
@@ -560,11 +620,15 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	sigaddset(&sig, SIGCHLD);
 	sigprocmask(SIG_BLOCK, &sig, &old);
 
-	pid = fork();
+	if (mode & PM_SHELL || mode & PM_TERM || env)
+		pid = fork();
+	else
+		pid = vfork();
+
 	if (pid == (-1))
 	{
 		stop_process(process);
-		sigprocmask(SIG_SETMASK, &old, &sig);
+		sigprocmask(SIG_SETMASK, &old, NULL);
 		goto __ABORT_ERRNO;
 	}
 
@@ -635,16 +699,17 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 			FREE(&argv);
 		}
 
-		sigprocmask(SIG_SETMASK, &old, &sig);
+		sigprocmask(SIG_SETMASK, &old, NULL);
 	}
-	
-	// child process
-	
-	else
+	else //------------ child process ----------------------------
 	{
+		int fd_slave;
+		bool pwd;
+		int ch_i, ch_n;
+
 		//bool stdin_isatty = isatty(STDIN_FILENO);
 		
-		sigprocmask(SIG_SETMASK, &old, &sig);
+		sigprocmask(SIG_SETMASK, &old, NULL);
 		
 		if (mode & PM_SHELL)
 			setpgid(0, 0);
@@ -656,10 +721,10 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 			fd_slave = open(slave, O_RDWR);
 			if (fd_slave < 0)
 				abort_child(CHILD_CANNOT_OPEN_TTY);
-			
-			#ifdef DEBUG_ME
+
+			/*#ifdef DEBUG_ME
 			fprintf(stderr, "run_process (child): slave = %s isatty = %d\n", slave, isatty(fd_slave));
-			#endif
+			#endif*/
 
 			if (mode & PM_WRITE)
 			{
@@ -718,10 +783,10 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		if (env)
 		{
 			char *str;
-			n = ARRAY_count(env->data);
-			for (i = 0; i < n; i++)
+			ch_n = ARRAY_count(env->data);
+			for (ch_i = 0; ch_i < ch_n; ch_i++)
 			{
-				str = ((char **)env->data)[i];
+				str = ((char **)env->data)[ch_i];
 				if (putenv(str))
 					ERROR_warning("cannot set environment string: %s", str);
 				if (strncmp(str, "PWD=", 4) == 0 && chdir(&str[4]) == 0)
@@ -733,12 +798,16 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		if (!pwd)
 			FILE_chdir(PROJECT_oldcwd);
 		
-		execvp(argv[0], (char **)argv);
-		//execve(argv[0], (char **)argv, environ);
+		execv(exec, (char **)argv);
 		abort_child(CHILD_CANNOT_EXEC);
 	}
 
 	update_stream(process);
+
+	#ifdef DEBUG_ME
+	fprintf(stderr, "run_process: child is OK\n");
+	#endif
+
 	return;
 	
 __ABORT_ERRNO:
@@ -748,10 +817,38 @@ __ABORT_ERRNO:
 	THROW_SYSTEM(save_errno, NULL);
 }
 
+void CPROCESS_check(void *_object)
+{
+	#ifdef DEBUG_ME
+	fprintf(stderr, "CPROCESS_check: %p\n", THIS);
+	#endif
+
+	usleep(100);
+	wait_child(THIS);
+	throw_last_child_error();
+
+	//fprintf(stderr, "CPROCESS_check: %p END\n", THIS);
+}
+
+static void wait_child(CPROCESS *process)
+{
+	int status;
+
+	if (wait4(process->pid, &status, WNOHANG, NULL) == process->pid)
+	{
+		process->status = status;
+		_last_status = status;
+
+		#ifdef DEBUG_ME
+		fprintf(stderr, "Process %d has returned %d\n", process->pid, status);
+		#endif
+
+		stop_process(process);
+	}
+}
 
 static void callback_child(int signum, intptr_t data)
 {
-	int status;
 	CPROCESS *process, *next;
 	//int buffer;
 
@@ -764,6 +861,7 @@ static void callback_child(int signum, intptr_t data)
 			ERROR_panic("Cannot read from SIGCHLD pipe: %s", strerror(errno));
 	}
 #endif
+
 	#ifdef DEBUG_ME
 	fprintf(stderr, ">> callback_child\n");
 	#endif
@@ -771,23 +869,7 @@ static void callback_child(int signum, intptr_t data)
 	for (process = _running_process_list; process; )
 	{
 		next = process->next;
-
-		if (wait4(process->pid, &status, WNOHANG, NULL) == process->pid)
-		{
-			process->status = status;
-			_last_status = status;
-
-			#ifdef DEBUG_ME
-			fprintf(stderr, "Process %d has returned %d\n", process->pid, status);
-			#endif
-
-			/*printf("** signal_child\n");*/
-
-			stop_process(process);
-			//stop_process_after(process);
-			//break; // one exit at a time // why ?
-		}
-
+		wait_child(process);
 		process = next;
 	}
 
@@ -838,12 +920,8 @@ CPROCESS *CPROCESS_create(int mode, void *cmd, char *name, CARRAY *env)
 {
 	CPROCESS *process;
 
-	/*printf("** CPROCESS_create <<<< \n");*/
-
-	//if (!name || !*name)
-	//	name = "Process";
-
 	_CPROCESS_create_process = process = OBJECT_new(CLASS_Process, name, OP  ? (OBJECT *)OP : (OBJECT *)CP);
+	//fprintf(stderr, "CPROCESS_create: %p\n", process);
 
 	ON_ERROR(error_CPROCESS_create)
 	{
@@ -857,8 +935,6 @@ CPROCESS *CPROCESS_create(int mode, void *cmd, char *name, CARRAY *env)
 	if (!name || !*name)
 		STREAM_blocking(CSTREAM_stream(process), TRUE);
 	
-	/*printf("** CPROCESS_create >>>> \n");*/
-
 	return process;
 }
 
@@ -926,7 +1002,7 @@ void CPROCESS_wait_for(CPROCESS *process, int timeout)
 	#endif
 }
 
-
+//-------------------------------------------------------------------------
 
 BEGIN_METHOD_VOID(Process_exit)
 
@@ -1113,7 +1189,7 @@ GB_DESC NATIVE_Process[] =
 
 	GB_STATIC_PROPERTY_READ("LastState", "i", Process_LastState),
 	GB_STATIC_PROPERTY_READ("LastValue", "i", Process_LastValue),
-	
+
 	GB_PROPERTY_READ("Id", "i", Process_Id),
 	GB_PROPERTY_READ("Handle", "i", Process_Id),
 	GB_PROPERTY_READ("State", "i", Process_State),
