@@ -33,8 +33,10 @@
 #include <pwd.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <syslog.h>
 
 #include "gb_common.h"
+#include "gb_common_case.h"
 #include "gb_error.h"
 #include "gbx_api.h"
 #include "gbx_class.h"
@@ -46,10 +48,27 @@
 #include "gbx_exec.h"
 #include "gbx_extern.h"
 #include "gbx_object.h"
-
 #include "gbx_c_process.h"
 #include "gbx_c_system.h"
 
+typedef
+	struct {
+		const char *prefix;
+		int prio;
+	}
+	SYSLOG_PREFIX;
+
+static const SYSLOG_PREFIX _syslog_prefix[] =
+{
+	{ "[error]", LOG_ERR },
+	{ "[warning]", LOG_WARNING },
+	{ "[notice]", LOG_NOTICE },
+	{ "[info]", LOG_INFO },
+	{ "[debug]", LOG_DEBUG },
+	{ NULL, 0 }
+};
+
+
 BEGIN_PROPERTY(User_Home)
 
 	GB_ReturnString(PROJECT_get_home());
@@ -222,11 +241,82 @@ BEGIN_PROPERTY(System_TimeZone)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(System_BreakOnError)
+
+	if (READ_PROPERTY)
+		GB_ReturnBoolean(EXEC_break_on_error);
+	else if (EXEC_debug)
+		EXEC_break_on_error = VPROP(GB_BOOLEAN);
+
+END_METHOD
+
+BEGIN_METHOD(System_Log, GB_STRING message)
+
+	static bool _opened = FALSE;
+
+	const SYSLOG_PREFIX *p;
+	char *msg;
+	int len;
+	int lenp;
+	int prio;
+
+	msg = STRING(message);
+	len = LENGTH(message);
+	prio = LOG_INFO;
+
+	for (p = _syslog_prefix; p->prefix; p++)
+	{
+		lenp = strlen(p->prefix);
+		if (len >= (lenp + 2) && strncasecmp(msg, p->prefix, lenp) == 0)
+		{
+			msg += lenp + 2;
+			len -= lenp + 2;
+			prio = p->prio;
+			break;
+		}
+	}
+
+	while (len > 0 && *msg == ' ')
+		msg++, len--;
+
+	if (len <= 0)
+		return;
+
+	if (!_opened)
+	{
+		_opened = TRUE;
+		openlog(PROJECT_name, LOG_PID, LOG_INFO);
+	}
+
+	syslog(prio, "%.*s", len, msg);
+
+END_METHOD
+
+BEGIN_METHOD(System_Find, GB_STRING program)
+
+	const char *path;
+
+	path = CPROCESS_search_program_in_path(GB_ToZeroString(ARG(program)));
+	if (!path)
+		GB_ReturnNull();
+	else
+		GB_ReturnNewZeroString(path);
+
+END_METHOD
+
+BEGIN_METHOD(System_Exist, GB_STRING program)
+
+	GB_ReturnBoolean(CPROCESS_search_program_in_path(GB_ToZeroString(ARG(program))) != NULL);
+
+END_METHOD
+
 #endif
 
+//-------------------------------------------------------------------------
+
 GB_DESC NATIVE_User[] =
 {
-	GB_DECLARE("User", 0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE_STATIC("User"),
 
 	GB_STATIC_PROPERTY_READ("Name", "s", User_Name),
 	GB_STATIC_PROPERTY_READ("Id", "i", User_Id),
@@ -238,12 +328,13 @@ GB_DESC NATIVE_User[] =
 
 GB_DESC NATIVE_System[] =
 {
-	GB_DECLARE("System", 0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE_STATIC("System"),
 
 	GB_STATIC_PROPERTY_READ("Path", "s", System_Path),
 	GB_CONSTANT("Version", "s", GAMBAS_VERSION_STRING),
 	GB_CONSTANT("FullVersion", "s", VERSION),
 	GB_STATIC_PROPERTY_READ("Backtrace", "String[]", System_Backtrace),
+	GB_STATIC_PROPERTY("BreakOnError", "b", System_BreakOnError),
 
 	GB_STATIC_PROPERTY("Language", "s", System_Language),
 	GB_STATIC_PROPERTY("FirstDayOfWeek", "i", System_FirstDayOfWeek),
@@ -264,8 +355,13 @@ GB_DESC NATIVE_System[] =
 	
 	GB_STATIC_METHOD("GetExternSymbol", "p", System_GetExternSymbol, "(Library)s(Symbol)s"),
 	GB_STATIC_METHOD("_Breakpoint", NULL, System_Breakpoint, NULL),
-	
+
 	GB_STATIC_PROPERTY_SELF("User", "User"),
 
+	GB_STATIC_METHOD("Log", NULL, System_Log, "(Message)s"),
+
+	GB_STATIC_METHOD("Exist", "b", System_Exist, "(Program)s"),
+	GB_STATIC_METHOD("Find", "s", System_Find, "(Program)s"),
+
 	GB_END_DECLARE
 };
