@@ -55,6 +55,9 @@
 //#define DEBUG_LOAD 1
 //#define DEBUG_DESC 1
 
+// We are exiting...
+bool CLASS_exiting = FALSE;
+
 /* Global class table */
 static TABLE _global_table;
 /* List of all classes */
@@ -103,7 +106,7 @@ static void exit_class(CLASS *class, bool native)
 	if (CLASS_is_native(class) != native)
 		return;
 	
-	if (class->state < CS_LOADED)
+	if (!CLASS_is_loaded(class))
 		return;
 
 	EXEC_public(class, NULL, "_exit", 0);
@@ -184,15 +187,17 @@ static void unload_class(CLASS *class)
 	FREE(&class->table);
 	FREE(&class->sort);
 
-	class->state = CS_NULL;
+	class->ready = FALSE;
+	class->loaded = FALSE;
 }
 
 #define SWAP_FIELD(_v, _s, _d, _f) (_v = _s->_f, _s->_f = _d->_f, _d->_f = _v)
 
-static void class_replace_global(const char *name)
+static void class_replace_global(CLASS *class)
 {
 	CLASS_DESC_SYMBOL *cds;
-	CLASS *class, *parent;
+	CLASS *parent;
+	const char *name;
 	char *old_name;
 	CLASS *old_class;
 	int len;
@@ -203,11 +208,11 @@ static void class_replace_global(const char *name)
 	int nprefix;
 	int i;
 
-	class = CLASS_find_global(name);
-	if (class->state)
+	if (CLASS_is_loaded(class))
 	{
 		//fprintf(stderr, "class_replace_global: %p %s\n", class, name);
 		
+		name = class->name;
 		len = strlen(name);
 
 		nprefix = 0;
@@ -285,6 +290,9 @@ static void class_replace_global(const char *name)
 
 static void release_class(CLASS *class)
 {
+	#if DEBUG_LOAD
+	fprintf(stderr, "Freeing %s\n", class->name);
+	#endif
 	OBJECT_release(class, NULL);
 	class->exit = TRUE;
 }
@@ -296,29 +304,47 @@ void CLASS_clean_up(bool silent)
 	CLASS *class;
 
 	#if DEBUG_LOAD
-	fprintf(stderr, "\n------------------- CLASS_exit -------------------\n\n");
+	fprintf(stderr, "\n------------------- CLASS_clean_up -------------------\n\n");
 	#endif
 
 	#if DEBUG_LOAD
 	fprintf(stderr, "Freeing auto-creatable objects...\n");
 	#endif
 
+	CLASS_exiting = TRUE;
+
+	// Free automatic instances
+
+	for (class = _classes; class; class = class->next)
+	{
+		if (class->instance)
+		{
+			#if DEBUG_LOAD
+			fprintf(stderr, "Freeing instance of %p %s\n", class, class->name);
+			#endif
+			OBJECT_UNREF(class->instance);
+		}
+	}
+
 	// Count how many classes should be freed
-	// And free automatic instances
 
 	nc = 0;
 
 	for (class = _classes; class; class = class->next)
 	{
-		if (class->instance)
-			OBJECT_UNREF(class->instance);
-		if (!CLASS_is_native(class) && class->state)
+		if (!CLASS_is_native(class) && CLASS_is_loaded(class))
 		{
-			/*printf("Must free: %s\n", class->name);*/
+			#if DEBUG_LOAD
+			fprintf(stderr, "Must free: %p (%s) %s [%d]\n", class, class->component ? class->component->name : "-", class->name, class->count);
+			#endif
 			nc++;
 		}
 	}
 
+	#if DEBUG_LOAD
+	fprintf(stderr, "Must free %d classes...\n", nc);
+	#endif
+
 	#if DEBUG_LOAD
 	fprintf(stderr, "Calling _exit on loaded classes...\n");
 	#endif
@@ -333,7 +359,7 @@ void CLASS_clean_up(bool silent)
 	// Free classes having no instance
 
 	n = 0;
-	while (n < nc)
+	for(;;)
 	{
 		nb = n;
 
@@ -342,12 +368,8 @@ void CLASS_clean_up(bool silent)
 			/*if (!CLASS_is_native(class) && class->ready && !class->exit)
 				printf("%s: %d ready = %d\n", class->name, class->count, class->ready);*/
 
-			if (class->count == 0 && !CLASS_is_native(class) && class->state && !class->exit)
+			if (class->count == 0 && !CLASS_is_native(class) && CLASS_is_loaded(class) && !class->exit)
 			{
-				/*printf("Freeing %s\n", class->name);*/
-				#if DEBUG_LOAD
-				fprintf(stderr, "Freeing %s\n", class->name);
-				#endif
 				release_class(class);
 				n++;
 			}
@@ -375,7 +397,7 @@ void CLASS_clean_up(bool silent)
 
 			for (class = _classes; class; class = class->next)
 			{
-				if (!CLASS_is_native(class) && class->state && !class->exit && (!class->array_class || class->array_class->exit)) // && !class->astruct_class)
+				if (!CLASS_is_native(class) && CLASS_is_loaded(class) && !class->exit && (!class->array_class || class->array_class->exit)) // && !class->astruct_class)
 				{
 					if (!silent)
 						fprintf(stderr, "gbx" GAMBAS_VERSION_STRING ": % 5d %s\n", class->count, class->name);
@@ -433,7 +455,7 @@ CLASS *CLASS_look(const char *name, int len)
 	int index;
 
 	#if DEBUG_COMP
-	fprintf(stderr, "CLASS_look: %s in %s\n", name, _global ? "global" : "local");
+	fprintf(stderr, "CLASS_look: %s in %s", name, _global ? "global" : "local");
 	#endif
 
 	/*if (strncasecmp(name, "listbox", 7) == 0)
@@ -516,8 +538,6 @@ CLASS *CLASS_find(const char *name)
 
 	ALLOC_ZERO(&class, sizeof(CLASS));
 	csym->class = class;
-	//class->state = CS_NULL;
-	//class->count = 0;
 	class->ref = 1;
 	
 	class->next = _classes;
@@ -550,7 +570,7 @@ CLASS *CLASS_get(const char *name)
 {
 	CLASS *class = CLASS_find(name);
 
-	if (class->state == CS_NULL)
+	if (!CLASS_is_loaded(class))
 		CLASS_load(class);
 
 	return class;
@@ -991,13 +1011,8 @@ static bool check_signature(char type, const CLASS_DESC *desc, const CLASS_DESC
 	TYPE *sd, *sp;
 	int nsd, nsp;
 	
-	if (desc->property.type != pdesc->property.type)
-	{
-		//fprintf(stderr, "type! %ld / %ld\n", desc->property.type, pdesc->property.type);
+	if (!TYPE_are_compatible(desc->property.type, pdesc->property.type))
 		return TRUE;
-	}
-	//else
-	//	return FALSE;
 	
 	switch (type)
 	{
@@ -1031,10 +1046,7 @@ static bool check_signature(char type, const CLASS_DESC *desc, const CLASS_DESC
 			return FALSE;
 	}
 
-	if (TYPE_compare_signature(sd, nsd, sp, nsp))
-		return TRUE;
-	else
-		return FALSE;
+	return TYPE_compare_signature(sd, nsd, sp, nsp, TRUE);
 }
 
 void CLASS_make_description(CLASS *class, const CLASS_DESC *desc, int n_desc, int *first)
@@ -1369,20 +1381,17 @@ void CLASS_search_special(CLASS *class)
 }
 
 
-CLASS *CLASS_check_global(char *name)
+CLASS *CLASS_check_global(CLASS *class)
 {
-	CLASS *class;
-
-	class = CLASS_find_global(name);
-	if (class->state)
+	if (CLASS_is_loaded(class))
 	{
 		if (COMPONENT_current && class->component == COMPONENT_current)
 			ERROR_panic("Class '%s' declared twice in the component '%s'.", CLASS_get_name(class), class->component->name);
 
-		if (class->has_child)
-			THROW(E_CLASS, name, "Overriding an already inherited class is forbidden", "");
+		/*if (class->has_child)
+			THROW(E_CLASS, class->name, "Overriding an already inherited class is forbidden", "");*/
 			
-		class_replace_global(name);
+		class_replace_global(class);
 	}
 
 	return class;
