@@ -179,6 +179,8 @@ static GB_DESC NATIVE_GambasLanguage[] =
 	GB_METHOD("IsLetter", "b", NULL, "(String)s"),
 	GB_METHOD("IsLower", "b", NULL, "(String)s"),
 	GB_METHOD("IsUpper", "b", NULL, "(String)s"),
+	GB_METHOD("IsLCase", "b", NULL, "(String)s"),
+	GB_METHOD("IsUCase", "b", NULL, "(String)s"),
 	GB_METHOD("IsDigit", "b", NULL, "(String)s"),
 	GB_METHOD("IsHexa", "b", NULL, "(String)s"),
 	GB_METHOD("IsSpace", "b", NULL, "(String)s"),
@@ -232,7 +234,7 @@ static GB_DESC NATIVE_GambasLanguage[] =
 	GB_METHOD("Time", "d", NULL, "[(DateOrHour)v(Minute)i(Second)i(MilliSecond)i]"),
 	GB_METHOD("Week", "i", NULL, "[(Date)d(Mode)i(Plain)b]"),
 
-	GB_METHOD("DateAdd", "d", NULL, "(Date)d(Period)i(Interval)i"),
+	GB_METHOD("DateAdd", "d", NULL, "(Date)d(Period)i(Count)i"),
 	GB_METHOD("DateDiff", "i", NULL, "(Date1)d(Date2)d(Period)i"),
 
 	GB_METHOD("Eval", "v", NULL, "(Expression)s[(Context)Collection;]"),
@@ -286,7 +288,11 @@ static GB_DESC NATIVE_GambasLanguage[] =
 	GB_METHOD("MkPointer", "s", NULL, "(Value)p"),
 	GB_METHOD("MkPointer$", "s", NULL, "(Value)p"),
 
+	GB_METHOD("Swap", "s", NULL, "(String)s[(Endianness)i]"),
+	GB_METHOD("Swap$", "s", NULL, "(String)s[(Endianness)i]"),
+
 	GB_METHOD("Bool@", "b", NULL, "(Pointer)p"),
+	GB_METHOD("Boolean@", "b", NULL, "(Pointer)p"),
 	GB_METHOD("Byte@", "c", NULL, "(Pointer)p"),
 	GB_METHOD("Short@", "h", NULL, "(Pointer)p"),
 	GB_METHOD("Int@", "i", NULL, "(Pointer)p"),
@@ -308,16 +314,33 @@ static GB_DESC NATIVE_GambasLanguage[] =
 	
 	GB_METHOD("Html", "s", NULL, "(String)s"),
 	GB_METHOD("Html$", "s", NULL, "(String)s"),
-	
+
+	GB_METHOD("Base64", "s", NULL, "(String)s"),
+	GB_METHOD("Base64$", "s", NULL, "(String)s"),
+
+	GB_METHOD("Url", "s", NULL, "(String)s"),
+	GB_METHOD("Url$", "s", NULL, "(String)s"),
+
 	GB_METHOD("Unquote", "s", NULL, "(String)s"),
 	GB_METHOD("Unquote$", "s", NULL, "(String)s"),
 	
+	GB_METHOD("UnBase64", "s", NULL, "(String)s"),
+	GB_METHOD("UnBase64$", "s", NULL, "(String)s"),
+
+	GB_METHOD("FromBase64", "s", NULL, "(String)s"),
+	GB_METHOD("FromBase64$", "s", NULL, "(String)s"),
+
+	GB_METHOD("FromUrl", "s", NULL, "(String)s"),
+	GB_METHOD("FromUrl$", "s", NULL, "(String)s"),
+
 	GB_METHOD("Odd", "b", NULL, "(Value)i"),
 	GB_METHOD("Even", "b", NULL, "(Value)i"),
 
 	GB_METHOD("IsNan", "b", NULL, "(Value)f"),
 	GB_METHOD("IsInf", "i", NULL, "(Value)f"),
-	
+
+	GB_METHOD("IsMissing", "b", NULL, "(Argument)?"),
+
 	GB_END_DECLARE
 };
 
