@@ -2,7 +2,7 @@
 
   gbx_compare.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -360,11 +360,15 @@ int COMPARE_string_natural(const char *a, int la, const char *b, int lb, bool no
 			}
 		}	
 
-		if (!ca && !cb) 
+		if (!ca)
 		{
-			/* The strings compare the same.  Perhaps the caller will want to call strcmp to break the tie. */
-			return 0;
+			if (!cb)
+				return 0;
+			else
+				return -1;
 		}
+		else if (!cb)
+			return 1;
 
 		lca = STRING_get_utf8_char_length(ca);
 		lcb = STRING_get_utf8_char_length(cb);
@@ -478,41 +482,104 @@ int COMPARE_object(void **a, void **b)
 		STACK_free_backtrace(&bt);
 	}*/
 	
-	ca = OBJECT_class(*a);
-	cb = OBJECT_class(*b);
+	ca = OBJECT_class_null(*a);
+	cb = OBJECT_class_null(*b);
 
-	if (ca && cb && ca->special[SPEC_COMPARE] != NO_SYMBOL)
+	if (ca && cb)
 	{
-	  STACK_check(1);
-		SP->_object.class = cb;
-		SP->_object.object = *b;
-		OBJECT_REF(*b, "compare_object");
-		SP++;
-		EXEC_special(SPEC_COMPARE, ca, *a, 1, FALSE);
-		VALUE_conv_integer(&SP[-1]);
-		SP--;
-		comp = SP->_integer.value;
-	}
-	else if (ca && cb && cb->special[SPEC_COMPARE] != NO_SYMBOL)
-	{
-	  STACK_check(1);
-		SP->_object.class = ca;
-		SP->_object.object = *a;
-		OBJECT_REF(*a, "compare_object");
-		SP++;
-		EXEC_special(SPEC_COMPARE, cb, *b, 1, FALSE);
-		VALUE_conv_integer(&SP[-1]);
-		SP--;
-		comp = (- SP->_integer.value);
-	}
-	else
-	{
-		comp = (*a == *b) ? 0 : (*a > *b) ? 1 : -1;
+		if (ca->has_operators && CLASS_has_operator(ca, CO_COMP) && ca == cb)
+		{
+			void *func = ca->operators[CO_COMP];
+			comp = (*(int (*)(void *, void *))func)(*a, *b);
+			goto __RETURN;
+		}
+
+		if (ca->special[SPEC_COMPARE] != NO_SYMBOL)
+		{
+			STACK_check(1);
+			SP->_object.class = cb;
+			SP->_object.object = *b;
+			OBJECT_REF(*b);
+			SP++;
+			EXEC_special(SPEC_COMPARE, ca, *a, 1, FALSE);
+			VALUE_conv_integer(&SP[-1]);
+			SP--;
+			comp = SP->_integer.value;
+			goto __RETURN;
+		}
+
+		if (cb->special[SPEC_COMPARE] != NO_SYMBOL)
+		{
+			STACK_check(1);
+			SP->_object.class = ca;
+			SP->_object.object = *a;
+			OBJECT_REF(*a);
+			SP++;
+			EXEC_special(SPEC_COMPARE, cb, *b, 1, FALSE);
+			VALUE_conv_integer(&SP[-1]);
+			SP--;
+			comp = (- SP->_integer.value);
+			goto __RETURN;
+		}
 	}
 
+	comp = (*a == *b) ? 0 : (*a > *b) ? 1 : -1;
+
+__RETURN:
+
 	return _descent ? (-comp) : comp;
 }
 
+int COMPARE_variant(VARIANT *a, VARIANT *b)
+{
+	TYPE type;
+	VALUE value;
+	int comp;
+	
+	if (a->type == T_NULL)
+		return b->type == T_NULL ? 0 : -1;
+	else if (b->type == T_NULL)
+		return 1;
+	
+	if (TYPE_is_object(a->type))
+	{
+		if (TYPE_is_object(b->type))
+			return COMPARE_object(&a->value._object, &b->value._object);
+		else
+			return 1;
+	}
+	else if (TYPE_is_object(b->type))
+		return -1;
+	
+	if (a->type == b->type)
+		return (*COMPARE_get(a->type, 0))(&a->value, &b->value);
+	
+	type = Max(a->type, b->type);
+	
+	if (b->type == type)
+	{
+		VARIANT *c;
+		
+		c = a;
+		a = b;
+		b = c;
+		_descent = !_descent;
+	}
+	
+	value.type = T_VARIANT;
+	value._variant.vtype = b->type;
+	value._variant.value.data = b->value.data;
+	
+	BORROW(&value);
+	VALUE_conv(&value, type);
+	VALUE_conv_variant(&value);
+	comp = (*COMPARE_get(type, 0))(&a->value, &value._variant.value);
+	RELEASE(&value);
+	
+	return comp;
+}
+
+
 COMPARE_FUNC COMPARE_get(TYPE type, int mode)
 {
   _descent = (mode & GB_COMP_DESCENT) != 0;
@@ -555,6 +622,16 @@ COMPARE_FUNC COMPARE_get(TYPE type, int mode)
 				return (COMPARE_FUNC)((mode & GB_COMP_NOCASE) ? compare_string_lang_case : compare_string_lang);
 			else
 				return (COMPARE_FUNC)((mode & GB_COMP_NOCASE) ? compare_string_case : compare_string_binary);
+			
+		case T_POINTER:
+			#ifdef OS_64BITS
+				return (COMPARE_FUNC)compare_long;
+			#else
+				return (COMPARE_FUNC)compare_integer;
+			#endif
+			
+		case T_VARIANT:
+			return (COMPARE_FUNC)COMPARE_variant;
 
     default:
       return (COMPARE_FUNC)compare_nothing;
