@@ -166,6 +166,7 @@ static int default_eof(STREAM *stream)
 void STREAM_open(STREAM *stream, const char *path, int mode)
 {
 	STREAM_CLASS *sclass;
+	int fd;
 	
 	stream->type = NULL;
 	
@@ -231,6 +232,10 @@ _OPEN:
 		THROW_SYSTEM(errno, path);
 	
 	stream->type = sclass;
+
+	fd = STREAM_handle(stream);
+	if (fd >= 0)
+		fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | O_CLOEXEC);
 	
 	#if DEBUG_STREAM
 	_tag++;
@@ -340,6 +345,7 @@ void STREAM_read(STREAM *stream, void *addr, int len)
 	}
 }
 
+#if 0
 char STREAM_getchar(STREAM *stream)
 {
 	char c = 0;
@@ -380,6 +386,7 @@ char STREAM_getchar(STREAM *stream)
 	
 	return c;
 }
+#endif
 
 
 int STREAM_read_max(STREAM *stream, void *addr, int len)
@@ -534,7 +541,7 @@ void STREAM_seek(STREAM *stream, int64_t pos, int whence)
 	}
 }
 
-static void fill_buffer(STREAM *stream, char *addr)
+static void fill_buffer(STREAM *stream, char *addr, bool do_not_wait_ready)
 {
 	bool err;
 	int flags, fd;
@@ -548,7 +555,8 @@ static void fill_buffer(STREAM *stream, char *addr)
 			err = (*(stream->type->read))(stream, addr, STREAM_BUFFER_SIZE);
 		else
 		{
-			wait_for_fd_ready_to_read(fd);
+			if (!do_not_wait_ready)
+				wait_for_fd_ready_to_read(fd);
 
 			flags = fcntl(fd, F_GETFL);
 			if ((flags & O_NONBLOCK) == 0)
@@ -570,8 +578,8 @@ static void fill_buffer(STREAM *stream, char *addr)
 		if (errno == EINTR)
 			continue;
 
-		if (STREAM_eff_read == 0)
-			stop_watching(stream, GB_WATCH_READ);
+		//if (STREAM_eff_read == 0 && stream->common.available_now)
+		//	WATCH_need_little_sleep = TRUE;
 
 		switch(errno)
 		{
@@ -587,6 +595,36 @@ static void fill_buffer(STREAM *stream, char *addr)
 	}
 }
 
+bool STREAM_read_ahead(STREAM *stream)
+{
+	if (stream->common.no_read_ahead)
+		return FALSE;
+	
+	if (stream->common.buffer && stream->common.buffer_pos < stream->common.buffer_len)
+		return FALSE;
+
+	if (!stream->common.buffer)
+		ALLOC(&stream->common.buffer, STREAM_BUFFER_SIZE);
+
+	fill_buffer(stream, stream->common.buffer, TRUE);
+
+	stream->common.buffer_pos = 0;
+	stream->common.buffer_len = STREAM_eff_read;
+
+	//fprintf(stderr, "STREAM_read_ahead: %d\n", STREAM_eff_read);
+
+	if (STREAM_eff_read == 0)
+	{
+		stream->common.eof = TRUE;
+		return TRUE;
+	}
+	else
+	{
+		stream->common.eof = FALSE;
+		return FALSE;
+	}
+}
+
 
 static char *input(STREAM *stream, bool line, char *escape)
 {
@@ -658,7 +696,7 @@ static char *input(STREAM *stream, bool line, char *escape)
 			stream->common.buffer_pos = buffer_pos;
 			stream->common.buffer_len = buffer_len;
 			
-			fill_buffer(stream, buffer);
+			fill_buffer(stream, buffer, FALSE);
 			
 			buffer_pos = 0;
 			buffer_len = STREAM_eff_read;
@@ -721,11 +759,10 @@ static char *input(STREAM *stream, bool line, char *escape)
 	}
 
 	if (len > 0)
-		//add_string(addr, &len_str, stream->common.buffer + start, len);
 		addr = STRING_add(addr, buffer + start, len);
+	else if (len < 0)
+		addr = STRING_extend(addr, STRING_length(addr) + len);
 
-	//STRING_extend_end(addr);
-	
 	stream->common.buffer = buffer;
 	stream->common.buffer_pos = buffer_pos;
 	stream->common.buffer_len = buffer_len;
@@ -1504,7 +1541,6 @@ bool STREAM_lock(STREAM *stream)
 	return FALSE;
 
 __ERROR:
-
 	THROW_SYSTEM(errno, NULL);
 	return TRUE;
 }
@@ -1518,7 +1554,9 @@ void STREAM_lof(STREAM *stream, int64_t *len)
 
 	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
-		
+
+	*len = 0;
+
 	if (stream->type->lof)
 	{
 		if (!(*(stream->type->lof))(stream, len))
