@@ -34,6 +34,7 @@
 #include "gbx_regexp.h"
 #include "gbx_string.h"
 #include "gbx_c_file.h"
+#include "gbx_math.h"
 
 typedef
 	struct _stream {
@@ -779,9 +780,45 @@ void SUBR_mkdir(ushort code)
 }
 
 
-void SUBR_rmdir(void)
+void SUBR_rmdir(ushort code)
 {
-	SUBR_kill(2);
+	SUBR_ENTER();
+	int min, max;
+
+	switch (code & 0x3F)
+	{
+		case 0: // Deprecated RmDir
+			SUBR_kill(2);
+			return;
+
+		default: // Rand
+
+			if (NPARAM == 1)
+			{
+				VALUE_conv_integer(PARAM);
+				min = 0;
+				max = PARAM->_integer.value;
+			}
+			else
+			{
+				VALUE_conv_integer(PARAM);
+				VALUE_conv_integer(&PARAM[1]);
+				min = PARAM->_integer.value;
+				max = PARAM[1]._integer.value;
+				if (max < min)
+				{
+					int temp = max;
+					max = min;
+					min = temp;
+				}
+			}
+
+			RETURN->type = T_INTEGER;
+			RETURN->_integer.value = min + (int)(rnd() * (max + 1 - min));
+			break;
+	}
+
+	SUBR_LEAVE();
 }
 
 
@@ -927,25 +964,48 @@ void SUBR_access(ushort code)
 
 void SUBR_lock(ushort code)
 {
-	SUBR_ENTER_PARAM(1);
+	code &= 0x1F;
 
-	if (code & 0x1F)
+	if (code == 1)
 	{
+		SUBR_ENTER_PARAM(1);
 		STREAM_close(get_stream(PARAM, FALSE));
 		SUBR_LEAVE_VOID();
 	}
 	else
 	{
+		SUBR_ENTER_PARAM((code == 2) ? 2 : 1);
 		STREAM stream;
 		CFILE *file;
 		const char *path = get_path(PARAM);
-		
+		double wait, timer;
+
 		if (FILE_is_relative(path))
 			THROW(E_BADPATH);
 
-		STREAM_open(&stream, path, ST_WRITE | ST_CREATE | ST_DIRECT);
-		if (STREAM_lock(&stream))
+		if (code == 2)
 		{
+			DATE_timer(&wait, FALSE);
+			wait += SUBR_get_float(&PARAM[1]);
+		}
+
+		STREAM_open(&stream, path, ST_WRITE | ST_CREATE | ST_DIRECT);
+
+		for(;;)
+		{
+			if (!STREAM_lock(&stream))
+				break;
+
+			if (code == 2)
+			{
+				DATE_timer(&timer, FALSE);
+				if (timer < wait)
+				{
+					usleep(10000);
+					continue;
+				}
+			}
+
 			STREAM_close(&stream);
 			THROW(E_LOCK);
 		}
