@@ -143,11 +143,7 @@ static char *FindDatabase(char *name, char *hostName)
 	return NULL;
 }
 
-/*****************************************************************************
-
-  open_database()
-
-*****************************************************************************/
+//-------------------------------------------------------------------------
 
 static int open_database(DB_DESC *desc, DB_DATABASE * db)
 {
@@ -160,7 +156,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE * db)
 	db_fullpath = FindDatabase(desc->name, desc->host);
 	if (!db_fullpath)
 	{
-		GB.Error("Unable to locate database: &1", desc->name);
+		GB.Error("Unable to locate database `&1` in `&2`", desc->name, desc->host);
 		return TRUE;
 	}
 					 	
@@ -193,16 +189,23 @@ __SQLITE3:
 }
 
 
+//-------------------------------------------------------------------------
+
+static int database_is_system(DB_DATABASE * db, const char *name)
+{
+	return FALSE;
+}
+
 /*****************************************************************************
 
   The driver interface
 
 *****************************************************************************/
 
-DB_DRIVER DB_sqlite_pseudo_driver = 
+DB_DRIVER DB_sqlite_pseudo_driver =
 {
-	"sqlite",
-	open_database,
-	0
+	.name = "sqlite",
+	.Open = open_database,
+	.Database.IsSystem = database_is_system
 };
 
