@@ -40,6 +40,7 @@ static GB_PAINT *_current = NULL;
 #define BPAINT (BTHIS->desc)
 
 #define CHECK_DEVICE() if (check_device()) return
+#define CHECK_PATH() if (check_path()) return
 
 #define RAD(_deg) ((_deg) * M_PI / 180)
 
@@ -69,6 +70,17 @@ static bool check_device()
 		return FALSE;
 }
 
+static bool check_path()
+{
+	if (THIS->has_path)
+	{
+		GB.Error("Pending path");
+		return TRUE;
+	}
+	else
+		return FALSE;
+}
+
 GB_PAINT *PAINT_get_current()
 {
 	if (check_device())
@@ -80,16 +92,22 @@ GB_PAINT *PAINT_get_current()
 GB_PAINT *PAINT_from_device(void *device)
 {
 	GB_PAINT *d;
-	
+
 	for (d = _current; d; d = d->previous)
 	{
 		if (d->device == device && d->opened)
 			break;
 	}
-	
+
 	return d;
 }
 
+bool PAINT_is_painted(void *device)
+{
+	return PAINT_from_device(device) != NULL;
+}
+
+
 bool PAINT_open(GB_PAINT *paint)
 {
 	if (paint->opened)
@@ -108,13 +126,19 @@ bool PAINT_open(GB_PAINT *paint)
 
 void PAINT_close(GB_PAINT *paint)
 {
-	if (paint->opened && !paint->other)
+	if (!paint->opened)
+		return;
+
+	if (!paint->other)
 	{
 		//fprintf(stderr, "PAINT_close: %p\n", paint);
 		paint->desc->End(paint);
 		GB.Free(POINTER(&paint->extra));
-		paint->opened = FALSE;
 	}
+	else
+		PAINT->Restore(THIS);
+
+	paint->opened = FALSE;
 }
 
 bool PAINT_begin(void *device)
@@ -143,7 +167,9 @@ bool PAINT_begin(void *device)
 	paint->brush = NULL;
 	paint->opened = FALSE;
 	paint->other = FALSE;
+	paint->has_path = FALSE;
 	paint->tag = NULL;
+	paint->area.x = paint->area.y = 0;
 	
 	paint->previous = _current;
 	_current = paint;
@@ -157,13 +183,13 @@ bool PAINT_begin(void *device)
 		paint->extra = other->extra;
 		paint->opened = TRUE;
 		paint->other = TRUE;
-		paint->width = other->width;
-		paint->height = other->height;
+		paint->area = other->area;
 		paint->resolutionX = other->resolutionX;
 		paint->resolutionY = other->resolutionY;
 		paint->brush = other->brush;
 		if (paint->brush)
 			GB.Ref(paint->brush);
+		PAINT->Save(THIS);
 	}
 	else
 	{
@@ -196,6 +222,33 @@ void PAINT_end()
 	GB.Free(POINTER(&paint));
 }
 
+void PAINT_translate(double tx, double ty)
+{
+	GB_TRANSFORM transform;
+
+	if (tx == 0.0 && ty == 0.0)
+		return;
+
+	MPAINT->Create(&transform);
+	PAINT->Matrix(THIS, FALSE, transform);
+	MPAINT->Translate(transform, (float)tx, (float)ty);
+	PAINT->Matrix(THIS, TRUE, transform);
+	MPAINT->Delete(&transform);
+}
+
+void PAINT_set_area(GEOM_RECTF *area)
+{
+	THIS->area.x += area->x;
+	THIS->area.y += area->y;
+
+	THIS->area.width = area->w; //MIN(area->w, THIS->area.width - THIS->area.x);
+	THIS->area.height = area->h; //MIN(area->h, THIS->area.height - THIS->area.y);
+
+	if (THIS->area.width <= 0 || THIS->area.height <= 0)
+		THIS->area.width = THIS->area.height = 0;
+
+	PAINT_translate(area->x, area->y);
+}
 
 //---- PaintExtents ---------------------------------------------------------
 
@@ -233,6 +286,8 @@ BEGIN_METHOD(PaintExtents_Merge, GB_OBJECT extents)
 	if (extents->ext.x2 > XTHIS->ext.x2) XTHIS->ext.x2 = extents->ext.x2;
 	if (extents->ext.y2 > XTHIS->ext.y2) XTHIS->ext.y2 = extents->ext.y2;
 
+	GB.ReturnObject(XTHIS);
+
 END_METHOD
 
 GB_DESC PaintExtentsDesc[] = 
@@ -483,14 +538,17 @@ GB_DESC PaintBrushDesc[] =
 //---- Paint ----------------------------------------------------------------
 
 
-BEGIN_METHOD(Paint_Begin, GB_OBJECT device)
+BEGIN_METHOD(Paint_Begin, GB_OBJECT device; GB_OBJECT area)
 
 	void *device = VARG(device);
+	GEOM_RECTF *area = VARGOPT(area, NULL);
 
 	if (GB.CheckObject(device))
 		return;
 
 	PAINT_begin(device);
+	if (area)
+		PAINT_set_area(area);
 
 END_METHOD
 
@@ -523,7 +581,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Paint_Width)
 
 	CHECK_DEVICE();
-	GB.ReturnFloat(THIS->width);
+	GB.ReturnFloat(THIS->area.width);
 
 END_PROPERTY
 
@@ -531,7 +589,7 @@ END_PROPERTY
 BEGIN_PROPERTY(Paint_Height)
 
 	CHECK_DEVICE();
-	GB.ReturnFloat(THIS->height);
+	GB.ReturnFloat(THIS->area.height);
 
 END_PROPERTY
 
@@ -560,8 +618,11 @@ END_METHOD
 
 #define IMPLEMENT_METHOD_PRESERVE(_method, _api) \
 BEGIN_METHOD(_method, GB_BOOLEAN preserve) \
+  bool preserve = VARGOPT(preserve, FALSE); \
 	CHECK_DEVICE(); \
-	PAINT->_api(THIS, VARGOPT(preserve, FALSE)); \
+	PAINT->_api(THIS, preserve); \
+	if (!preserve) \
+		THIS->has_path = FALSE; \
 END_METHOD
 
 #define IMPLEMENT_PROPERTY_EXTENTS(_property, _api) \
@@ -603,7 +664,26 @@ END_METHOD
 
 IMPLEMENT_PROPERTY_BOOLEAN(Paint_Invert, Invert)
 IMPLEMENT_PROPERTY_INTEGER(Paint_FillStyle, FillStyle)
-IMPLEMENT_PROPERTY_UNSIGNED_INTEGER(Paint_Background, Background)
+
+BEGIN_PROPERTY(Paint_Background)
+
+	uint value;
+
+	CHECK_DEVICE();
+
+	if (READ_PROPERTY)
+	{
+		PAINT->Background(THIS, FALSE, &value);
+		GB.ReturnInteger(value);
+	}
+	else
+	{
+		value = (uint)VPROP(GB_INTEGER);
+		PAINT->Background(THIS, TRUE, &value);
+		GB.StoreObject(NULL, POINTER(&THIS->brush));
+	}
+
+END_METHOD
 
 IMPLEMENT_PROPERTY_BOOLEAN(Paint_Antialias, Antialias)
 IMPLEMENT_METHOD(Paint_Save, Save)
@@ -616,6 +696,37 @@ IMPLEMENT_METHOD_PRESERVE(Paint_Stroke, Stroke)
 IMPLEMENT_PROPERTY_EXTENTS(Paint_PathExtents, PathExtents)
 
 
+static GB_ARRAY _outline;
+static GB_ARRAY _outline_p;
+
+static void make_path_outline(int cmd, float x, float y)
+{
+	if (!_outline)
+		GB.Array.New(&_outline, GB.FindClass("PointF[]"), 0);
+
+	if (cmd == GB_PAINT_PATH_MOVE)
+	{
+		GB.Array.New(&_outline_p, GB.FindClass("PointF"), 0);
+		GB.Ref(_outline_p);
+		*((void **)GB.Array.Add(_outline)) = _outline_p;
+	}
+
+	GEOM_POINTF *p = GEOM.CreatePointF(x, y);
+	*((void **)GB.Array.Add(_outline_p)) = p;
+	GB.Ref(p);
+}
+
+BEGIN_PROPERTY(Paint_PathOutline)
+
+	CHECK_DEVICE();
+
+	_outline = NULL;
+	PAINT->PathOutline(THIS, make_path_outline);
+	GB.ReturnObject(_outline);
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(Paint_ClipRect)
 
 	GB_EXTENTS ext;
@@ -771,7 +882,16 @@ END_PROPERTY
 
 
 IMPLEMENT_PROPERTY_FLOAT(Paint_DashOffset, DashOffset)
-IMPLEMENT_METHOD(Paint_NewPath, NewPath)
+
+BEGIN_METHOD_VOID(Paint_NewPath)
+
+	CHECK_DEVICE();
+
+	PAINT->NewPath(THIS);
+	THIS->has_path = FALSE;
+
+END_METHOD
+
 IMPLEMENT_METHOD(Paint_ClosePath, ClosePath)
 
 
@@ -807,6 +927,7 @@ BEGIN_METHOD(Paint_Arc, GB_FLOAT xc; GB_FLOAT yc; GB_FLOAT radius; GB_FLOAT angl
 		pie = FALSE;
 	
 	PAINT->Arc(THIS, VARG(xc), VARG(yc), VARG(radius), angle, length, pie);
+	THIS->has_path = TRUE;
 
 END_METHOD
 
@@ -823,6 +944,7 @@ BEGIN_METHOD(Paint_Ellipse, GB_FLOAT x; GB_FLOAT y; GB_FLOAT width; GB_FLOAT hei
 		pie = FALSE;
 	
 	PAINT->Ellipse(THIS, VARG(x), VARG(y), VARG(width), VARG(height), angle, length, pie);
+	THIS->has_path = TRUE;
 
 END_METHOD
 
@@ -850,7 +972,8 @@ BEGIN_METHOD(Paint_Polygon, GB_OBJECT points)
 	for (i = 2; i < n; i+= 2)
 		PAINT->LineTo(THIS, p[i], p[i + 1]);
 	PAINT->LineTo(THIS, p[0], p[1]);
-	
+	THIS->has_path = TRUE;
+
 END_METHOD
 
 
@@ -858,6 +981,7 @@ BEGIN_METHOD(Paint_CurveTo, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y2;
 
 	CHECK_DEVICE();
 	PAINT->CurveTo(THIS, VARG(x1), VARG(y1), VARG(x2), VARG(y2), VARG(x3), VARG(y3));
+	THIS->has_path = TRUE;
 
 END_METHOD
 
@@ -868,6 +992,7 @@ BEGIN_METHOD(Paint_RelCurveTo, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y
 	CHECK_DEVICE();
 	PAINT->GetCurrentPoint(THIS, &x, &y);
 	PAINT->CurveTo(THIS, x + VARG(x1), y + VARG(y1), x + VARG(x2), y + VARG(y2), x + VARG(x3), y + VARG(y3));
+	THIS->has_path = TRUE;
 
 END_METHOD
 
@@ -876,6 +1001,7 @@ BEGIN_METHOD(Paint_LineTo, GB_FLOAT x; GB_FLOAT y)
 
 	CHECK_DEVICE();
 	PAINT->LineTo(THIS, VARG(x), VARG(y));
+	THIS->has_path = TRUE;
 
 END_METHOD
 
@@ -886,6 +1012,7 @@ BEGIN_METHOD(Paint_RelLineTo, GB_FLOAT x; GB_FLOAT y)
 	CHECK_DEVICE();
 	PAINT->GetCurrentPoint(THIS, &fx, &fy);
 	PAINT->LineTo(THIS, fx + VARG(x), fy + VARG(y));
+	THIS->has_path = TRUE;
 
 END_METHOD
 
@@ -938,6 +1065,8 @@ BEGIN_METHOD(Paint_Rectangle, GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB
 		PAINT->CurveTo(THIS, x, y + r2, x + r2, y, x + r, y);
 	}
 
+	THIS->has_path = TRUE;
+
 END_METHOD
 
 
@@ -952,7 +1081,8 @@ BEGIN_METHOD(Paint_Text, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_
 		PAINT->MoveTo(THIS, (float)VARG(x), (float)VARG(y));
 	
 	PAINT->Text(THIS, STRING(text), LENGTH(text), VARGOPT(w, -1), VARGOPT(h, -1), VARGOPT(align, GB_DRAW_ALIGN_DEFAULT), FALSE);
-	
+	THIS->has_path = TRUE;
+
 END_METHOD
 
 
@@ -964,14 +1094,16 @@ BEGIN_METHOD(Paint_RichText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w;
 		PAINT->MoveTo(THIS, (float)VARG(x), (float)VARG(y));
 	
 	PAINT->RichText(THIS, STRING(text), LENGTH(text), VARGOPT(w, -1), VARGOPT(h, -1), VARGOPT(align, GB_DRAW_ALIGN_DEFAULT), FALSE);
-	
+	THIS->has_path = TRUE;
+
 END_METHOD
 
 
 BEGIN_METHOD(Paint_DrawText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER align)
 
 	CHECK_DEVICE();
-	
+	CHECK_PATH();
+
 	if (!MISSING(x) && !MISSING(y))
 		PAINT->MoveTo(THIS, (float)VARG(x), (float)VARG(y));
 	
@@ -983,7 +1115,8 @@ END_METHOD
 BEGIN_METHOD(Paint_DrawRichText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER align)
 
 	CHECK_DEVICE();
-	
+	CHECK_PATH();
+
 	if (!MISSING(x) && !MISSING(y))
 		PAINT->MoveTo(THIS, (float)VARG(x), (float)VARG(y));
 	
@@ -1186,20 +1319,15 @@ BEGIN_METHOD_VOID(Paint_Reset)
 
 	CHECK_DEVICE();
 	PAINT->Matrix(THIS, TRUE, NULL);
+	PAINT_translate(THIS->area.x, THIS->area.y);
 
 END_METHOD
 
 
 BEGIN_METHOD(Paint_Translate, GB_FLOAT tx; GB_FLOAT ty)
 
-	GB_TRANSFORM transform;
-
 	CHECK_DEVICE();
-	MPAINT->Create(&transform);
-	PAINT->Matrix(THIS, FALSE, transform);
-	MPAINT->Translate(transform, (float)VARG(tx), (float)VARG(ty));
-	PAINT->Matrix(THIS, TRUE, transform);
-	MPAINT->Delete(&transform);
+	PAINT_translate(VARG(tx), VARG(ty));
 
 END_METHOD
 
@@ -1240,7 +1368,8 @@ BEGIN_METHOD(Paint_DrawImage, GB_OBJECT image; GB_FLOAT x; GB_FLOAT y; GB_FLOAT
 	GEOM_RECT *source = (GEOM_RECT *)VARGOPT(source, NULL);
 
 	CHECK_DEVICE();
-	
+	CHECK_PATH();
+
 	image = (GB_IMG *)VARG(image);
 	
 	if (GB.CheckObject(image))
@@ -1276,6 +1405,7 @@ BEGIN_METHOD(Paint_DrawPicture, GB_OBJECT picture; GB_FLOAT x; GB_FLOAT y; GB_FL
 	float x, y, w, h;
 
 	CHECK_DEVICE();
+	CHECK_PATH();
 
 	if (GB.CheckObject(picture))
 		return;
@@ -1313,6 +1443,7 @@ BEGIN_METHOD(Paint_ZoomImage, GB_OBJECT image; GB_INTEGER zoom; GB_INTEGER x; GB
 	float opacity = 1.0; //VARGOPT(opacity, 1.0);
 
 	CHECK_DEVICE();
+	CHECK_PATH();
 
 	if (GB.CheckObject(image))
 		return;
@@ -1430,6 +1561,7 @@ END_PROPERTY
 BEGIN_METHOD(Paint_FillRect, GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER color)
 
 	CHECK_DEVICE();
+	CHECK_PATH();
 	PAINT->FillRect(THIS, VARG(x), VARG(y), VARG(w), VARG(h), VARG(color));
 	
 END_METHOD
@@ -1446,6 +1578,45 @@ BEGIN_PROPERTY(Paint_Tag)
 
 END_PROPERTY
 
+#if 0
+Static Public Sub DrawEllipsizedText(Text As String, X As Integer, Y As Integer, W As Integer, H As Integer, Optional Alignment As Integer = Align.TopLeft)
+
+  Dim aElt As String[] = Split(Text, " \n")
+  Dim sElt As String
+  Dim I As Integer
+  Dim HL As Integer = Paint.Font.Height
+  Dim YT, WT As Integer
+  Dim sText As String
+
+  YT = 0
+
+  For I = 0 To aElt.Max
+    sElt = aElt[I]
+
+    If (YT + HL * 2) > H Then
+      WT = Paint.Font.TextWidth(LTrim(sText & " " & sElt & "…"))
+      If WT > W Then
+        Paint.DrawText(sText & "…", X, Y + YT, W, H, Align.TopLeft)
+        Return
+      Endif
+    Else
+      WT = Paint.Font.TextWidth(LTrim(sText & " " & sElt))
+      If WT > W Then
+        Paint.DrawText(sText, X, Y + YT, W, H, Align.TopLeft)
+        sText = ""
+        YT += HL
+      Endif
+    Endif
+
+    sText = RTrim(sText & " " & sElt)
+
+  Next
+
+  Paint.DrawText(sText, X, Y + YT, W, H, Align.TopLeft)
+
+End
+#endif
+
 //-- Style API --------------------------------------------------------------
 
 #define GET_COORD() \
@@ -1498,7 +1669,7 @@ GB_DESC PaintDesc[] =
 	GB_CONSTANT("OperatorAdd", "i",      GB_PAINT_OPERATOR_ADD),
 	GB_CONSTANT("OperatorSaturate", "i", GB_PAINT_OPERATOR_SATURATE),
 
-	GB_STATIC_METHOD("Begin", NULL, Paint_Begin, "(Device)o"),
+	GB_STATIC_METHOD("Begin", NULL, Paint_Begin, "(Device)o[(Area)RectF;]"),
 	GB_STATIC_METHOD("End", NULL, Paint_End, NULL),
 	
 	GB_STATIC_PROPERTY_READ("Device", "o", Paint_Device),
@@ -1537,7 +1708,8 @@ GB_DESC PaintDesc[] =
 
 	GB_STATIC_PROPERTY_READ("PathExtents", "PaintExtents", Paint_PathExtents),
 	GB_STATIC_METHOD("PathContains", "b", Paint_PathContains, "(X)f(Y)f"),
-	
+	GB_STATIC_PROPERTY_READ("PathOutline", "PointF[][]", Paint_PathOutline),
+
 	GB_STATIC_PROPERTY("Brush", "PaintBrush", Paint_Brush),
 	GB_STATIC_PROPERTY("BrushOrigin", "PointF", Paint_BrushOrigin),
 	GB_STATIC_PROPERTY("Dash", "Float[]", Paint_Dash),
@@ -1570,12 +1742,12 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_METHOD("MoveTo", NULL, Paint_MoveTo, "(X)f(Y)f"),
 	GB_STATIC_METHOD("RelMoveTo", NULL, Paint_RelMoveTo, "(X)f(Y)f"),
 	GB_STATIC_PROPERTY("Font", "Font", Paint_Font),
-	GB_STATIC_METHOD("Text", NULL, Paint_Text, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
+	GB_STATIC_METHOD("Text", NULL, Paint_Text, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i]"),
 	GB_STATIC_METHOD("TextSize", "RectF", Paint_TextSize, "(Text)s"),
-	GB_STATIC_METHOD("RichText", NULL, Paint_RichText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
+	GB_STATIC_METHOD("RichText", NULL, Paint_RichText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i]"),
 	GB_STATIC_METHOD("RichTextSize", "RectF", Paint_RichTextSize, "(Text)s[(Width)f]"),
-	GB_STATIC_METHOD("DrawText", NULL, Paint_DrawText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
-	GB_STATIC_METHOD("DrawRichText", NULL, Paint_DrawRichText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
+	GB_STATIC_METHOD("DrawText", NULL, Paint_DrawText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i]"),
+	GB_STATIC_METHOD("DrawRichText", NULL, Paint_DrawRichText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i]"),
 	GB_STATIC_METHOD("TextExtents", "PaintExtents", Paint_TextExtents, "(Text)s"),
 	GB_STATIC_METHOD("RichTextExtents", "PaintExtents", Paint_RichTextExtents, "(Text)s[(Width)f]"),
 	
@@ -1597,6 +1769,7 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_METHOD("DrawPicture", NULL, Paint_DrawPicture, "(Picture)Picture;(X)f(Y)f[(Width)f(Height)f(Source)Rect;]"),
 	GB_STATIC_METHOD("ZoomImage", NULL, Paint_ZoomImage, "(Image)Image;(Zoom)i(X)i(Y)i[(Grid)i(Source)Rect;]"),
 
+
 	GB_END_DECLARE
 };
 
