@@ -533,7 +533,7 @@ GB_DESC CColorInfoDesc[] =
 
 GB_DESC CColorDesc[] =
 {
-  GB_DECLARE("Color", 0), GB_NOT_CREATABLE(),
+  GB_DECLARE_STATIC("Color"),
 
   GB_CONSTANT("Default", "i", COLOR_DEFAULT),
 
@@ -566,7 +566,7 @@ GB_DESC CColorDesc[] =
   GB_CONSTANT("Violet", "i", 0x8000FF),
   GB_CONSTANT("Pink", "i", 0xFF80FF),
 
-  GB_CONSTANT("Transparent", "i", 0xFFFFFFFF),
+  GB_CONSTANT("Transparent", "i", 0xFF000000),
 
   GB_STATIC_METHOD("RGB", "i", Color_RGB, "(Red)i(Green)i(Blue)i[(Alpha)i]"),
   GB_STATIC_METHOD("HSV", "i", Color_HSV, "(Hue)i(Saturation)i(Value)i[(Alpha)i]"),
